<?php

if (!defined('ABSPATH')) {
    exit();
}

class WAMOptions implements WAMConstants {

    private $dbManager;
    private $defaultExpireTime;
    public $wamIsDisplayOnReplies;
    public $wamIsCachingDisabled;
    public $wamCacheExpireTime;

    public function __construct($dbManager) {
        $this->dbManager = $dbManager;
        $options = maybe_unserialize(get_option(self::OPTION_MAIN_OPTIONS));
        $this->defaultExpireTime = 60 * 60 * 12; // seconds * minutes * hours
        $this->addOptions();
        $this->initOptions($options);
    }

    public function addOptions() {
        $options = array(
            'wamIsDisplayOnReplies' => 0,
            'wamIsCachingDisabled' => 0,
            'wamCacheExpireTime' => $this->defaultExpireTime
        );
        add_option(self::OPTION_MAIN_OPTIONS, $options, '', 'no');
    }

    public function initOptions($options) {
        $options = maybe_unserialize($options);
        $this->wamIsDisplayOnReplies = isset($options['wamIsDisplayOnReplies']) ? $options['wamIsDisplayOnReplies'] : 0;
        $this->wamIsCachingDisabled = isset($options['wamIsCachingDisabled']) ? $options['wamIsCachingDisabled'] : 0;
        $this->wamCacheExpireTime = isset($options['wamCacheExpireTime']) ? $options['wamCacheExpireTime'] : $this->defaultExpireTime;
    }

    public function toArray() {
        $options = array(
            'wamIsDisplayOnReplies' => $this->wamIsDisplayOnReplies,
            'wamIsCachingDisabled' => $this->wamIsCachingDisabled,
            'wamCacheExpireTime' => $this->wamCacheExpireTime,
        );
        return $options;
    }

    public function tabTitle() {
        echo '<li>' . __('Ads Manager', 'wpdiscuz-ads-manager') . '</li>';
    }

    public function tabContent() {
        include_once 'settings.php';
    }

    public function saveOptions() {
        $this->wamIsDisplayOnReplies = isset($_POST['wamIsDisplayOnReplies']) ? $_POST['wamIsDisplayOnReplies'] : 0;
        $this->wamIsCachingDisabled = isset($_POST['wamIsCachingDisabled']) ? $_POST['wamIsCachingDisabled'] : 0;
        $this->wamCacheExpireTime = isset($_POST['wamCacheExpireTime']) ? absint($_POST['wamCacheExpireTime']) * HOUR_IN_SECONDS : $this->defaultExpireTime;
        update_option(self::OPTION_MAIN_OPTIONS, $this->toArray());
    }

    public function resetCache() {
        $validNonce = isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'resetCache');
        if ($validNonce && isset($_GET['cache']) && ($key = filter_var($_GET['cache'], FILTER_SANITIZE_STRING)) && current_user_can('manage_options')) {
            delete_transient($key);
        }
        wp_redirect(admin_url('edit-comments.php?page=' . WpdiscuzCore::PAGE_SETTINGS));
    }

}
