<?php

if (!defined('ABSPATH')) {
    exit();
}
/*
 * Plugin Name: wpDiscuz - Advanced Liking
 * Description: See comment likers and voters of each comment. Adds user reputation and badges based on received likes.
 * Version: 1.2.0
 * Author: gVectors Team (A. Chakhoyan, G. Zakaryan, H. Martirosyan)
 * Author URI: https://gvectors.com/
 * Plugin URI: https://gvectors.com/product/wpdiscuz-advanced-liking/
 * Text Domain: wpdiscuz_likers
 * Domain Path: /languages/
 */

define('WV_DIR_PATH', dirname(__FILE__));
define('WV_DIR_NAME', basename(WV_DIR_PATH));
include_once "manager/class.WVDBManager.php";
include_once "settings/class.WVSettings.php";
include_once 'includes/gvt-api-manager.php';

class WpdiscuzVoters {

    public $comment_id;
    public $dbmanager;
    public $settings;
    private $version;
    private $versionName = "wv_version";
    public $voteTypeUpIcon;
    public $voteTypeDownIcon;

    public function __construct() {
        add_action('plugins_loaded', array(& $this, 'pluginsLoaded'), 11);
    }

    public function pluginsLoaded() {
        if (function_exists('wpDiscuz')) {
            $this->version = get_option($this->versionName);
            if (!isset($this->version)) {
                $this->version = '1.0.0';
            }
            new GVT_API_Manager(__FILE__, 'wpdiscuz_options_page', 'wpdiscuz_option_page');
            $this->settings = new WVSettings();
            $this->dbmanager = new WVDBManager();
            load_plugin_textdomain('wpdiscuz_likers', false, WV_DIR_NAME . '/languages/');
            add_action('wp_ajax_nopriv_viewAll', array(&$this, 'viewAllAjax'), 11);
            add_action('wp_ajax_nopriv_voters', array(&$this, 'votersAjax'), 11);
            add_action('wpdiscuz_check_version', array(&$this, 'pluginNewVersion'), 11);
            add_filter('wpdiscuz_comment_vote', array(&$this, 'voteStyling'), 11);
            add_filter('wpdiscuz_vote_up_icon_classes', array(&$this, 'wvUp'), 11, 3);
            add_filter('wpdiscuz_vote_down_icon_classes', array(&$this, 'wvDown'), 11, 3);
            add_action('wpdiscuz_front_scripts', array(&$this, 'registerScripts'), 11);
            add_action('wp_ajax_viewAll', array(&$this, 'viewAllAjax'), 11);
            add_action('wp_ajax_voters', array(&$this, 'votersAjax'), 11);
            add_action('wp_footer', array(&$this, 'concatDiv'), 11);
            add_action('wpdiscuz_after_label', array(&$this, 'label'), 11, 2);
            add_action('wpdiscuz_update_vote', array(&$this, 'updateRating'), 11, 3);
            add_action('wpdiscuz_add_vote', array(&$this, 'addRating'), 11, 2);
            add_action('wpdiscuz_js_options', array(&$this, 'wvJsOptions'), 11, 2);
        } else {
            add_action('admin_notices', array(&$this, 'wvRequirements'), 1);
        }
    }

    public function wvRequirements() {
        if (current_user_can('manage_options')) {
            echo "<div class='error'><p>" . __('wpDiscuz Advanced Liking requires wpDiscuz to be installed!', 'wpdiscuz_likers') . "</p></div>";
        }
    }

    public function wvJsOptions($jsOptions, $wpdiscuzOptions) {
        $jsOptions['wvRealTime'] = $this->settings->wvOptions['wv_real_time'];
        return $jsOptions;
    }

    public function updateRating($voteType, $isUserVoted, $comment) {
        $this->addRating($voteType, $comment);
    }

    public function addRating($voteType, $comment) {
        if ($comment->user_id) {
            $rating = get_user_meta($comment->user_id, 'wv_rating', true);
            update_user_meta($comment->user_id, "wv_rating", $rating + $voteType);
        }
    }

    public function label($afterLabelHtml, $comment) {
        if (0 == $comment->user_id) {
            return $afterLabelHtml;
        }
        $user_rating = intval(get_user_meta($comment->user_id, 'wv_rating', true));
        $level = $this->settings->wvOptions["wv_level"];
        if ($user_rating < $level[1]["vote"]["value"]) {
            return $afterLabelHtml;
        }
        $icon = '';
        $title = '';
        for ($i = count($level); $i > 0; $i--) {
            if ($user_rating >= intval($level[$i]["vote"]["value"])) {
                if ($level[$i]["icon"]["enable"]) {
                    $icon = "<i class='" . $level[$i]['icon']['value'] . "' style='color:" . $level[$i]['icon']['color'] . "'></i>";
                }
                if ($level[$i]["label"]["enable"]) {
                    $title = $level[$i]["label"]["value"];
                }
                break;
            }
        }
        $afterLabelHtml .= $title || $icon ? '<div class="wv_level_box"><div class="wv-badge">' . $icon . '</div><div class="wv-title">' . $title . '</div></div>' : '';
        return $afterLabelHtml;
    }

    public function pluginNewVersion() {
        $pluginData = get_plugin_data(__FILE__);
        if (version_compare($pluginData['Version'], $this->version, '>')) {
            update_option('wv_plugin_version', $pluginData['Version']);
        } else {
            update_option('wv_plugin_version', $this->version);
        }
    }

    public function voteStyling($args) {
        $args['callbackFunctions'][] = "voteRealTime";
        return $args;
    }

    public function wvUp($classes, $comment, $currentUser) {
        $voteType = !empty($currentUser->ID) ? $this->dbmanager->getCurentUserVote($currentUser->ID, $comment->comment_ID, 1) : 0;
        if ($voteType == '1') {
            $classes[] = 'wv-up-color';
        }
        return $classes;
    }

    public function wvDown($classes, $comment, $currentUser) {
        $voteType = !empty($currentUser->ID) ? $this->dbmanager->getCurentUserVote($currentUser->ID, $comment->comment_ID, -1) : 0;
        if ($voteType == '-1') {
            $classes[] = 'wv-down-color';
        }
        return $classes;
    }

    public function registerScripts($optionsSerialized) {
        wp_register_style('wpdiscuz-voters-style', plugins_url(WV_DIR_NAME . '/assets/css/wv.css'));
        wp_enqueue_style('wpdiscuz-voters-style');
        if (is_rtl()) {
            if ($optionsSerialized->votingButtonsStyle) {
                wp_register_style('wpdiscuz-voters-style-rtl', plugins_url(WV_DIR_NAME . '/assets/css/wv-rtl-seperate.css'));
                wp_enqueue_style('wpdiscuz-voters-style-rtl');
            } else {
                wp_register_style('wpdiscuz-voters-style-rtl', plugins_url(WV_DIR_NAME . '/assets/css/wv-rtl.css'));
                wp_enqueue_style('wpdiscuz-voters-style-rtl');
            }
        }
        wp_register_script('wpdiscuz-voters-script', plugins_url('assets/js/wv.js', __FILE__), array('jquery'), $this->version, $optionsSerialized->isLoadScriptsInFooter);
        wp_enqueue_script('wpdiscuz-voters-script');
    }

    public function votersAjax() {
        $html = "";
        if (isset($_POST['id']) && ($id_comment = intval($_POST['id'])) && !empty($_POST['hovered']) && ($typeVote = trim($_POST['hovered']))) {
            $wpdiscuz = wpDiscuz();
            $voteIconType = $wpdiscuz->optionsSerialized->votingButtonsIcon ? explode('|', $wpdiscuz->optionsSerialized->votingButtonsIcon) : array('fa-plus', 'fa-minus');
            $count = $this->settings->wvOptions['wv_count'];
            $users_id = $this->dbmanager->getVotes($id_comment, $typeVote, $count);
            $getAvatar = $this->settings->wvOptions['wv_get_avatars'];
            foreach ($users_id as $userInfo) {
                $id = intval($userInfo['user_id']);
                $user = get_userdata($id);
                $username = $wpdiscuz->helper->getCurrentUserDisplayName($user);
                $userPageLinkHTML = "<a href='" . esc_url(get_author_posts_url($id)) . "'><span class='wv-username'>" . esc_html($username) . "</span></a>";
                $avatar = $getAvatar == 'yes' ? get_avatar($id) : '';
                if ($userInfo['vote_type'] == 1) {
                    $html .= "<div class='wv-user-info'>" . $avatar . "<span class='wv-plus'><i class='fas " . $voteIconType[0] . " wv-up-color'></i></span> " . $userPageLinkHTML . "</div>";
                } else {
                    $html .= "<div class='wv-user-info'>" . $avatar . "<span class='wv-minus'><i class='fas " . $voteIconType[1] . " wv-down-color'></i></span> " . $userPageLinkHTML . "</div>";
                }
            }
            $count_plus = '';
            $count_minus = '';
            if ($typeVote == 'like') {
                $count_plus = $this->dbmanager->getGuestVotesCount($id_comment, 1);
            } elseif ($typeVote == 'dislike') {
                $count_minus = $this->dbmanager->getGuestVotesCount($id_comment, -1);
            } else {
                $count_plus = $this->dbmanager->getGuestVotesCount($id_comment, 1);
                $count_minus = $this->dbmanager->getGuestVotesCount($id_comment, -1);
            }
            $phraseGuests = esc_html($this->settings->wvPhrases['wv_guests']);
            $guest_avatar = $getAvatar == 'yes' ? get_avatar(0) : '';
            if ($count_plus) {
                $html .= "<div class='wv-user-info'>" . $guest_avatar . "<span class='wv-plus'>" . esc_html($count_plus) . " <i class='fas " . $voteIconType[0] . " wv-up-color'></i></span> " . $phraseGuests . "</div>";
            }
            if ($count_minus) {
                $html .= "<div class='wv-user-info'>" . $guest_avatar . "<span class='wv-minus'>" . esc_html($count_minus) . " <i class='fas " . $voteIconType[1] . " wv-down-color'></i></span> " . $phraseGuests . "</div>";
            }
            if ($html) {
                if ('yes' == $this->settings->wvOptions['wv_read_more']) {
                    $html .= "<span class='wv-view-all-button-" . $typeVote . " wv-view-all-button'>" . esc_html($this->settings->wvPhrases['wv_view_all']) . "</span>";
                }
                $output = '<div class="wv-body" data-type="' . $typeVote . '">';
                $output .= "<div class='wv-head vote-head'>";
                $output .= "<span class='wv-arrow vote-arrow'></span><span class='wv-arrow-no-border vote-arrow-no-border'></span>";
                $output .= "<div class='wv-vote-content'>";
                $output .= $html;
                $output .= '</div>';
                $output .= '</div>';
                $output .= '</div>';
                $html = $output;
            }
        }
        wp_die($html);
    }

    public function viewAllAjax() {
        $html = "";
        if (isset($_POST['id']) && ($id = intval($_POST['id'])) && !empty($_POST['hovered']) && ($hoverType = trim($_POST['hovered']))) {
            $page = !empty($_POST['page']) ? intval($_POST['page']) : 1;
            $limit = $this->settings->wvOptions['wv_count_all'];
            $offset = ($page - 1) * $limit;
            $count = $this->dbmanager->getVotesCount($id, $hoverType);
            $loadMore = '';
            $getAvatar = $this->settings->wvOptions['wv_read_more_get_avatar'];
            $na = $getAvatar == 'no' ? ' wv-no-avatar' : '';
            $wpdiscuz = wpDiscuz();
            $voteIconType = $wpdiscuz->optionsSerialized->votingButtonsIcon ? explode('|', $wpdiscuz->optionsSerialized->votingButtonsIcon) : array('fa-plus', 'fa-minus');
            if ($offset < $count) {
                $users_ids = $this->dbmanager->getVotes($id, $hoverType, $limit, $offset);
                foreach ($users_ids as $userInfo) {
                    $user_id = intval($userInfo['user_id']);
                    $userData = get_userdata($user_id);
                    $username = $wpdiscuz->helper->getCurrentUserDisplayName($userData);
                    $userPageLinkHTML = "<a href='" . esc_url(get_author_posts_url($user_id)) . "'><span class='wv-username'>" . esc_html($username) . "</span></a>";
                    $avatar = $getAvatar == 'yes' ? get_avatar($user_id, 64) : '';
                    if ($userInfo['vote_type'] == 1) {
                        $html .= "<div class='wv-user-info-all" . $na . "'>" . $avatar . "<span class='wv-plus'><i class='fas " . $voteIconType[0] . " wv-up-color'></i></span> " . $userPageLinkHTML . "</div>";
                    } else {
                        $html .= "<div class='wv-user-info-all" . $na . "'>" . $avatar . "<span class='wv-minus'><i class='fas " . $voteIconType[1] . " wv-down-color'></i></span> " . $userPageLinkHTML . "</div>";
                    }
                }
                if ($offset + count($users_ids) < $count) {
                    $loadMore .= "<input class='wv_hidden_page' type='hidden' name='wv_page' value='" . ($page + 1) . "' />";
                    $loadMore .= "<input class='wv_hidde_type' type='hidden' name='wv_type' value='$hoverType' />";
                    $loadMore .= "<input class='wv_hidden_id' type='hidden' name='wv_id' value='$id' />";
                    $loadMore .= "<span class='wv-read_more'>" . esc_html($this->settings->wvPhrases['wv_view_all']) . "</span>";
                }
            }
            if ($page == 1) {
                $count_plus = '';
                $count_minus = '';
                if ($hoverType == 'like') {
                    $count_plus = $this->dbmanager->getGuestVotesCount($id, 1);
                } elseif ($hoverType == 'dislike') {
                    $count_minus = $this->dbmanager->getGuestVotesCount($id, -1);
                } else {
                    $count_plus = $this->dbmanager->getGuestVotesCount($id, 1);
                    $count_minus = $this->dbmanager->getGuestVotesCount($id, -1);
                }
                $phraseGuests = esc_html($this->settings->wvPhrases['wv_guests']);
                $guest_avatar = $getAvatar == 'yes' ? get_avatar(0, 64) : '';
                if ($count_plus) {
                    $html .= "<div class='wv-user-info-all" . $na . "'>" . $guest_avatar . "<span class='wv-plus'><i class='fas " . $voteIconType[0] . " wv-up-color'></i> " . esc_html($count_plus) . "</span> " . $phraseGuests . "</div>";
                }
                if ($count_minus) {
                    $html .= "<div class='wv-user-info-all" . $na . "'>" . $guest_avatar . "<span class='wv-minus'><i class='fas " . $voteIconType[1] . " wv-down-color'></i> " . esc_html($count_minus) . "</span> " . $phraseGuests . "</div>";
                }
            }
            $html .= $loadMore;
        }
        wp_die($html);
    }

    public function concatDiv() {
        global $post;
        $wpdiscuz = wpDiscuz();
        if ($wpdiscuz->helper->isLoadWpdiscuz($post)) {
            echo "<div class='wv-view-all'></div><div class='wv-all-html'></div>";
        }
    }

}

$wpDiscuzVoters = new WpdiscuzVoters();
?>