<?php

if (!defined('ABSPATH')) {
    exit();
}

class WVSettings {

    private $settingsOptionName = 'wv_settings';
    private $phrasesOptionName = 'wv_phrases';
    public $wvOptions;
    public $wvPhrases;

    public function __construct() {
        add_action('wpdiscuz_addon_tab_title', array(&$this, 'wvOptionTabName'), 11);
        add_action('wpdiscuz_save_options', array(&$this, 'wvSaveOptions'), 11);
        add_action('wpdiscuz_dynamic_css', array(&$this, 'stylingVoters'), 11);
        add_action('admin_enqueue_scripts', array(&$this, 'settingScripts'), 11);
        add_action('wpdiscuz_addon_tab_content', array(&$this, 'resetOptions'), 11);
        $this->addOptionsAndPhrases();
        $this->initOptions(get_option($this->settingsOptionName));
        $this->initPhrases(get_option($this->phrasesOptionName));
    }

    public function settingScripts() {
        if (isset($_GET['page']) && ($_GET['page'] == WpdiscuzCore::PAGE_SETTINGS)) {
            wp_register_script('pluginScript', plugins_url(WV_DIR_NAME . '/assets/js/wv-settings.js'), array('jquery'), '1.0.0', false);
            wp_enqueue_script('pluginScript');
            wp_register_style('wvsettingsstyle', plugins_url(WV_DIR_NAME . '/assets/css/wv-settings.css'));
            wp_enqueue_style('wvsettingsstyle');
            wp_enqueue_style('wv-iconpicker-css', plugins_url(WV_DIR_NAME . '/assets/third-party/iconpicker/fontawesome-iconpicker.min.css'));
            wp_register_script('wv-iconpicker-js', plugins_url(WV_DIR_NAME . '/assets/third-party/iconpicker/fontawesome-iconpicker.js'), array('jquery'));
            wp_enqueue_script('wv-iconpicker-js');
        }
    }

    public function wvOptionTabName() {
        echo '<li>' . __('Advanced Likers', 'wpdiscuz_likers') . '</li>';
    }

    public function resetOptions() {
        if (isset($_GET['wv_reset_options']) && current_user_can('manage_options') && 1 == $_GET['wv_reset_options'] && isset($_GET['_wv_nonce']) && wp_verify_nonce($_GET['_wv_nonce'], 'wv_nonce')) {
            $this->initOptions($this->defaultOptions());
            $this->initPhrases($this->defaultPhrases());
        }
        include 'settings.php';
    }

    public function addOptionsAndPhrases() {
        add_option($this->settingsOptionName, $this->defaultOptions());
        add_option($this->phrasesOptionName, $this->defaultPhrases());
    }

    public function defaultOptions() {
        $options = array(
            'wv_get_avatars' => 'yes',
            'wv_read_more' => 'yes',
            'wv_read_more_get_avatar' => 'yes',
            'wv_real_time' => 'yes',
            'wv_background_color' => '#FAFAFA',
            'wv_border_color' => '#AAAAAA',
            'wv_up_color' => '#00bc74',
            'wv_down_color' => '#ff5959',
            'wv_count' => '8',
            'wv_count_all' => '8',
            "wv_level" => array(
                1 => array(
                    'vote' => array("value" => 1),
                    'icon' => array(
                        "value" => "fas fa-user",
                        "enable" => 1,
                        "color" => "#0CD85D"
                    ),
                    'label' => array(
                        "value" => __('Member', 'wpdiscuz_likers'),
                        "enable" => 1
                    ),
                ),
                2 => array(
                    'vote' => array("value" => 10),
                    'icon' => array(
                        "value" => "fas fa-star",
                        "enable" => 1,
                        "color" => "#E5D600"
                    ),
                    'label' => array(
                        "value" => __('Active Member', 'wpdiscuz_likers'),
                        "enable" => 1
                    ),
                ),
                3 => array(
                    'vote' => array("value" => 50),
                    'icon' => array(
                        "value" => "fas fa-certificate",
                        "enable" => 1,
                        "color" => "#FF812D"
                    ),
                    'label' => array(
                        "value" => __('Trusted Member', 'wpdiscuz_likers'),
                        "enable" => 1,
                    ),
                ),
                4 => array(
                    'vote' => array("value" => 100),
                    'icon' => array(
                        "value" => "fas fa-shield-alt",
                        "enable" => 1,
                        "color" => "#43A6DF"
                    ),
                    'label' => array(
                        "value" => __('Noble Member', 'wpdiscuz_likers'),
                        "enable" => 1
                    ),
                ),
                5 => array(
                    'vote' => array("value" => 500),
                    'icon' => array(
                        "value" => "fas fa-trophy",
                        "enable" => 1,
                        "color" => "#E04A47"
                    ),
                    'label' => array(
                        "value" => __('Famed Member', 'wpdiscuz_likers'),
                        "enable" => 1
                    ),
                ))
        );
        return $options;
    }

    public function defaultPhrases() {
        $phrases = array(
            'wv_guests' => __('Guests', 'wpdiscuz_likers'),
            'wv_view_all' => __('View all', 'wpdiscuz_likers'),
        );
        return $phrases;
    }

    public function initOptions($serialize_options) {
        $options = maybe_unserialize($serialize_options);
        if ($options['wv_level'][1]['vote']['value'] != 0 && 
                $options['wv_level'][1]['vote']['value'] < $options['wv_level'][2]['vote']['value'] &&
                $options['wv_level'][2]['vote']['value'] < $options['wv_level'][3]['vote']['value'] &&
                $options['wv_level'][3]['vote']['value'] < $options['wv_level'][4]['vote']['value'] &&
                $options['wv_level'][4]['vote']['value'] < $options['wv_level'][5]['vote']['value']) {
            
        } else {
            $options['wv_level'][1]['vote']['value'] = 1;
            $options['wv_level'][2]['vote']['value'] = 10;
            $options['wv_level'][3]['vote']['value'] = 50;
            $options['wv_level'][4]['vote']['value'] = 100;
            $options['wv_level'][5]['vote']['value'] = 500;
        }
        for ($i = 1; $i < 6; $i++) {
            $options['wv_level'][$i]['label']['enable'] = isset($options['wv_level'][$i]['label']['enable']) ? absint($options['wv_level'][$i]['label']['enable']) : 0;
            $options['wv_level'][$i]['icon']['enable'] = isset($options['wv_level'][$i]['icon']['enable']) ? absint($options['wv_level'][$i]['icon']['enable']) : 0;
        }
        $default = $this->defaultOptions();
        foreach ($default as $key => $value) {
            $this->wvOptions[$key] = isset($options[$key]) ? $options[$key] : $value;
        }
        update_option($this->settingsOptionName, $this->wvOptions);
    }

    public function initPhrases($serialize_phrases) {
        $phrases = maybe_unserialize($serialize_phrases);
        $default = $this->defaultPhrases();
        foreach ($default as $key => $value) {
            $this->wvPhrases[$key] = isset($phrases[$key]) ? wp_unslash($phrases[$key]) : wp_unslash($value);
        }
        update_option($this->phrasesOptionName, $this->wvPhrases);
    }

    public function wvSaveOptions($post) {
        $this->initOptions($post);
        $this->initPhrases($post);
    }

    public function stylingVoters($optionsSerialized) {
        $css = "#wpcomm .vote-head, .vote-head-right,.wv-vote-content{background-color:" . $this->wvOptions['wv_background_color'] . ";border-color:" . $this->wvOptions['wv_border_color'] . "!important; color: #666666; font-size:13px;}";
        $css .= "#wpcomm .wv-head .vote-arrow,#wpcomm .wv-head-right .vote-arrow{border-bottom:10px solid " . $this->wvOptions['wv_border_color'] . ";}";
        $css .= "#wpcomm .wv-head .vote-arrow-no-border,#wpcomm .wv-head-right .vote-arrow-no-border{border-bottom-color:" . $this->wvOptions['wv_background_color'] . ";}";
        $css .= ".vote-head a:hover,.wv-vote-content a:hover,.wv-view-all-button:hover{color:" . $optionsSerialized->primaryColor . " !important}";
        $css .= ".wv-up-color,.wc-up i:hover{color:" . $this->wvOptions['wv_up_color'] . " !important}";
        $css .= ".wv-down-color,.wc-down i:hover{color:" . $this->wvOptions['wv_down_color'] . " !important}";
        echo $css;
    }

}
