<?php

if (!defined('ABSPATH')) {
    exit();
}

class WCAIHelper implements WCAIConstants {

    private $options;
    private $dbManager;
    private $currentUser;

    public function __construct($options, $dbManager) {
        $this->options = $options;
        $this->dbManager = $dbManager;
    }

    public function addInfoIcon($output, $comment, $user, $currentUser) {
        if ($this->options->sections && ($this->isAllowedRole($currentUser) || (!WpdiscuzCommentAuthorInfo::$IS_LOGGED_IN && $this->options->showForGuests))) {
            $this->currentUser = $currentUser;
            $output .= "<i id='wcai-comment_{$comment->comment_ID}' class='fas fa-info wpf-cta wcai-info wcai-not-clicked'></i>";
        }
        return $output;
    }

    public function wcaiResetOptions() {
        if (isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'wcaiResetOptions') && current_user_can('manage_options')) {
            delete_option(self::OPTION_MAIN_OPTIONS);
            $this->options->addOptions();
            $this->options->initOptions(get_option(self::OPTION_MAIN_OPTIONS));
        }
        wp_redirect(admin_url('edit-comments.php?page=' . WpdiscuzCore::PAGE_SETTINGS));
    }

    public function addLityPopup() {
        $wpdiscuz = wpDiscuz();
        $html = "<a id='wcaiInfoAnchor' style='display:none;' rel='#wcaiInfo' data-wcai-lity>Comment Author Info</a>";
        $html .= "<div id='wcaiInfo' style='overflow:auto;background:#FDFDF6;padding:20px;width:600px;max-width:100%;border-radius:6px' class='lity-hide'></div>";
        $html .= "<div id='wcaiInfoShort' class='wcaiInfoShort'></div>";
        $html .= "<style>";
        $html .= ".wcaiInfoShort .wcai-wrapper .wcai-full-info a.wcai-finfo{background-color:{$wpdiscuz->optionsSerialized->primaryColor};}";
        $html .= ".wcai-wrapper .wcai-list-item.wcai-active{border-top: 3px solid {$wpdiscuz->optionsSerialized->primaryColor};}";
        $html .= "</style>";
        echo $html;
    }

    public function wcaiGetInfo() {
        $wpdiscuz = wpDiscuz();
        $response = '';
        if ($_POST && isset($_POST['wcaiAjaxData'])) {
            parse_str($_POST['wcaiAjaxData'], $data);
            if (isset($data['commentId']) && ($commentId = intval($data['commentId']))) {
                $isProfileActive = in_array(self::SECTION_PROFILE, $this->options->sections);
                $isActivityActive = in_array(self::SECTION_ACTIVITY, $this->options->sections);
                $isVotesActive = in_array(self::SECTION_VOTES, $this->options->sections);
                $isSubscriptionsActive = in_array(self::SECTION_SUBSCRIPTION, $this->options->sections);
                $isFollowsActive = in_array(self::SECTION_FOLLOWS, $this->options->sections);

                $comment = get_comment($commentId);
                $user = get_user_by('ID', $comment->user_id);
                if (!$user && $wpdiscuz->optionsSerialized->isUserByEmail) {
                    $user = get_user_by('email', $comment->comment_author_email);
                }
                $response .= "<div class='wcai-wrapper'>";
                $response .= "<ul class='wcai-list'>";
                if ($isProfileActive) {
                    $response .= $this->getProfileTitleHtml();
                }
                if ($isActivityActive) {
                    $response .= $this->getActivityTitleHtml();
                }
                if ($user && $user->ID) {
                    if ($isVotesActive) {
                        $response .= $this->getVotesTitleHtml();
                    }
                    if ($isSubscriptionsActive) {
                        $response .= $this->getSubscriptionsTitleHtml();
                    }
                    if ($isFollowsActive) {
                        $response .= $this->getFollowsTitleHtml();
                    }
                }
                $response .= "</ul>";
                $response .= "<div class='wcai-content'>";
                if ($isProfileActive) {
                    $response .= $this->getProfileContentHtml($wpdiscuz, $user, $comment);
                }
                if ($isActivityActive) {
                    $response .= $this->getActivityContentHtml($wpdiscuz, $user, $comment);
                }
                if ($user && $user->ID) {
                    if ($isVotesActive) {
                        $response .= $this->getVotesContentHtml($wpdiscuz, $user, $comment);
                    }
                    if ($isSubscriptionsActive) {
                        $response .= $this->getSubscriptionsContentHtml($wpdiscuz, $user, $comment);
                    }
                    if ($isFollowsActive) {
                        $response .= $this->getFollowsContentHtml($wpdiscuz, $user, $comment);
                    }
                }
                $response .= "</div>";
                $response .= "<input type='hidden' class='wcai-comment-id' value='{$comment->comment_ID}'/>";
                $response .= "<input type='hidden' class='wcai-info-type' value='wcaiInfo'/>";
                $response .= "</div>";
            }
        }
        wp_die($response);
    }

    public function wcaiGetShortInfo() {
        $wpdiscuz = wpDiscuz();
        $response = '';
        if ($_POST && isset($_POST['wcaiAjaxData'])) {
            parse_str($_POST['wcaiAjaxData'], $data);
            if (isset($data['commentId']) && ($commentId = intval($data['commentId']))) {
                $comment = get_comment($commentId);
                $user = get_user_by('ID', $comment->user_id);
                if (!$user && $wpdiscuz->optionsSerialized->isUserByEmail) {
                    $user = get_user_by('email', $comment->comment_author_email);
                }
                $response .= "<div class='wcai-wrapper'>";
                $response .= "<div class='wcai-content'>";
                $response .= $this->getProfileShortContentHtml($wpdiscuz, $user, $comment);
                $response .= "</div>";
                $response .= "<input type='hidden' class='wcai-comment-id' value='{$comment->comment_ID}'/>";
                $response .= "<input type='hidden' class='wcai-info-type' value='wcaiInfoShort'/>";
                $response .= "</div>";
            }
        }
        wp_die($response);
    }

    private function getProfileTitleHtml() {
        $html = '';
        include_once 'layouts/profile/title.php';
        return $html;
    }

    private function getActivityTitleHtml() {
        $html = '';
        include_once 'layouts/activity/title.php';
        return $html;
    }

    private function getVotesTitleHtml() {
        $html = '';
        include_once 'layouts/votes/title.php';
        return $html;
    }

    private function getSubscriptionsTitleHtml() {
        $html = '';
        include_once 'layouts/subscriptions/title.php';
        return $html;
    }

    private function getFollowsTitleHtml() {
        $html = '';
        include_once 'layouts/follows/title.php';
        return $html;
    }

    private function getProfileContentHtml($wpdiscuz, $user, $comment) {
        $html = "<div id='wcai-content-item-1' class='wcai-content-item wcai-active'>";
        if ($user && $user->ID) {
            $allComments = $this->dbManager->getAuthorAllComments($user->ID);
            $commentStatistic = $this->getAuthorStatistic($allComments);
            include_once 'layouts/profile/content-user.php';
        } else {
            $allComments = $this->dbManager->getAuthorAllComments(0, $comment->comment_author_email);
            $commentStatistic = $this->getAuthorStatistic($allComments);
            include_once 'layouts/profile/content-guest.php';
        }
        $html .= "</div>";
        return $html;
    }

    private function getProfileShortContentHtml($wpdiscuz, $user, $comment) {
        $html = "<div id='wcai-content-item-1' class='wcai-content-item wcai-content-item-short wcai-active'>";
        if ($user && $user->ID) {
            $allComments = $this->dbManager->getAuthorAllComments($user->ID);
            $commentStatistic = $this->getAuthorStatistic($allComments);
            include_once 'layouts/profile/content-user-short.php';
        } else {
            $allComments = $this->dbManager->getAuthorAllComments(0, $comment->comment_author_email);
            $commentStatistic = $this->getAuthorStatistic($allComments);
            include_once 'layouts/profile/content-guest-short.php';
        }
        $html .= "</div>";
        return $html;
    }

    private function getActivityContentHtml($wpdiscuz, $user, $comment) {
        $html = "<div id='wcai-content-item-2' class='wcai-content-item'>";
        include_once 'layouts/activity/content.php';
        $html .= "</div>";
        return $html;
    }

    private function getVotesContentHtml($wpdiscuz, $user, $comment) {
        $perPage = $this->options->perPage;
        $itemsCount = $this->dbManager->getGivenLikesDislikesCount($user->ID);
        $items = $this->dbManager->getGivenLikesDislikes($user->ID, $perPage, 0);
        $html = "<div id='wcai-content-item-3' class='wcai-content-item'>";
        include_once 'layouts/votes/content.php';
        $html .= "</div>";
        return $html;
    }

    private function getSubscriptionsContentHtml($wpdiscuz, $user, $comment) {
        $perPage = $this->options->perPage;
        $itemsCount = $this->dbManager->getSubscriptionsCount($user->user_email);
        $items = $this->dbManager->getSubscriptions($user->user_email, $perPage, 0);
        $html = "<div id='wcai-content-item-4' class='wcai-content-item'>";
        include_once 'layouts/subscriptions/content.php';
        $html .= "</div>";
        return $html;
    }

    private function getFollowsContentHtml($wpdiscuz, $user, $comment) {
        $perPage = $this->options->perPage;
        $itemsCount = $this->dbManager->getFollowsCount($user->user_email);
        $items = $this->dbManager->getFollows($user->user_email, $perPage, 0);
        $html = "<div id='wcai-content-item-5' class='wcai-content-item'>";
        include_once 'layouts/follows/content.php';
        $html .= "</div>";
        return $html;
    }

    public function getActivityPage() {
        $wpdiscuz = wpDiscuz();
        $html = '';
        include_once 'layouts/activity/activity-page.php';
        wp_die($html);
    }

    public function getVotesPage() {
        $wpdiscuz = wpDiscuz();
        $html = '';
        include_once 'layouts/votes/votes-page.php';
        wp_die($html);
    }

    public function getSubscriptionsPage() {
        $wpdiscuz = wpDiscuz();
        $html = '';
        include_once 'layouts/subscriptions/subscriptions-page.php';
        wp_die($html);
    }

    public function getFollowsPage() {
        $wpdiscuz = wpDiscuz();
        $html = '';
        include_once 'layouts/follows/follows-page.php';
        wp_die($html);
    }

    public function wcaiUnsubscribe() {
        if (isset($_POST['wcaiAjaxData']) && $_POST['wcaiAjaxData']) {
            parse_str($_POST['wcaiAjaxData'], $data);
            $sId = isset($data['sId']) ? intval($data['sId']) : 0;
            $this->dbManager->wcaiUnsubscribe($sId);
            $this->getSubscriptionsPage();
        }
    }
    
    public function wcaiUnfollow() {
        if (isset($_POST['wcaiAjaxData']) && $_POST['wcaiAjaxData']) {
            parse_str($_POST['wcaiAjaxData'], $data);
            $sId = isset($data['fId']) ? intval($data['fId']) : 0;
            $this->dbManager->wcaiUnfollow($sId);
            $this->getFollowsPage();
        }
    }

    private function isAllowedRole($user) {
        $isAllowed = false;
        if ($user && isset($user->roles) && is_array($user->roles)) {
            foreach ($user->roles as $role) {
                if (in_array($role, $this->options->showForRoles)) {
                    $isAllowed = true;
                    break;
                }
            }
        }
        return $isAllowed;
    }

    private function getAuthorStatistic($allComments) {
        $commentStatistic = array('all' => 0, 'approved' => 0, 'pending' => 0, 'spam' => 0, 'trash' => 0, 'likes' => 0, 'dislikes' => 0);
        if ($allComments && is_array($allComments)) {
            $approved = 0;
            $pending = 0;
            $spam = 0;
            $trash = 0;
            $commentIds = array();
            foreach ($allComments as $comment) {
                if ($comment->comment_approved === '1') {
                    $commentIds[] = $comment->comment_ID;
                    $approved++;
                } else if ($comment->comment_approved === '0') {
                    $pending++;
                } else if ($comment->comment_approved === 'spam') {
                    $spam++;
                } else if ($comment->comment_approved === 'trash') {
                    $trash++;
                }
            }
            $commentStatistic['all'] = count($allComments);
            $commentStatistic['approved'] = $approved;
            $commentStatistic['pending'] = $pending;
            $commentStatistic['spam'] = $spam;
            $commentStatistic['trash'] = $trash;
            $votes = $this->dbManager->getReceivedLikesDislikes($commentIds);
            $commentStatistic['likes'] = $votes['likes'];
            $commentStatistic['dislikes'] = $votes['dislikes'];
        }
        return $commentStatistic;
    }

    public function getCommentDate($comment) {
        $wpdiscuz = wpDiscuz();
        if ($wpdiscuz->optionsSerialized->simpleCommentDate) {
            $dateFormat = $wpdiscuz->optionsSerialized->wordpressDateFormat;
            $timeFormat = $wpdiscuz->optionsSerialized->wordpressTimeFormat;
            if (self::isPostedToday($comment)) {
                $postedDate = $wpdiscuz->optionsSerialized->phrases['wc_posted_today_text'] . ' ' . mysql2date($timeFormat, $comment->comment_date);
            } else {
                $postedDate = get_comment_date($dateFormat . ' ' . $timeFormat, $comment->comment_ID);
            }
        } else {
            $postedDate = $this->dateDiff($comment->comment_date_gmt);
        }
        return $postedDate;
    }

    public function getPostDate($post) {
        $wpdiscuz = wpDiscuz();
        if ($wpdiscuz->optionsSerialized->simpleCommentDate) {
            $dateFormat = $wpdiscuz->optionsSerialized->wordpressDateFormat;
            $timeFormat = $wpdiscuz->optionsSerialized->wordpressTimeFormat;
            if ($this->isPostPostedToday($post)) {
                $postedDate = $wpdiscuz->optionsSerialized->phrases['wc_posted_today_text'] . ' ' . mysql2date($timeFormat, $post->post_date);
            } else {
                $postedDate = get_the_date($dateFormat . ' ' . $timeFormat, $post);
            }
        } else {
            $postedDate = $this->dateDiff($post->post_date_gmt);
        }
        return $postedDate;
    }

    public function getDate($comment) {
        $wpdiscuz = wpDiscuz();
        if ($wpdiscuz->optionsSerialized->simpleCommentDate) {
            $dateFormat = $wpdiscuz->optionsSerialized->wordpressDateFormat;
            $timeFormat = $wpdiscuz->optionsSerialized->wordpressTimeFormat;
            if (self::isPostedToday($comment)) {
                $postedDate = $wpdiscuz->optionsSerialized->phrases['wc_posted_today_text'] . ' ' . mysql2date($timeFormat, $comment);
            } else {
                $postedDate = date($dateFormat . ' ' . $timeFormat, strtotime($comment));
            }
        } else {
            $postedDate = $this->dateDiff($comment);
        }
        return $postedDate;
    }

    /**
     * check if comment has been posted today or not
     * return boolean
     */
    public static function isPostedToday($comment) {
        if ($comment && is_object($comment)) {
            return date('Ymd', strtotime(current_time('Ymd'))) <= date('Ymd', strtotime($comment->comment_date));
        } else {
            return date('Ymd', strtotime(current_time('Ymd'))) <= date('Ymd', strtotime($comment));
        }
    }

    private function isPostPostedToday($post) {
        return date('Ymd', strtotime(current_time('Ymd'))) <= date('Ymd', strtotime($post->post_date));
    }

    public function dateDiff($datetime) {
        $text = '';
        if ($datetime) {
            $wpdiscuz = wpDiscuz();
            $now = new DateTime();
            $ago = new DateTime($datetime);
            $diff = $now->diff($ago);
            if ($diff->y) {
                $text .= $diff->y . ' ';
                $text .= $diff->y > 1 ? $wpdiscuz->optionsSerialized->phrases['wc_year_text_plural'] : $wpdiscuz->optionsSerialized->phrases['wc_year_text'];
            } else if ($diff->m) {
                $text .= $diff->m . ' ';
                $text .= $diff->m > 1 ? $wpdiscuz->optionsSerialized->phrases['wc_month_text_plural'] : $wpdiscuz->optionsSerialized->phrases['wc_month_text'];
            } else if ($diff->d) {
                $text .= $diff->d . ' ';
                $text .= $diff->d > 1 ? $wpdiscuz->optionsSerialized->phrases['wc_day_text_plural'] : $wpdiscuz->optionsSerialized->phrases['wc_day_text'];
            } else if ($diff->h) {
                $text .= $diff->h . ' ';
                $text .= $diff->h > 1 ? $wpdiscuz->optionsSerialized->phrases['wc_hour_text_plural'] : $wpdiscuz->optionsSerialized->phrases['wc_hour_text'];
            } else if ($diff->i) {
                $text .= $diff->i . ' ';
                $text .= $diff->i > 1 ? $wpdiscuz->optionsSerialized->phrases['wc_minute_text_plural'] : $wpdiscuz->optionsSerialized->phrases['wc_minute_text'];
            } else if ($diff->s) {
                $text .= $diff->s . ' ';
                $text .= $diff->s > 1 ? $wpdiscuz->optionsSerialized->phrases['wc_second_text_plural'] : $wpdiscuz->optionsSerialized->phrases['wc_second_text'];
            }
            $text .= ($text) ? ' ' . $wpdiscuz->optionsSerialized->phrases['wc_ago_text'] : ' ' . $wpdiscuz->optionsSerialized->phrases['wc_right_now_text'];
        }
        return $text;
    }

    public function commentLeftClasses($classes) {
        $currentUser = wp_get_current_user();
        if (($currentUser && $this->isAllowedRole($currentUser) || (!WpdiscuzCommentAuthorInfo::$IS_LOGGED_IN && $this->options->showForGuests)) && $this->options->shortInfoOnAvatarHover) {
            $classes .= 'wcai-short-info wcai-not-clicked';
        }
        return $classes;
    }

    public function usernameClasses($classes) {
        $currentUser = wp_get_current_user();
        if (($currentUser && $this->isAllowedRole($currentUser) || (!WpdiscuzCommentAuthorInfo::$IS_LOGGED_IN && $this->options->showForGuests)) && $this->options->fullInfoOnUsernameClick && $this->options->sections) {
            $classes .= 'wcai-uname-info wcai-not-clicked';
        }
        return $classes;
    }

}
