<?php

if (!defined('ABSPATH')) {
    exit();
}

class WCAIOptions implements WCAIConstants {

    private $dbManager;
    public $sections;
    public $showForRoles;
    public $showForGuests;
    public $shortInfoOnAvatarHover;
    public $fullInfoOnUsernameClick;
    /* SECTION MAIN INFO */
    public $profileShowDisplayName;
    public $profileShowNickName;
    public $profileShowBio;
    public $profileShowWebUrl;
    public $profileShowStatistics;
    public $profileShowMycredData;
    /* PER PAGE OPTION */
    public $perPage;

    /* PHRASES */
    /* === SECTIONS AND COMMON === */
    public $phraseProfileSectionTitle;
    public $phraseActivitySectionTitle;
    public $phraseVotesSectionTitle;
    public $phraseSubscriptionsSectionTitle;
    public $phraseFollowsSectionTitle;
    public $phraseNotAvailable;
    public $phraseNoData;
    public $phraseFullInfo;
    /* === PROFILE === */
    public $phraseProfileLastActivity;
    public $phraseProfileComments;
    public $phraseProfilePosts;
    public $phraseProfileReceivedLikes;
    public $phraseProfileReceivedDisikes;
    public $phraseProfileAuthorBio;
    public $phraseProfileCommentsStat;
    public $phraseProfileCommentsStatAll;
    public $phraseProfileCommentsStatApproved;
    public $phraseProfileCommentsStatPending;
    public $phraseProfileCommentsStatSpam;
    public $phraseProfileCommentsStatTrashed;
    /* === ACTIVITY === */
    public $phraseActivityInResponseTo;
    /* === VOTES === */
    public $phraseVotesInResponseTo;
    /* === SUBSCRIPTIONS === */
    public $phraseSubscriptionsReply;
    public $phraseSubscriptionsAllComment;
    public $phraseSubscriptionsPost;
    /* === PAGINATION === */
    public $phrasePaginationFirst;
    public $phrasePaginationPrevious;
    public $phrasePaginationNext;
    public $phrasePaginationLast;

    public function __construct($dbManager) {
        $this->dbManager = $dbManager;
        $options = maybe_unserialize(get_option(self::OPTION_MAIN_OPTIONS));
        $this->addOptions();
        $this->initOptions($options);
    }

    public function addOptions() {
        $options = array(
            'sections' => $this->getDefaultSections(),
            'showForRoles' => $this->getDefaultRoles(),
            'showForGuests' => 1,
            'shortInfoOnAvatarHover' => 1,
            'fullInfoOnUsernameClick' => 1,
            'profileShowDisplayName' => 1,
            'profileShowNickName' => 1,
            'profileShowBio' => 1,
            'profileShowWebUrl' => 1,
            'profileShowStatistics' => 1,
            'profileShowMycredData' => 1,
            'perPage' => 5,
            'phraseProfileSectionTitle' => __('Profile', 'wpdiscuz-comment-author-info'),
            'phraseActivitySectionTitle' => __('Activity', 'wpdiscuz-comment-author-info'),
            'phraseVotesSectionTitle' => __('Votes', 'wpdiscuz-comment-author-info'),
            'phraseSubscriptionsSectionTitle' => __('Subscriptions', 'wpdiscuz-comment-author-info'),
            'phraseFollowsSectionTitle' => __('Follows', 'wpdiscuz-comment-author-info'),
            'phraseNotAvailable' => __('Not available', 'wpdiscuz-comment-author-info'),
            'phraseNoData' => __('No Data', 'wpdiscuz-comment-author-info'),
            'phraseFullInfo' => __('View full info', 'wpdiscuz-comment-author-info'),
            'phraseProfileLastActivity' => __('Last Activity:', 'wpdiscuz-comment-author-info'),
            'phraseProfileComments' => __('Comments', 'wpdiscuz-comment-author-info'),
            'phraseProfilePosts' => __('Posts', 'wpdiscuz-comment-author-info'),
            'phraseProfileReceivedLikes' => __('Received Likes', 'wpdiscuz-comment-author-info'),
            'phraseProfileReceivedDisikes' => __('Received Disikes', 'wpdiscuz-comment-author-info'),
            'phraseProfileAuthorBio' => __('Comment Author Biography', 'wpdiscuz-comment-author-info'),
            'phraseProfileCommentsStat' => __('Comments Statistic', 'wpdiscuz-comment-author-info'),
            'phraseProfileCommentsStatAll' => __('All', 'wpdiscuz-comment-author-info'),
            'phraseProfileCommentsStatApproved' => __('Approved', 'wpdiscuz-comment-author-info'),
            'phraseProfileCommentsStatPending' => __('Pending', 'wpdiscuz-comment-author-info'),
            'phraseProfileCommentsStatSpam' => __('Spam', 'wpdiscuz-comment-author-info'),
            'phraseProfileCommentsStatTrashed' => __('Trashed', 'wpdiscuz-comment-author-info'),
            'phraseProfileMyCredData' => __('MyCred Data', 'wpdiscuz-comment-author-info'),
            'phraseProfileBadges' => __('Badges', 'wpdiscuz-comment-author-info'),
            'phraseProfileRank' => __('Rank', 'wpdiscuz-comment-author-info'),
            'phraseActivityInResponseTo' => __('In Response To:', 'wpdiscuz-comment-author-info'),
            'phraseVotesInResponseTo' => __('In Response To:', 'wpdiscuz-comment-author-info'),
            'phraseSubscriptionsReply' => __('subscribed to replies to this comment', 'wpdiscuz-comment-author-info'),
            'phraseSubscriptionsAllComment' => __('subscribed to replies to own comments', 'wpdiscuz-comment-author-info'),
            'phraseSubscriptionsPost' => __('subscribed to all follow-up comments of this post', 'wpdiscuz-comment-author-info'),
            'phrasePaginationFirst' => __('&laquo;', 'wpdiscuz-comment-author-info'),
            'phrasePaginationPrevious' => __('&lsaquo;', 'wpdiscuz-comment-author-info'),
            'phrasePaginationNext' => __('&rsaquo;', 'wpdiscuz-comment-author-info'),
            'phrasePaginationLast' => __('&raquo;', 'wpdiscuz-comment-author-info'),
        );
        add_option(self::OPTION_MAIN_OPTIONS, $options, '', true);
    }

    public function initOptions($options) {
        $options = maybe_unserialize($options);
        $this->sections = isset($options['sections']) && is_array($options['sections']) ? $options['sections'] : $this->getDefaultSections();
        $this->showForRoles = isset($options['showForRoles']) && is_array($options['showForRoles']) ? $options['showForRoles'] : array();
        $this->showForGuests = isset($options['showForGuests']) && ($option = intval($options['showForGuests'])) ? $option : 0;
        $this->shortInfoOnAvatarHover = isset($options['shortInfoOnAvatarHover']) && ($option = intval($options['shortInfoOnAvatarHover'])) ? $option : 0;
        $this->fullInfoOnUsernameClick = isset($options['fullInfoOnUsernameClick']) && ($option = intval($options['fullInfoOnUsernameClick'])) ? $option : 0;
        $this->profileShowDisplayName = isset($options['profileShowDisplayName']) && ($option = intval($options['profileShowDisplayName'])) ? $option : 0;
        $this->profileShowNickName = isset($options['profileShowNickName']) && ($option = intval($options['profileShowNickName'])) ? $option : 0;
        $this->profileShowBio = isset($options['profileShowBio']) && ($option = intval($options['profileShowBio'])) ? $option : 0;
        $this->profileShowWebUrl = isset($options['profileShowWebUrl']) && ($option = intval($options['profileShowWebUrl'])) ? $option : 0;
        $this->profileShowStatistics = isset($options['profileShowStatistics']) && ($option = intval($options['profileShowStatistics'])) ? $option : 0;
        $this->profileShowMycredData = isset($options['profileShowMycredData']) && ($option = intval($options['profileShowMycredData'])) ? $option : 0;
        $this->perPage = isset($options['perPage']) && ($option = intval($options['perPage'])) ? $option : 5;
        $this->phraseProfileSectionTitle = isset($options['phraseProfileSectionTitle']) && ($phrase = esc_html($options['phraseProfileSectionTitle'])) ? $phrase : '';
        $this->phraseActivitySectionTitle = isset($options['phraseActivitySectionTitle']) && ($phrase = esc_html($options['phraseActivitySectionTitle'])) ? $phrase : '';
        $this->phraseVotesSectionTitle = isset($options['phraseVotesSectionTitle']) && ($phrase = esc_html($options['phraseVotesSectionTitle'])) ? $phrase : '';
        $this->phraseSubscriptionsSectionTitle = isset($options['phraseSubscriptionsSectionTitle']) && ($phrase = esc_html($options['phraseSubscriptionsSectionTitle'])) ? $phrase : '';
        $this->phraseFollowsSectionTitle = isset($options['phraseFollowsSectionTitle']) && ($phrase = esc_html($options['phraseFollowsSectionTitle'])) ? $phrase : '';
        $this->phraseNotAvailable = isset($options['phraseNotAvailable']) && ($phrase = esc_html($options['phraseNotAvailable'])) ? $phrase : '';
        $this->phraseNoData = isset($options['phraseNoData']) && ($phrase = esc_html($options['phraseNoData'])) ? $phrase : '';
        $this->phraseFullInfo = isset($options['phraseFullInfo']) && ($phrase = esc_html($options['phraseFullInfo'])) ? $phrase : '';
        $this->phraseProfileLastActivity = isset($options['phraseProfileLastActivity']) && ($phrase = esc_html($options['phraseProfileLastActivity'])) ? $phrase : '';
        $this->phraseProfileComments = isset($options['phraseProfileComments']) && ($phrase = esc_html($options['phraseProfileComments'])) ? $phrase : '';
        $this->phraseProfilePosts = isset($options['phraseProfilePosts']) && ($phrase = esc_html($options['phraseProfilePosts'])) ? $phrase : '';
        $this->phraseProfileReceivedLikes = isset($options['phraseProfileReceivedLikes']) && ($phrase = esc_html($options['phraseProfileReceivedLikes'])) ? $phrase : '';
        $this->phraseProfileReceivedDisikes = isset($options['phraseProfileReceivedDisikes']) && ($phrase = esc_html($options['phraseProfileReceivedDisikes'])) ? $phrase : '';
        $this->phraseProfileAuthorBio = isset($options['phraseProfileAuthorBio']) && ($phrase = esc_html($options['phraseProfileAuthorBio'])) ? $phrase : '';
        $this->phraseProfileCommentsStat = isset($options['phraseProfileCommentsStat']) && ($phrase = esc_html($options['phraseProfileCommentsStat'])) ? $phrase : '';
        $this->phraseProfileCommentsStatAll = isset($options['phraseProfileCommentsStatAll']) && ($phrase = esc_html($options['phraseProfileCommentsStatAll'])) ? $phrase : '';
        $this->phraseProfileCommentsStatApproved = isset($options['phraseProfileCommentsStatApproved']) && ($phrase = esc_html($options['phraseProfileCommentsStatApproved'])) ? $phrase : '';
        $this->phraseProfileCommentsStatPending = isset($options['phraseProfileCommentsStatPending']) && ($phrase = esc_html($options['phraseProfileCommentsStatPending'])) ? $phrase : '';
        $this->phraseProfileCommentsStatSpam = isset($options['phraseProfileCommentsStatSpam']) && ($phrase = esc_html($options['phraseProfileCommentsStatSpam'])) ? $phrase : '';
        $this->phraseProfileCommentsStatTrashed = isset($options['phraseProfileCommentsStatTrashed']) && ($phrase = esc_html($options['phraseProfileCommentsStatTrashed'])) ? $phrase : '';
        $this->phraseActivityInResponseTo = isset($options['phraseActivityInResponseTo']) && ($phrase = esc_html($options['phraseActivityInResponseTo'])) ? $phrase : '';
        $this->phraseVotesInResponseTo = isset($options['phraseVotesInResponseTo']) && ($phrase = esc_html($options['phraseVotesInResponseTo'])) ? $phrase : '';
        $this->phraseSubscriptionsReply = isset($options['phraseSubscriptionsReply']) && ($phrase = esc_html($options['phraseSubscriptionsReply'])) ? $phrase : '';
        $this->phraseSubscriptionsAllComment = isset($options['phraseSubscriptionsAllComment']) && ($phrase = esc_html($options['phraseSubscriptionsAllComment'])) ? $phrase : '';
        $this->phraseSubscriptionsPost = isset($options['phraseSubscriptionsPost']) && ($phrase = esc_html($options['phraseSubscriptionsPost'])) ? $phrase : '';
        $this->phrasePaginationFirst = isset($options['phrasePaginationFirst']) && ($phrase = esc_html($options['phrasePaginationFirst'])) ? $phrase : '';
        $this->phrasePaginationPrevious = isset($options['phrasePaginationPrevious']) && ($phrase = esc_html($options['phrasePaginationPrevious'])) ? $phrase : '';
        $this->phrasePaginationNext = isset($options['phrasePaginationNext']) && ($phrase = esc_html($options['phrasePaginationNext'])) ? $phrase : '';
        $this->phrasePaginationLast = isset($options['phrasePaginationLast']) && ($phrase = esc_html($options['phrasePaginationLast'])) ? $phrase : '';
    }

    public function toArray() {
        $options = array(
            'sections' => $this->sections,
            'showForRoles' => $this->showForRoles,
            'showForGuests' => $this->showForGuests,
            'shortInfoOnAvatarHover' => $this->shortInfoOnAvatarHover,
            'fullInfoOnUsernameClick' => $this->fullInfoOnUsernameClick,
            'profileShowDisplayName' => $this->profileShowDisplayName,
            'profileShowNickName' => $this->profileShowNickName,
            'profileShowBio' => $this->profileShowBio,
            'profileShowWebUrl' => $this->profileShowWebUrl,
            'profileShowStatistics' => $this->profileShowStatistics,
            'profileShowMycredData' => $this->profileShowMycredData,
            'perPage' => $this->perPage,
            'phraseProfileSectionTitle' => $this->phraseProfileSectionTitle,
            'phraseActivitySectionTitle' => $this->phraseActivitySectionTitle,
            'phraseVotesSectionTitle' => $this->phraseVotesSectionTitle,
            'phraseSubscriptionsSectionTitle' => $this->phraseSubscriptionsSectionTitle,
            'phraseFollowsSectionTitle' => $this->phraseFollowsSectionTitle,
            'phraseNotAvailable' => $this->phraseNotAvailable,
            'phraseNoData' => $this->phraseNoData,
            'phraseFullInfo' => $this->phraseFullInfo,
            'phraseProfileLastActivity' => $this->phraseProfileLastActivity,
            'phraseProfileComments' => $this->phraseProfileComments,
            'phraseProfilePosts' => $this->phraseProfilePosts,
            'phraseProfileReceivedLikes' => $this->phraseProfileReceivedLikes,
            'phraseProfileReceivedDisikes' => $this->phraseProfileReceivedDisikes,
            'phraseProfileAuthorBio' => $this->phraseProfileAuthorBio,
            'phraseProfileCommentsStat' => $this->phraseProfileCommentsStat,
            'phraseProfileCommentsStatAll' => $this->phraseProfileCommentsStatAll,
            'phraseProfileCommentsStatApproved' => $this->phraseProfileCommentsStatApproved,
            'phraseProfileCommentsStatPending' => $this->phraseProfileCommentsStatPending,
            'phraseProfileCommentsStatSpam' => $this->phraseProfileCommentsStatSpam,
            'phraseProfileCommentsStatTrashed' => $this->phraseProfileCommentsStatTrashed,
            'phraseActivityInResponseTo' => $this->phraseActivityInResponseTo,
            'phraseVotesInResponseTo' => $this->phraseVotesInResponseTo,
            'phraseSubscriptionsReply' => $this->phraseSubscriptionsReply,
            'phraseSubscriptionsAllComment' => $this->phraseSubscriptionsAllComment,
            'phraseSubscriptionsPost' => $this->phraseSubscriptionsPost,
            'phrasePaginationFirst' => $this->phrasePaginationFirst,
            'phrasePaginationPrevious' => $this->phrasePaginationPrevious,
            'phrasePaginationNext' => $this->phrasePaginationNext,
            'phrasePaginationLast' => $this->phrasePaginationLast,
        );
        return $options;
    }

    public function tabTitle() {
        echo '<li>' . __('Comment Author Info', 'wpdiscuz-comment-author-info') . '</li>';
    }

    public function tabContent() {
        include_once 'wcai-html-options.php';
    }

    public function saveOptions() {
        $this->sections = isset($_POST['sections']) && is_array($_POST['sections']) ? $_POST['sections'] : array();
        $this->showForRoles = isset($_POST['showForRoles']) && is_array($_POST['showForRoles']) ? $_POST['showForRoles'] : array();
        $this->showForGuests = filter_input(INPUT_POST, 'showForGuests', FILTER_VALIDATE_INT);
        $this->shortInfoOnAvatarHover = filter_input(INPUT_POST, 'shortInfoOnAvatarHover', FILTER_VALIDATE_INT);
        $this->fullInfoOnUsernameClick = filter_input(INPUT_POST, 'fullInfoOnUsernameClick', FILTER_VALIDATE_INT);
        $this->profileShowDisplayName = filter_input(INPUT_POST, 'profileShowDisplayName', FILTER_VALIDATE_INT);
        $this->profileShowNickName = filter_input(INPUT_POST, 'profileShowNickName', FILTER_VALIDATE_INT);
        $this->profileShowBio = filter_input(INPUT_POST, 'profileShowBio', FILTER_VALIDATE_INT);
        $this->profileShowWebUrl = filter_input(INPUT_POST, 'profileShowWebUrl', FILTER_VALIDATE_INT);
        $this->profileShowStatistics = filter_input(INPUT_POST, 'profileShowStatistics', FILTER_VALIDATE_INT);
        $this->profileShowMycredData = filter_input(INPUT_POST, 'profileShowMycredData', FILTER_VALIDATE_INT);
        $this->perPage = filter_input(INPUT_POST, 'perPage', FILTER_VALIDATE_INT);
        $this->phraseProfileSectionTitle = filter_input(INPUT_POST, 'phraseProfileSectionTitle', FILTER_SANITIZE_STRING);
        $this->phraseActivitySectionTitle = filter_input(INPUT_POST, 'phraseActivitySectionTitle', FILTER_SANITIZE_STRING);
        $this->phraseVotesSectionTitle = filter_input(INPUT_POST, 'phraseVotesSectionTitle', FILTER_SANITIZE_STRING);
        $this->phraseSubscriptionsSectionTitle = filter_input(INPUT_POST, 'phraseSubscriptionsSectionTitle', FILTER_SANITIZE_STRING);
        $this->phraseFollowsSectionTitle = filter_input(INPUT_POST, 'phraseFollowsSectionTitle', FILTER_SANITIZE_STRING);
        $this->phraseNotAvailable = filter_input(INPUT_POST, 'phraseNotAvailable', FILTER_SANITIZE_STRING);
        $this->phraseNoData = filter_input(INPUT_POST, 'phraseNoData', FILTER_SANITIZE_STRING);
        $this->phraseFullInfo = filter_input(INPUT_POST, 'phraseFullInfo', FILTER_SANITIZE_STRING);
        $this->phraseProfileLastActivity = filter_input(INPUT_POST, 'phraseProfileLastActivity', FILTER_SANITIZE_STRING);
        $this->phraseProfileComments = filter_input(INPUT_POST, 'phraseProfileComments', FILTER_SANITIZE_STRING);
        $this->phraseProfilePosts = filter_input(INPUT_POST, 'phraseProfilePosts', FILTER_SANITIZE_STRING);
        $this->phraseProfileReceivedLikes = filter_input(INPUT_POST, 'phraseProfileReceivedLikes', FILTER_SANITIZE_STRING);
        $this->phraseProfileReceivedDisikes = filter_input(INPUT_POST, 'phraseProfileReceivedDisikes', FILTER_SANITIZE_STRING);
        $this->phraseProfileAuthorBio = filter_input(INPUT_POST, 'phraseProfileAuthorBio', FILTER_SANITIZE_STRING);
        $this->phraseProfileCommentsStat = filter_input(INPUT_POST, 'phraseProfileCommentsStat', FILTER_SANITIZE_STRING);
        $this->phraseProfileCommentsStatAll = filter_input(INPUT_POST, 'phraseProfileCommentsStatAll', FILTER_SANITIZE_STRING);
        $this->phraseProfileCommentsStatApproved = filter_input(INPUT_POST, 'phraseProfileCommentsStatApproved', FILTER_SANITIZE_STRING);
        $this->phraseProfileCommentsStatPending = filter_input(INPUT_POST, 'phraseProfileCommentsStatPending', FILTER_SANITIZE_STRING);
        $this->phraseProfileCommentsStatSpam = filter_input(INPUT_POST, 'phraseProfileCommentsStatSpam', FILTER_SANITIZE_STRING);
        $this->phraseProfileCommentsStatTrashed = filter_input(INPUT_POST, 'phraseProfileCommentsStatTrashed', FILTER_SANITIZE_STRING);
        $this->phraseActivityInResponseTo = filter_input(INPUT_POST, 'phraseActivityInResponseTo', FILTER_SANITIZE_STRING);
        $this->phraseVotesInResponseTo = filter_input(INPUT_POST, 'phraseVotesInResponseTo', FILTER_SANITIZE_STRING);
        $this->phraseSubscriptionsReply = filter_input(INPUT_POST, 'phraseSubscriptionsReply', FILTER_SANITIZE_STRING);
        $this->phraseSubscriptionsAllComment = filter_input(INPUT_POST, 'phraseSubscriptionsAllComment', FILTER_SANITIZE_STRING);
        $this->phraseSubscriptionsPost = filter_input(INPUT_POST, 'phraseSubscriptionsPost', FILTER_SANITIZE_STRING);
        $this->phrasePaginationFirst = filter_input(INPUT_POST, 'phrasePaginationFirst', FILTER_SANITIZE_STRING);
        $this->phrasePaginationPrevious = filter_input(INPUT_POST, 'phrasePaginationPrevious', FILTER_SANITIZE_STRING);
        $this->phrasePaginationNext = filter_input(INPUT_POST, 'phrasePaginationNext', FILTER_SANITIZE_STRING);
        $this->phrasePaginationLast = filter_input(INPUT_POST, 'phrasePaginationLast', FILTER_SANITIZE_STRING);
        update_option(self::OPTION_MAIN_OPTIONS, $this->toArray());
    }

    private function getDefaultRoles() {
        return array('administrator', 'editor', 'author', 'contributor', 'subscriber');
    }

    private function getDefaultSections() {
        return array('profile', 'activity', 'votes', 'subscriptions', 'follows');
    }

}
