<?php

/*
 * Plugin Name: wpDiscuz - Comment Author Info
 * Description: Displays comment author profile information, activity, voted comments and subscriptions
 * Version: 1.1.1
 * Author: gVectors Team (A. Chakhoyan, G. Zakaryan, H. Martirosyan)
 * Author URI: https://gvectors.com/
 * Plugin URI: https://gvectors.com/product/wpdiscuz-comment-author-info/
 * Text Domain: wpdiscuz-comment-author-info
 * Domain Path: /languages/
 */
if (!defined('ABSPATH')) {
    exit();
}

define('WCAI_DIR_PATH', dirname(__FILE__));
define('WCAI_DIR_NAME', basename(WCAI_DIR_PATH));
define('WCAI_BASENAME_FILE', basename(__FILE__));

include_once 'includes/gvt-api-manager.php';
include_once 'includes/wcai-constants.php';
include_once 'includes/wcai-dbmanager.php';
include_once 'includes/wcai-helper.php';
include_once 'options/wcai-options.php';

class WpdiscuzCommentAuthorInfo implements WCAIConstants {

    private $version;
    private $dbManager;
    public $helper;
    private $options;
    public static $IS_LOGGED_IN;

    public function __construct() {
        add_action('admin_notices', array($this, 'requirements'), 1);
        add_action('plugins_loaded', array($this, 'dependencies'), 1);
    }

    public function requirements() {
        if (current_user_can('manage_options')) {
            if (!is_plugin_active('wpdiscuz/class.WpdiscuzCore.php')) {
                echo "<div class='error'><p>" . __('wpDiscuz Comment Author Info requires wpDiscuz to be installed!', 'wpdiscuz-comment-author-info') . "</p></div>";
            }

            if (!function_exists('ob_start') || !function_exists('ob_get_contents') || !function_exists('ob_end_clean')) {
                echo "<div class='error'><p>" . __('wpDiscuz Comment Author Info requires object buffering to be enabled!', 'wpdiscuz-comment-author-info') . "</p></div>";
            }
        }
    }

    public function dependencies() {
        if (function_exists('wpDiscuz')) {
            new GVT_API_Manager(__FILE__, 'wpdiscuz_options_page', 'wpdiscuz_option_page');
            self::$IS_LOGGED_IN = is_user_logged_in();
            $this->dbManager = new WCAIDBManager();
            $this->options = new WCAIOptions($this->dbManager);
            $this->helper = new WCAIHelper($this->options, $this->dbManager);

            $this->version = get_option(self::OPTION_VERSION);
            if (!$this->version) {
                $this->version = '1.0.0';
                update_option(self::OPTION_VERSION, $this->version);
            }
            load_plugin_textdomain('wpdiscuz-comment-author-info', false, WCAI_DIR_NAME . '/languages/');

            add_action('admin_enqueue_scripts', array(&$this, 'backendFiles'));
            add_action('wpdiscuz_front_scripts', array(&$this, 'frontendFiles'));
            add_action('wpdiscuz_check_version', array(&$this, 'version'));

            add_action('wpdiscuz_save_options', array(&$this->options, 'saveOptions'));
            add_action('admin_post_wcaiResetOptions', array(&$this->helper, 'wcaiResetOptions'));
            add_action('wpdiscuz_addon_tab_title', array(&$this->options, 'tabTitle'), 1);
            add_action('wpdiscuz_addon_tab_content', array(&$this->options, 'tabContent'), 1);
            add_filter('wpdiscuz_before_comment_link', array(&$this->helper, 'addInfoIcon'), 10, 4);
            add_filter('wpdiscuz_username_classes', array(&$this->helper, 'usernameClasses'));
            add_filter('wpdiscuz_comment_left_class', array(&$this->helper, 'commentLeftClasses'));
            add_action('wp_footer', array(&$this->helper, 'addLityPopup'));
            add_action('wp_ajax_wcaiGetInfo', array(&$this->helper, 'wcaiGetInfo'));
            add_action('wp_ajax_nopriv_wcaiGetInfo', array(&$this->helper, 'wcaiGetInfo'));
            add_action('wp_ajax_wcaiGetShortInfo', array(&$this->helper, 'wcaiGetShortInfo'));
            add_action('wp_ajax_nopriv_wcaiGetShortInfo', array(&$this->helper, 'wcaiGetShortInfo'));
            add_action('wp_ajax_wcaiGetActivityPage', array(&$this->helper, 'getActivityPage'));
            add_action('wp_ajax_nopriv_wcaiGetActivityPage', array(&$this->helper, 'getActivityPage'));
            add_action('wp_ajax_wcaiGetVotesPage', array(&$this->helper, 'getVotesPage'));
            add_action('wp_ajax_nopriv_wcaiGetVotesPage', array(&$this->helper, 'getVotesPage'));
            add_action('wp_ajax_wcaiGetSubscriptionsPage', array(&$this->helper, 'getSubscriptionsPage'));
            add_action('wp_ajax_nopriv_wcaiGetSubscriptionsPage', array(&$this->helper, 'getSubscriptionsPage'));
            add_action('wp_ajax_wcaiGetFollowsPage', array(&$this->helper, 'getFollowsPage'));
            add_action('wp_ajax_nopriv_wcaiGetFollowsPage', array(&$this->helper, 'getFollowsPage'));
            add_action('wp_ajax_wcaiUnsubscribe', array(&$this->helper, 'wcaiUnsubscribe'));
            add_action('wp_ajax_wcaiUnfollow', array(&$this->helper, 'wcaiUnfollow'));
        }
    }

    public function backendFiles() {
        $vars = array(
            'wcaiText_Yes' => __('Yes', 'wpdiscuz-comment-author-info'),
            'wcaiText_No' => __('No', 'wpdiscuz-comment-author-info'),
            'wcaiConfirmResetOptions' => __('Do you really want to reset all options?', 'wpdiscuz-comment-author-info'),
        );

        wp_enqueue_style('wcai-backend-css', plugins_url(WCAI_DIR_NAME . '/assets/css/wcai-backend.css'), null, $this->version);
        wp_register_script('wcai-backend-js', plugins_url(WCAI_DIR_NAME . '/assets/js/wcai-backend.js'), array('jquery'), $this->version, false);
        wp_enqueue_script('wcai-backend-js');
        wp_localize_script('wcai-backend-js', 'wcaiBackendVars', $vars);
    }

    public function frontendFiles($optionsSerialized) {
        wp_enqueue_style('wcai-frontend-css', plugins_url(WCAI_DIR_NAME . '/assets/css/wcai-frontend.css'), null, $this->version);
        wp_register_script('wcai-frontend-js', plugins_url(WCAI_DIR_NAME . '/assets/js/wcai-frontend.js'), array('jquery'), $this->version, $optionsSerialized->isLoadScriptsInFooter);
        wp_enqueue_script('wcai-frontend-js');
    }

    public function version() {
        $pluginData = get_plugin_data(__FILE__);
        if (version_compare($pluginData['Version'], $this->version, '>')) {
            $options = get_option(self::OPTION_MAIN_OPTIONS);
            $this->addNewOptions($options);
            update_option(self::OPTION_VERSION, $pluginData['Version']);
        }
    }

    private function addNewOptions($options) {
        $this->options->initOptions($options);
        $newOptions = $this->options->toArray();
        update_option(self::OPTION_MAIN_OPTIONS, $newOptions);
    }

}

$wpdiscuzCommentAuthorInfo = new WpdiscuzCommentAuthorInfo();
