<?php

if (!defined('ABSPATH')) {
    exit();
}

class wpDiscuzCommentSearchOption {

    private $optionName;
    public $savedOptions;

    public function __construct() {
        $this->optionName = 'wpdiscuz_comment_search';
        $this->addOption();
        $this->initOption();
    }

    public function searchOptionTabName() {
        echo '<li>' . __('Comment Search', 'wpdiscuz-search') . '</li>';
    }

    public function searchOptionTabContent() {
        if (current_user_can('manage_options') && isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'wpdiscuz_search_reset_options_nonce') && isset($_GET['search_reset_options']) && 1 == $_GET['search_reset_options']) {
            $this->resetOptions();
        }
        include_once 'option-html.php';
    }

    public function resetOptions() {
        $options = $this->defaultValues();
        update_option($this->optionName, $options);
        $this->initOption();
    }

    private function addOption() {
        add_option($this->optionName, $this->defaultValues());
        $options = array(
            'widget_title' => __('Comment Search', 'wpdiscuz-search'),
            'widget_display_post_count' => 5,
            'widget_display_avatar' => 'off',
        );
        add_option('widget_wpdiscuz_search_widget_options', $options);
    }

    public function defaultValues() {
        $options = array(
            'display_form_for_guests' => 1,
            'search_text_min_length' => 3,
            'display_setting' => 1,
            'search_available_fields' => $this->getAvailableFields(),
            'search_default_field' => 'all',
            'searched_data_bg' => '#C4ECE4',
            'search_icons_color' => '#00B38F',
            'search_text_color' => '#666666',
            'search_bg_color' => '#FFFFFF',
            'search_box_border_color' => '#CDCDCD',
            'dialog_text_color' => '#666666',
            'dialog_bg_color' => '#FFFFFF',
            'dialog_hover_color' => '#EEEEEE',
            'pagination_text_color' => '#666666',
            'pagination_item_bg_color' => '#C4ECE4',
            'widget_post_letters_count' => 60,
            'widget_post_title_color' => '#00B490',
            'widget_post_content_color' => '#686868',
            'widget_post_author_date_color' => '#CCC',
            'widget_title' => __('wpDiscuz All Comment Search', 'wpdiscuz-search'),
            'widget_display_post_count' => 10,
            'widget_display_avatar' => 'on',
            'widget_loadmore_bg' => '#DAF3EE',
            'widget_loadmore_text' => '#666666',
            'widget_loadmore_border' => '#CCCCCC',
            'search_result_phrase' => __('Search result', 'wpdiscuz-search'),
            'search_no_result_phrase' => __('No comment found', 'wpdiscuz-search'),
            'search_placeholder' => __('Comment search...', 'wpdiscuz-search'),
            'dialog_search_by_all' => __('All', 'wpdiscuz-search'),
            'dialog_search_by_content' => __('Content', 'wpdiscuz-search'),
            'dialog_search_by_author' => __('Author', 'wpdiscuz-search'),
            'dialog_search_by_email' => __('E-mail', 'wpdiscuz-search'),
            'dialog_search_by_custom_fields' => __('Custom Fields', 'wpdiscuz-search'),
        );
        return $options;
    }

    public function searchSaveOptions($args) {
        $this->options['display_form_for_guests'] = isset($args['display-form-for-guests']) ? $args['display-form-for-guests'] : 0;
        $this->options['search_text_min_length'] = isset($args['search_text_min_length']) && ($v = absint($args['search_text_min_length'])) ? $v : 3;
        $this->options['display_setting'] = isset($args['display-setting']) ? $args['display-setting'] : 0;
        $this->options['search_available_fields'] = isset($args['search-available-fields']) ? $args['search-available-fields'] : $this->getAvailableFields();
        $this->options['search_default_field'] = isset($args['search-default-field']) && in_array($args['search-default-field'], $this->options['search_available_fields']) ? $args['search-default-field'] : 'content';
        $this->options['searched_data_bg'] = isset($args['wpdiscuz-searched-data-bg']) ? $args['wpdiscuz-searched-data-bg'] : '#C4ECE4';
        $this->options['search_icons_color'] = isset($args['wpdiscuz-search-icons-color']) ? $args['wpdiscuz-search-icons-color'] : '#00B38F';
        $this->options['search_text_color'] = isset($args['wpdiscuz-search-text-color']) ? $args['wpdiscuz-search-text-color'] : '#666666';
        $this->options['search_bg_color'] = isset($args['wpdiscuz-search-bg-color']) ? $args['wpdiscuz-search-bg-color'] : '#FFFFFF';
        $this->options['search_box_border_color'] = isset($args['wpdiscuz-search-box-border-color']) ? $args['wpdiscuz-search-box-border-color'] : '#CDCDCD';
        $this->options['dialog_text_color'] = isset($args['search-dialog-text-color']) ? $args['search-dialog-text-color'] : '#666666';
        $this->options['dialog_bg_color'] = isset($args['search-dialog-bg-color']) ? $args['search-dialog-bg-color'] : '#FFFFFF';
        $this->options['dialog_hover_color'] = isset($args['search-dialog-hover-color']) ? $args['search-dialog-hover-color'] : '#EEEEEE';
        $this->options['pagination_text_color'] = isset($args['search-pagination-text-color']) ? $args['search-pagination-text-color'] : '#666666';
        $this->options['pagination_item_bg_color'] = isset($args['search-pagination-item-bg-color']) ? $args['search-pagination-item-bg-color'] : '#C4ECE4';
        $this->options['widget_post_letters_count'] = isset($_POST['post-letters-count']) && ($v = absint($_POST['post-letters-count'])) ? $v : 60;
        $this->options['widget_post_title_color'] = isset($args['widget-post-title-color']) ? $args['widget-post-title-color'] : '#00B490';
        $this->options['widget_post_content_color'] = isset($args['widget-post-content-color']) ? $args['widget-post-content-color'] : '#686868';
        $this->options['widget_post_author_date_color'] = isset($args['widget-post-author-date-color']) ? $args['widget-post-author-date-color'] : '#CCC';
        $this->options['widget_loadmore_bg'] = isset($args['widget-loadmore-bg']) ? $args['widget-loadmore-bg'] : '#DAF3EE';
        $this->options['widget_loadmore_text'] = isset($args['widget-loadmore-text']) ? $args['widget-loadmore-text'] : '#666666';
        $this->options['widget_loadmore_border'] = isset($args['widget-loadmore-border']) ? $args['widget-loadmore-border'] : '#CCCCCC';
        $this->options['search_result_phrase'] = isset($args['wpdiscuz-search-result-phrase']) ? $args['wpdiscuz-search-result-phrase'] : __('Search result', 'wpdiscuz-search');
        $this->options['search_no_result_phrase'] = isset($args['wpdiscuz-search-no-result-phrase']) ? $args['wpdiscuz-search-no-result-phrase'] : __('No comment found', 'wpdiscuz-search');
        $this->options['search_placeholder'] = isset($args['wpdiscuz-search-placeholder']) ? $args['wpdiscuz-search-placeholder'] : __('Comment search...', 'wpdiscuz-search');
        $this->options['dialog_search_by_all'] = isset($args['wpdiscuz-search-by-all']) ? $args['wpdiscuz-search-by-all'] : __('All', 'wpdiscuz-search');
        $this->options['dialog_search_by_content'] = isset($args['wpdiscuz-search-by-content']) ? $args['wpdiscuz-search-by-content'] : __('Content', 'wpdiscuz-search');
        $this->options['dialog_search_by_author'] = isset($args['wpdiscuz-search-by-author']) ? $args['wpdiscuz-search-by-author'] : __('Author', 'wpdiscuz-search');
        $this->options['dialog_search_by_email'] = isset($args['wpdiscuz-search-by-email']) ? $args['wpdiscuz-search-by-email'] : __('E-mail', 'wpdiscuz-search');
        $this->options['dialog_search_by_custom_fields'] = isset($args['wpdiscuz-search-by-custom-fields']) ? $args['wpdiscuz-search-by-custom-fields'] : __('Custom Fields', 'wpdiscuz-search');

        update_option($this->optionName, $this->options);
        $this->savedOptions = get_option($this->optionName);
    }

    public function initOption() {
        $this->savedOptions = get_option($this->optionName);
    }

    public function commentSearchDynamicStyles() {
        $wpdiscuz = wpDiscuz();
        $style = '<style>';
        $style .= '#wpcomm .wc-footer-left .wc-cta-button a{color:' . $wpdiscuz->optionsSerialized->buttonColor['secondary_button_color'] . '}';
        $style .= '#wpcomm .wc-footer-left .wc-cta-button:hover a{color:#FFFFFF}';
        $style .= '#wpdiscuz-search-form .fas,.wpdiscuz-widget-search-form .fas{color:' . $this->savedOptions['search_icons_color'] . '}';
        $style .= '#wpdiscuz-search-form .wpdiscuz-comm-search{color:' . $this->savedOptions['search_text_color'] . '!important}';
        $style .= '#wpdiscuz-search-form .wpdiscuz-search-box{background-color:' . $this->savedOptions['search_bg_color'] . ';border:1px solid ' . $this->savedOptions['search_box_border_color'] . '}';
        $style .= '#wpdiscuz-search-form .wpdiscuz-search-setting{border:1px solid' . $this->savedOptions['search_box_border_color'] . '}';
        $style .= '.wpdiscuz-search-setting .shearch-arrow{border-color: transparent transparent ' . $this->savedOptions['search_box_border_color'] . '}';
        $style .= '.wpdiscuz-search-setting{background-color:' . $this->savedOptions['dialog_bg_color'] . '!important;}';
        $style .= '.wpdiscuz-search-setting .shearch-arrow-no-border{border-bottom: 9px solid ' . $this->savedOptions['dialog_bg_color'] . '!important}';
        $style .= '.wpdiscuz-search-setting input{color:' . $this->savedOptions['dialog_text_color'] . ' !important}';
        $style .= '.wc-thread-wrapper-search p.wpd-search-result-title{border-bottom:1px solid ' . $this->savedOptions['search_box_border_color'] . '}';
        $style .= '.wpdiscuz-search-setting p:hover{background-color:' . $this->savedOptions['dialog_hover_color'] . '}';
        $style .= '#wpdiscuz-search-pagination .wpdiscuz-search-pagination-item{background-color:' . $this->savedOptions['pagination_item_bg_color'] . ';color:' . $this->savedOptions['pagination_text_color'] . '}';
        $style .= '#wpdiscuz-search-pagination .pagination-current-page{border:2px solid' . $this->savedOptions['pagination_text_color'] . '}';
        $style .= '.wpdiscuz-search-widget-loadmore{background-color:' . $this->savedOptions['widget_loadmore_bg'] . ';color:' . $this->savedOptions['widget_loadmore_text'] . ';border:1px solid' . $this->savedOptions['widget_loadmore_border'] . '}';
        $style .= '.wpdiscuz-searched-data{background-color:' . $this->savedOptions['searched_data_bg'] . '}';
        $style .= '</style>';
        echo $style;
    }

    public function addNewOptions($options) {
        update_option('wpdiscuz_search_version', $options);
        $this->initOption();
    }

    public function getAvailableFields() {
        return array('all', 'content', 'author', 'email', 'custom_fields');
    }

}
