<?php

if (!defined('ABSPATH')) {
    exit();
}

/*
 * Plugin Name: wpDiscuz - Comment Search
 * Description: This plugin for wpDiscuz comment search
 * Version: 1.3.0
 * Author: gVectors Team (A. Chakhoyan, G. Zakaryan, H. Martirosyan)
 * Author URI: https://gvectors.com/
 * Plugin URI: https://gvectors.com/wpdiscuz-comment-search/
 * Text Domain: wpdiscuz-search
 * Domain Path: /languages/
 */

define('PLUGIN_DIR_PATH', dirname(__FILE__));
define('PLUGIN_DIR_NAME', basename(PLUGIN_DIR_PATH));

include_once 'includes/wcsDBManager.php';
include_once 'options/wpDiscuzCommentSearchOption.php';
include_once 'includes/gvt-api-manager.php';
include_once 'wpDiscuzCommentSearchWidget.php';
include_once 'wpDiscuzSearchWidgetWalker.php';
include_once 'wpDiscuzSearchFormWalker.php';

class wpDiscuzCommentSearch {

    private $dbManager;
    private $options;
    private $comment_list_args;
    private $version;
    private $pageCount;

    public function __construct() {
        add_action('plugins_loaded', array(&$this, 'pluginsLoaded'), 13);
    }

    public function pluginsLoaded() {
        if (function_exists('wpDiscuz')) {
            $this->version = get_option('wpdiscuz_search_version');
            if (!$this->version) {
                $this->version = '1.0.0';
            }
            new GVT_API_Manager(__FILE__, 'wpdiscuz_options_page', 'wpdiscuz_option_page');
            $this->dbManager = new SearchDBManager();
            $this->options = new wpDiscuzCommentSearchOption();
            $this->dbManager->setOptions($this->options);
            load_plugin_textdomain('wpdiscuz-search', false, dirname(plugin_basename(__FILE__)) . '/languages/');
            add_action('wp_enqueue_scripts', array(&$this, 'registerFrontScripts'), 105);
            add_action('wpdiscuz_before_comments', array(&$this, 'printSearchForm'), 15, 3);
            add_action('wp_ajax_wpdiscuz_search_data', array(&$this, 'printSearchData'), 15);
            if ($this->options->savedOptions['display_form_for_guests']) {
                add_action('wp_ajax_nopriv_wpdiscuz_search_data', array(&$this, 'printSearchData'), 15);
            }
            add_action('wpdiscuz_check_version', array(&$this, 'pluginNewVersion'), 15);
            add_action('widgets_init', array(&$this, 'wpb_load_widget'), 15);
            add_shortcode('commSearch', array(&$this, 'wpdiscuzCommentSaerchShortcode'), 15);
            add_action('wpdiscuz_addon_tab_title', array($this->options, 'searchOptionTabName'), 15);
            add_action('wpdiscuz_addon_tab_content', array($this->options, 'searchOptionTabContent'), 15);
            add_action('wpdiscuz_save_options', array($this->options, 'searchSaveOptions'), 15);
            add_action('wp_head', array($this->options, 'commentSearchDynamicStyles'), 15);
        } else {
            add_action('admin_notices', array(&$this, 'csRequirements'), 1);
        }
    }

    public function wpdiscuzCommentSaerchShortcode() {
        ob_start();
        include 'search-form-all-comments.php';
        return ob_get_clean();
    }

    public function csRequirements() {
        if (current_user_can('manage_options')) {
            echo "<div class='error'><p>" . __('wpDiscuz Comment Search requires wpDiscuz to be installed!', 'wpdiscuz-search') . "</p></div>";
        }
    }

    public function wpb_load_widget() {
        register_widget('wpDiscuzCommentSearchWidget');
    }

    public function printSearchForm($post, $currentUser, $commentsCount) {
        if (($currentUser->ID || $this->options->savedOptions['display_form_for_guests']) && $commentsCount) {
            include_once 'search-form.php';
        }
    }

    public function printSearchData() {
        $wpdiscuz = wpDiscuz();
        $search_data = isset($_POST['search_data']) ? $_POST['search_data'] : '';
        $isPage = isset($_POST['is_page']) ? intval($_POST['is_page']) : 0;
        $post_id = isset($_POST['post_id']) ? $_POST['post_id'] : 0;
        $searchBy = isset($_POST['searchBy']) ? $_POST['searchBy'] : '';
        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $allcomm = isset($_POST['allComm']) ? $_POST['allComm'] : false;
        $printed_data = isset($_POST['printed_data']) ? $_POST['printed_data'] : '';
        if ($allcomm) {
            $this->SearchAllCommData($search_data, $wpdiscuz, $isPage, $post_id, $searchBy, $page, $allcomm, $printed_data);
        } else {
            $this->SearchCommData($search_data, $wpdiscuz, $isPage, $post_id, $searchBy, $page);
        }
        wp_die();
    }

    private function SearchCommData($search_data, $wpdiscuz, $isPage, $post_id, $searchBy, $page) {
        if ($searchResult = $this->dbManager->searchData($search_data, $searchBy, $post_id)) {
            $per_page = intval($wpdiscuz->optionsSerialized->wordpressCommentPerPage);
            if ($commentIds = array_slice($searchResult, ($page - 1) * $per_page, $per_page)) {
                $comments = get_comments(array('comment__in' => $commentIds));
                $post = get_post($post_id);
                $postsAuthors = ($post->comment_count && (!$wpdiscuz->optionsSerialized->disableProfileURLs || !$wpdiscuz->optionsSerialized->authorTitlesShowHide)) ? $wpdiscuz->dbManager->getPostsAuthors() : array();
                $icons = $wpdiscuz->optionsSerialized->votingButtonsIcon ? explode('|', $wpdiscuz->optionsSerialized->votingButtonsIcon) : array('fa-plus', 'fa-minus');
                $currentUser = $wpdiscuz->helper->getCurrentUser();
                $currentUserEmail = '';
                $isUserLoggedIn = false;
                if (!empty($currentUser->ID)) {
                    $currentUserEmail = $currentUser->user_email;
                    $isUserLoggedIn = true;
                } else if (isset($_COOKIE['comment_author_email_' . COOKIEHASH]) && $_COOKIE['comment_author_email_' . COOKIEHASH]) {
                    $currentUserEmail = urldecode(trim($_COOKIE['comment_author_email_' . COOKIEHASH]));
                }
                $form = $wpdiscuz->wpdiscuzForm->getForm($post_id);
                $high_level_user = current_user_can('moderate_comments');
                $can_stick_or_close = $post->post_author == $currentUser->ID;
                $post_permalink = get_permalink($post_id);
                $args = array(
                    'style' => 'div',
                    'echo' => false,
                    'max_depth' => 1,
                    'reverse_top_level' => true,
                    'post_author' => $post->post_author,
                    'posts_authors' => $postsAuthors,
                    'voting_icons' => array('like' => $icons[0], 'dislike' => $icons[1]),
                    'high_level_user' => $high_level_user,
                    'avatar_trackback' => apply_filters('wpdiscuz_avatar_trackback', plugins_url(WPDISCUZ_DIR_NAME . '/assets/img/trackback.png')),
                    'wpdiscuz_gravatar_size' => apply_filters('wpdiscuz_gravatar_size', 64),
                    'can_stick_or_close' => $can_stick_or_close,
                    'user_follows' => $wpdiscuz->dbManager->getUserFollows($currentUserEmail),
                    'current_user' => $currentUser,
                    'current_user_email' => $currentUserEmail,
                    'is_share_enabled' => false,
                    'post_permalink' => $post_permalink,
                    'can_user_reply' => comments_open($post->ID) && $wpdiscuz->optionsSerialized->wordpressThreadComments && (($form ? $form->isUserCanComment($currentUser, $post_id) : true) || $high_level_user),
                    'can_user_follow' => $wpdiscuz->optionsSerialized->isFollowActive && $isUserLoggedIn && !empty($currentUserEmail),
                    'can_user_vote' => $currentUser->ID || $wpdiscuz->optionsSerialized->isGuestCanVote,
                    'wc_stick_btn' => $wpdiscuz->optionsSerialized->enableStickButton && $high_level_user || $can_stick_or_close ? '<span class="wc_stick_btn wc-cta-button"><span class="wc_stick_text">%s</span></span>' : '',
                    'wc_close_btn' => $wpdiscuz->optionsSerialized->enableCloseButton && $high_level_user || $can_stick_or_close ? '<span class="wc_close_btn wc-cta-button"><span class="wc_close_text">%s</span></span>' : '',
                    'share_buttons' => '',
                    'walker' => new wpDiscuzSearchFormWalker($search_data, $this->options, $wpdiscuz),
                );
                if ($args['is_share_enabled'] = $wpdiscuz->optionsSerialized->isShareEnabled()) {
                    if ($wpdiscuz->optionsSerialized->enableFbShare && $wpdiscuz->optionsSerialized->fbAppID) {
                        $args['share_buttons'] .= '<span class="wc_fb"><i class="fab fa-facebook-f wpf-cta" aria-hidden="true" title="' . $wpdiscuz->optionsSerialized->phrases['wc_share_facebook'] . '"></i></span>';
                    }
                    if ($wpdiscuz->optionsSerialized->enableGoogleShare) {
                        $args['share_buttons'] .= '<a class="wc_go" target="_blank" href="https://plus.google.com/share?url=' . $post_permalink . '" title="' . $wpdiscuz->optionsSerialized->phrases['wc_share_google'] . '"><i class="fab fa-google wpf-cta" aria-hidden="true"></i></a>';
                    }
                    if ($wpdiscuz->optionsSerialized->enableVkShare) {
                        $args['share_buttons'] .= '<a class="wc_vk" target="_blank" href="http://vk.com/share.php?url=' . $post_permalink . '" title="' . $wpdiscuz->optionsSerialized->phrases['wc_share_vk'] . '"><i class="fab fa-vk wpf-cta" aria-hidden="true"></i></a>';
                    }
                    if ($wpdiscuz->optionsSerialized->enableOkShare) {
                        $args['share_buttons'] .= '<a class="wc_ok" target="_blank" href="http://www.odnoklassniki.ru/dk?st.cmd=addShare&st.s=1&st._surl=' . $post_permalink . '" title=""><i class="fab fa-odnoklassniki wpf-cta" aria-hidden="true"></i></a>';
                    }
                }
                $print_data = wp_list_comments($args, $comments);
                $search_results_phrase = isset($this->options->savedOptions['search_result_phrase']) ? $this->options->savedOptions['search_result_phrase'] : __('Search result', 'wpdiscuz-search');
                $html = "<div class='search-result'><p class='wpd-search-result-title'>" . $search_results_phrase . "</p>" . $print_data . "</div>";
                $pageCount = ceil(count($searchResult) / $per_page);
                if ($isPage == 0 && $pageCount > 1) {
                    $html .= '<div id="wpdiscuz-search-pagination">';
                    for ($i = 1; $i <= $pageCount; $i++) {
                        if ($i == 1) {
                            $html .= "<span id='pagination-$i' class='wpdiscuz-search-pagination-item pagination-current-page'>$i</span>";
                        } else {
                            $html .= "<span id='pagination-$i' class='wpdiscuz-search-pagination-item'>$i</span>";
                        }
                    }
                    $html .= "</div>";
                }
                echo $html;
            } else {
                $search_no_results_phrase = isset($this->options->savedOptions['search_no_result_phrase']) ? $this->options->savedOptions['search_no_result_phrase'] : __('No comment found', 'wpdiscuz-search');
                echo "<p class='wpd-search-result-title'>" . $search_no_results_phrase . "</p>";
            }
        } else {
            $search_no_results_phrase = isset($this->options->savedOptions['search_no_result_phrase']) ? $this->options->savedOptions['search_no_result_phrase'] : __('No comment found', 'wpdiscuz-search');
            echo "<p class='wpd-search-result-title'>" . $search_no_results_phrase . "</p>";
        }
    }

    private function SearchAllCommData($search_data, $wpdiscuz, $isPage, $post_id, $searchBy, $page, $allComments, $printed_data) {
        if ($searchResult = $this->dbManager->searchData($search_data, $searchBy, $post_id, $allComments)) {
            $widget_options = get_option('widget_wpdiscuz_search_widget_options');
            $per_page = $widget_options['widget_display_post_count'];
            if ($commentIds = array_slice($searchResult, ($page - 1) * $per_page, $per_page)) {
                $display_avatar = $widget_options['widget_display_avatar'];
                $comments = get_comments(array('comment__in' => $commentIds));
                echo stripslashes($printed_data);
                $args = array(
                    'walker' => new wpDiscuzSearchWidgetWalker($this->options, $search_data, $display_avatar),
                    'reverse_top_level' => true,
                    'style' => 'div',
                    'echo' => false,
                );

                if ($page == 1) {
                    $search_results_phrase = isset($this->options->savedOptions['search_result_phrase']) ? $this->options->savedOptions['search_result_phrase'] : __('Search result', 'wpdiscuz-search');
                    echo '<p class="wpd-search-result-title">' . $search_results_phrase . "</p>";
                }
                echo wp_list_comments($args, $comments);
                $pageCount = ceil(count($searchResult) / $per_page);
                if ($pageCount > 1 && $pageCount > $page) {
                    echo '<button class="wpdiscuz-search-widget-loadmore" data-id="' . $page . '">' . __('Load More', 'wpdiscuz-search') . '</button>';
                }
            } else {
                $search_no_results_phrase = isset($this->options->savedOptions['search_no_result_phrase']) ? $this->options->savedOptions['search_no_result_phrase'] : __('No comment found', 'wpdiscuz-search');
                echo "<p class='wpd-search-result-title'>" . $search_no_results_phrase . "</p>";
            }
        } else {
            $search_no_results_phrase = isset($this->options->savedOptions['search_no_result_phrase']) ? $this->options->savedOptions['search_no_result_phrase'] : __('No comment found', 'wpdiscuz-search');
            echo "<p class='wpd-search-result-title'>" . $search_no_results_phrase . "</p>";
        }
    }

    public function pluginNewVersion() {
        $pluginData = get_plugin_data(__FILE__);
        if (version_compare($pluginData['Version'], $this->version, '>')) {
            $this->options->addNewOptions($this->options->defaultValues());
            update_option('wpdiscuz_search_version', $pluginData['Version']);
        } else {
            update_option('wpdiscuz_search_version', $this->version);
        }
    }

    public function registerFrontScripts() {
        $defaultField = is_array($this->options->savedOptions['search_available_fields']) && in_array($this->options->savedOptions['search_default_field'], $this->options->savedOptions['search_available_fields']) ? $this->options->savedOptions['search_default_field'] : 'all';
        $searchTextMinLength = isset($this->options->savedOptions['search_text_min_length']) && $this->options->savedOptions['search_text_min_length'] ? $this->options->savedOptions['search_text_min_length'] : 3;
        $vars = array('url' => admin_url('admin-ajax.php'), 'searchDefaultField' => $defaultField, 'searchTextMinLength' => $searchTextMinLength);
        wp_register_script('wpdiscuz-search-scripts', plugins_url(PLUGIN_DIR_NAME) . '/assets/js/front.js', array('jquery'), $this->version, false);
        wp_enqueue_script('wpdiscuz-search-scripts');
        wp_localize_script('wpdiscuz-search-scripts', 'search_ajax', $vars);
        wp_register_style('wpdiscuz-search-styles', plugins_url(PLUGIN_DIR_NAME) . '/assets/css/front.css');
        wp_enqueue_style('wpdiscuz-search-styles');
        wp_enqueue_style('wpdiscuz-font-awesome');
    }

}

$wpDiscuzCommentSearch = new wpDiscuzCommentSearch();
