<?php

defined('ABSPATH') or exit;

class CommentTrns {

    private $apiType;
    private $keyValid;
    private $apiKey;
    private $bgColor;
    private $borderColor;
    private $lineColor;
    private $langList;

    public function __construct($settings) {
        $this->apiType = isset($settings['api']) ? $settings['api'] : 'yandex';
        $this->keyValid = isset($settings['valid']) ? $settings['valid'] : '0';
        $this->apiKey = isset($settings['key']) ? $settings['key'] : '';
        $this->bgColor = isset($settings['trns_form_bg_color']) ? $settings['trns_form_bg_color'] : '#FFFFFF';
        $this->borderColor = isset($settings['trns_form_border_color']) ? $settings['trns_form_border_color'] : '#333333';
        $this->lineColor = isset($settings['trns_text_bottom_line_color']) ? $settings['trns_text_bottom_line_color'] : '#CCCCCC';
        $this->langList = langList($this->apiType);
    }

    public function dynamicStyles($optionsSerialized) {
        $style = '.trns-lang{color:' . $optionsSerialized->buttonColor['secondary_button_color'] . ';}'
                . '.trns-lang:hover{color:' . $optionsSerialized->primaryColor . ';}'
                . '.show-all-langs{color:' . $optionsSerialized->buttonColor['secondary_button_color'] . ';}'
                . '.show-all-langs:hover{color:' . $optionsSerialized->primaryColor . ';}'
                . '.single-trns-lang{color:' . $optionsSerialized->buttonColor['secondary_button_color'] . ';}'
                . '.single-trns-lang:hover{color:' . $optionsSerialized->buttonColor['secondary_button_color'] . ';}'
                . '.original-text, .trns-moderate-comment{color:' . $optionsSerialized->buttonColor['secondary_button_color'] . ';}'
                . '#wpcomm .trns-moderate-comments .lang-board{background-color:' . $this->bgColor . ';border:1px solid ' . $this->borderColor . ';}'
                . '#wpcomm .trns-moderate-comments .trns-arrow{border-bottom: 9px solid ' . $this->borderColor . ';}'
                . '#wpcomm .trns-moderate-comments .trns-arrow-no-border{border-bottom:9px solid ' . $this->bgColor . ';}'
                . '.translate-buttons span{border-bottom:1px solid ' . $this->lineColor . ';font-size:' . $optionsSerialized->commentTextSize . ' !important;}'
                . '.all-lang-content{background-color:' . $this->bgColor . ';border:1px solid ' . $this->borderColor . ';border-top:none;}'
                . '.trns-into{border:1px solid ' . $this->borderColor . ';border-bottom:none;background-color:' . $this->bgColor . ';color:' . $optionsSerialized->buttonColor['secondary_button_color'] . ';}'
                . '.trns-into-cc{border-bottom: 1px solid ' . $this->lineColor . ';}'
                . '.trns-into-cl > font{color:' . $optionsSerialized->primaryColor . ';}'
                . '.trns-lang-all{border-left:1px solid ' . $this->lineColor . ';}'
                . 'html[dir=rtl] .trns-lang-all{border-left: none;border-right:1px solid ' . $this->lineColor . ';}';

        echo $style;
    }

    public function addTranslateButton($output, $comment, $user, $current_user) {
        $primary_langs = array();

        if (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
            $accept_lang = ' ' . $_SERVER['HTTP_ACCEPT_LANGUAGE'];
            preg_match_all('#([a-z]{2,3})#s', $accept_lang, $match);
            $langs = array_unique($match[1]);
            foreach ($this->langList as $lang_kay => $lang_val) {
                if (in_array($lang_kay, $langs)) {
                    $primary_langs[$lang_kay] = $lang_val;
                }
            }
        }
        $output .= '<div class="trns-moderate-comments" id="mod-' . esc_attr($comment->comment_ID) . '">'
                . '<span data-lang="none" class="trns-moderate-comment wc-cta-button" id="trns-' . esc_attr($comment->comment_ID) . '"> ' . phrases('Translate') . '</span>'
                . '<div class="lang-board" style="display: none;">'
                . '<span class="trns-arrow"></span>'
                . '<span class="trns-arrow-no-border"></span>'
                . '<div class="translate-buttons">';
        foreach ($primary_langs as $kay => $language) {
            $output .= '<span class="trns-lang trns-primary"  id="' . esc_attr($kay) . '-' . esc_attr($comment->comment_ID) . '">' . esc_attr($language) . '</span>';
        }
        if (isset($_COOKIE["translateLanguage"]) && isset($this->langList[$_COOKIE["translateLanguage"]])) {
            $temp_kay = $_COOKIE["translateLanguage"];
            $output .= '<span class="trns-lang trns-temporary"  id="' . esc_attr($temp_kay) . '-' . esc_attr($comment->comment_ID) . '">' . esc_attr($this->langList[$temp_kay]) . '</span>';
        }
        $output .= '<span class="show-all-langs" id="all-' . esc_attr($comment->comment_ID) . '" title="' . phrases('Show all languages') . '" >' . esc_html(phrases('Show all')) . '</span>'
                . '</div></div></div>'
                . '<span class="original-text wc-cta-button" id="orig-' . esc_attr($comment->comment_ID) . '" style="display: none;"> ' . esc_html(phrases('Original')) . '</span>';

        return $output;
    }

    public function allLangList() {
        global $post;
        $wpdiscuz = wpDiscuz();
        if ($wpdiscuz->helper->isLoadWpdiscuz($post)) {
            $lang_board = '<div class="all-lang-board" style="display:none;">'
                    . '<div class="trns-into">'
                    . '<span class="trns-into-cc"><span class="trns-into-cl"> ' . esc_html(phrases('Translate into')) . ' <font> </font></span>'
                    . '<span class="trns-into-cr"><i class="fas fa-times"></i>'
                    . '</span></span></div>'
                    . '<div class="all-lang-content">'
                    . '<div class="translate-buttons-all">';
            foreach ($this->langList as $kay => $language) {
                $lang_board .= '<span class="trns-lang trns-lang-all trns-' . esc_attr($kay) . '" id="' . esc_attr($kay) . '">' . esc_html($language) . '</span>';
            }
            $lang_board .= '</div></div></div>';

            echo $lang_board;
        }
    }

    public function deleteCommentMeta($messageArray) {
        if ($messageArray['code']) {
            global $wpdb;
            $comment_id = isset($_POST['commentId']) ? intval($_POST['commentId']) : 0;
            $wpdb->query("DELETE FROM {$wpdb->commentmeta} WHERE (`meta_key` = '_comment_leng' OR `meta_key` LIKE '_comment_trns_%') AND `comment_id` = '$comment_id'");
        }
        return $messageArray;
    }

}
