<?php

defined('ABSPATH') or exit;

class TrnsCommentOption {

    private $apiType;
    private $keyValid;
    private $apiKey;
    private $bgColor;
    private $borderColor;
    private $lineColor;
    private $langList;

    public function getTrnsSettings() {
        $settings = get_option('translate_settings');
        $this->apiType = isset($settings['api']) ? $settings['api'] : 'yandex';
        $this->keyValid = isset($settings['valid']) ? $settings['valid'] : '0';
        $this->apiKey = isset($settings['key']) ? $settings['key'] : '';
        $this->bgColor = isset($settings['trns_form_bg_color']) ? $settings['trns_form_bg_color'] : '#FFFFFF';
        $this->borderColor = isset($settings['trns_form_border_color']) ? $settings['trns_form_border_color'] : '#333333';
        $this->lineColor = isset($settings['trns_text_bottom_line_color']) ? $settings['trns_text_bottom_line_color'] : '#CCCCCC';
        $this->langList = langList($this->apiType);
    }

    public function translateOptionTabName() {
        echo '<li>' . __('Translation', 'wpdiscuz-translate') . '</li>';
    }

    public function translateOptionTabContent() {
        $verify = isset($_GET['_trns_nonce']) ? wp_verify_nonce($_GET['_trns_nonce'], 'trns-nonce') : false;
        $trns_options = isset($_GET['trns_options']) ? $_GET['trns_options'] : null;
        if ($verify && $trns_options === 'reset') {
            $this->getTrnsSettings();
            $this->resetOptions();
        } elseif ($verify && $trns_options === 'empty_database') {
            global $wpdb;
            $wpdb->query("DELETE FROM {$wpdb->commentmeta} WHERE `meta_key` = '_comment_leng' OR `meta_key` LIKE '_comment_trns_%'");
        }
        $this->getTrnsSettings();
        require_once 'html-options.php';
    }

    public function translateSaveOptions() {
        $this->getTrnsSettings();

        $options_arr = array(
            'api' => $this->apiType,
            'valid' => $this->keyValid,
            'key' => $this->apiKey,
            'trns_form_bg_color' => $_POST['trns_form_bg_color'] ? $_POST['trns_form_bg_color'] : '#FFFFFF',
            'trns_form_border_color' => $_POST['trns_form_border_color'] ? $_POST['trns_form_border_color'] : '#333333',
            'trns_text_bottom_line_color' => $_POST['trns_text_bottom_line_color'] ? $_POST['trns_text_bottom_line_color'] : '#CCCCCC'
        );
        update_option('translate_settings', $options_arr);

        $phrases_arr = array(
            'Translate' => $_POST['tr_translate'] ? trim($_POST['tr_translate']) : '',
            'Show all' => $_POST['tr_show_all'] ? trim($_POST['tr_show_all']) : '',
            'Show all languages' => $_POST['tr_show_all_languages'] ? trim($_POST['tr_show_all_languages']) : '',
            'Original' => $_POST['tr_original'] ? trim($_POST['tr_original']) : '',
            'Translate into' => $_POST['tr_translate_into'] ? trim($_POST['tr_translate_into']) : '',
            "Can't translate" => $_POST['tr_cant_translate'] ? trim($_POST['tr_cant_translate']) : '',
            "Can't translate this comment" => $_POST['tr_cant_translate_the_comment'] ? trim($_POST['tr_cant_translate_the_comment']) : '',
            "The original coment in" => $_POST['tr_the_original_comment_in'] ? trim($_POST['tr_the_original_comment_in']) : ''
        );
        $phrases_arr = array_diff($phrases_arr, array(''));
        update_option('translate_phrases', $phrases_arr);
    }

    public function resetOptions() {
        $this->getTrnsSettings();
        $options = array('api' => $this->apiType, 'valid' => $this->keyValid, 'key' => $this->apiKey);
        update_option('translate_settings', $options);
        delete_option('translate_phrases');
    }

}
