<?php

/*
 * Plugin Name: wpDiscuz - Front-end Moderation
 * Description: All in one toolset to manage comments on front-end (approve, unapprove, trash, spam, email, move, blacklist, delete)
 * Version: 1.1.4
 * Author: gVectors Team (A. Chakhoyan, G. Zakaryan, H. Martirosyan)
 * Author URI: https://gvectors.com/
 * Plugin URI: https://gvectors.com/wpdiscuz-frontend-moderation/
 * Text Domain: wpdiscuz-frontend-moderation
 * Domain Path: /languages/
 */

if (!defined('ABSPATH')) {
    exit();
}

include_once 'includes/gvt-api-manager.php';
include 'options/class.wpDiscuzFrontEndModerationSettings.php';

class wpDiscuzFrontEndModeration {

    private $settings;
    private $version;
    private $versionOptionName = 'wpdiscuz_fem_plugin_version';
    private $wpdiscuz;

    public function __construct() {
        add_action('plugins_loaded', array(&$this, 'pluginsLoaded'), 13);
    }

    public function pluginsLoaded() {
        if (function_exists('wpDiscuz')) {
            if ($version = get_option($this->versionOptionName)) {
                $this->version = $version;
            } else if ($modVersion = get_option('wpdiscuz_mod_plugin_version')) {
                delete_option('wpdiscuz_mod_plugin_version');
                $this->version = $modVersion;
                add_option($this->versionOptionName, $modVersion);
            } else {
                $this->version = '1.0.0';
            }
            new GVT_API_Manager(__FILE__, 'wpdiscuz_options_page', 'wpdiscuz_option_page');
            load_plugin_textdomain('wpdiscuz-frontend-moderation', false, dirname(plugin_basename(__FILE__)) . '/languages/');
            $this->settings = new wpDiscuzFrontEndModerationSettings();
            $this->wpdiscuz = wpDiscuz();
            add_action('wp_ajax_wfem_move_comment', array(&$this, 'moveComment'), 13);
            add_action('wp_ajax_wfem_get_buttons', array(&$this, 'getButtons'), 13);
            add_action('wp_ajax_wfem_post_titles', array(&$this, 'postTitles'), 13);
            add_filter('wpdiscuz_comment_buttons', array(&$this, 'addButtons'), 13, 4);
            add_action('wp_ajax_wfem_blacklist', array(&$this, 'setToBlacklist'), 13);
            add_action('wpdiscuz_check_version', array(&$this, 'pluginNewVersion'), 13);
            add_filter('wpdiscuz_comments_args', array(&$this, 'commentArguments'), 13);
            add_action('wpdiscuz_front_scripts', array(&$this, 'moderationScriptsStyles'), 13);
            add_action('wp_ajax_wfem_moderate', array(&$this, 'responseToModeration'), 13);
            add_filter('wpdiscuz_js_options', array(&$this, 'moderateScriptArray'), 13);
            add_action('wp_ajax_wfem_delete', array(&$this, 'deleteComment'), 13);
            add_action('wp_ajax_wfem_email', array(&$this, 'sendMail'), 13);
            add_action('wp_footer', array(&$this, 'emailFormAndMoveComment'), 13);
        } else {
            add_action('admin_notices', array(&$this, 'femRequirements'), 1);
        }
    }

    public function femRequirements() {
        if (current_user_can('manage_options')) {
            echo "<div class='error'><p>" . __('wpDiscuz Front-end Moderation requires wpDiscuz to be installed!', 'wpdiscuz-frontend-moderation') . "</p></div>";
        }
    }

    public function pluginNewVersion() {
        $pluginData = get_plugin_data(__FILE__);
        if (version_compare($pluginData['Version'], $this->version, '>')) {
            if (version_compare($this->version, '1.1.3', '<=')) {
                $this->changePrefix();
            }
            $this->settings->initOptions(get_option($this->settings->settingsOptionName));
            $this->settings->initPhrases(get_option($this->settings->phrasesOptionName));
            update_option($this->versionOptionName, $pluginData['Version']);
        } else {
            update_option($this->versionOptionName, $this->version);
        }
    }

    private function changePrefix() {
        if ($modOptions = get_option('wpdiscuz_mod_settings')) {
            delete_option('wpdiscuz_mod_settings');
            $femOptions = array();
            foreach ($modOptions as $key => $modOption) {
                $femOptions[preg_replace('#^mod_([\w\d]+)#is', 'fem_$1', $key)] = $modOption;
            }
            update_option($this->settings->settingsOptionName, $femOptions);
        }
        if ($modPhrases = get_option('wpdiscuz_mod_phrases')) {
            delete_option('wpdiscuz_mod_phrases');
            $femPhrases = array();
            foreach ($modPhrases as $key => $modPhrase) {
                $femPhrases[preg_replace('#^mod_([\w\d]+)#is', 'fem_$1', $key)] = $modPhrase;
            }
            update_option($this->settings->phrasesOptionName, $femPhrases);
        }
    }

    public function moderationScriptsStyles($optionsSerialized) {
        if (is_user_logged_in()) {
            wp_register_script('wpdiscuz-fem-moderation-script', plugins_url('assets/js/script.js', __FILE__), array('jquery'), $this->version, $optionsSerialized->isLoadScriptsInFooter);
            wp_register_style('wpdiscuz-fem-moderation-styles', plugins_url('assets/css/style.css', __FILE__), array(), $this->version, 'all');
            wp_enqueue_script('wpdiscuz-fem-moderation-script');
            wp_enqueue_style('wpdiscuz-fem-moderation-styles');
        }
    }

    public function commentArguments($args) {
        global $post;
        if (!$post) {
            $post = get_post($args['post_id']);
        }
        $current_user = wp_get_current_user();
        if (!$this->wpdiscuz->optionsSerialized->wordpressIsPaginate && (isset($current_user->allcaps['moderate_comments']) || $post->post_author == $current_user->ID)) {
            $args['status'] = 'all';
        }
        return $args;
    }

    public function moderateScriptArray($fem_obj) {
        $fem_obj['confirm_approve'] = esc_html($this->settings->phrases['fem_phrase_confirm']) . ' ' . esc_html($this->settings->phrases['fem_phrase_approved']) . '?';
        $fem_obj['confirm_unapprove'] = esc_html($this->settings->phrases['fem_phrase_confirm']) . ' ' . esc_html($this->settings->phrases['fem_phrase_unapproved']) . '?';
        $fem_obj['confirm_trash'] = esc_html($this->settings->phrases['fem_phrase_confirm']) . ' ' . esc_html($this->settings->phrases['fem_phrase_trashed']) . '?';
        $fem_obj['confirm_spam'] = esc_html($this->settings->phrases['fem_phrase_confirm']) . ' ' . esc_html($this->settings->phrases['fem_phrase_spam_confirm']) . '?';
        $fem_obj['confirm_blacklist'] = esc_html($this->settings->phrases['fem_phrase_confirm_blacklist']) . '?';
        $fem_obj['confirm_delete'] = esc_html($this->settings->phrases['fem_phrase_confirm_delete']) . '?';
        $fem_obj['please_fill'] = esc_html($this->settings->phrases['fem_phrase_please_fill']);
        $fem_obj['choose_post'] = esc_html($this->settings->phrases['fem_phrase_choose_post']);
        return $fem_obj;
    }

    public function emailFormAndMoveComment() {
        global $post;
        if ($this->wpdiscuz->helper->isLoadWpdiscuz($post)) {
            $forms = '<div class="wpdiscuz-fem-email" style="display: none;"></div><div class="wpdiscuz-fem-email-form" style="display: none;">';
            $forms .= '<span class="wpdiscuz-fem-author">' . esc_html($this->settings->phrases['fem_phrase_going_to_mail']) . ' <em></em></span>';
            $forms .= '<i class="fas fa-times"></i><div class="wpdiscuz_clear"></div>';
            $forms .= '<input placeholder="' . esc_attr($this->settings->phrases['fem_phrase_email_subject']) . '" type="text" class="wpdiscuz-fem-subj" value="" />';
            $forms .= '<textarea class="wpdiscuz-fem-msg" placeholder="' . esc_attr($this->settings->phrases['fem_phrase_email_message']) . '"></textarea><br />';
            $forms .= '<div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-send">' . esc_html($this->settings->phrases['fem_phrase_send']) . '</button></div>';
            $forms .= '<input type="hidden" id="wpdiscuz_fem_email_comment_id" /></div>';
            $forms .= '<div class="wpdiscuz-fem-moving" style="display: none;"></div><div class="wpdiscuz-fem-move-form" style="display: none;">';
            $forms .= '<span class="wpdiscuz-fem-author">' . esc_html($this->settings->phrases['fem_phrase_move_comment']) . '<br /><em></em></span>';
            $forms .= '<i class="fas fa-times"></i><div class="wpdiscuz_clear"></div>';
            $forms .= '<div class="wpdiscuz-fem-posts-search"><input type="text" class="wpdiscuz-fem-post" placeholder="' . esc_attr($this->settings->phrases['fem_phrase_post_title']) . '" />';
            $forms .= '<div class="wpdiscuz-fem-posts"></div></div>';
            $forms .= '<div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-move">' . esc_html($this->settings->phrases['fem_phrase_move']) . '</button></div>';
            $forms .= '<input type="hidden" id="wpdiscuz_fem_move_comment_id" /></div>';
            echo $forms;
        }
    }

    public function addButtons($output, $comment, $user, $current_user) {
        if (!empty($current_user->ID)) {
            if (current_user_can('edit_comment', $comment->comment_ID)) {
                $output .= '<div class="wpdiscuz-fem-moderate-comments">';
                $output .= '<span class="wpdiscuz-fem-moderate-comment wc-cta-button">' . esc_html($this->settings->phrases['fem_phrase_moderate']) . '</span>';
                $output .= '</div>';
            } else if ($this->settings->options['fem_user_can_delete'] === 'yes' && $comment->user_id == $current_user->ID && $this->wpdiscuz->helper->isCommentEditable($comment)) {
                $output .= '<span class="wpdiscuz-fem-delete-comment-not-admin wc-cta-button">' . esc_html($this->settings->phrases['fem_phrase_delete']) . '</span>';
            }
        }
        return $output;
    }

    public function getButtons() {
        $response = array('success' => false, 'message' => esc_html($this->settings->phrases['fem_phrase_ops_message']), 'class' => 'wc_self_vote');
        if (!empty($_POST['commentID']) && ($commentID = intval($_POST['commentID'])) && ($comment = get_comment($commentID))) {
            if (current_user_can('edit_comment', $comment->comment_ID)) {
                $response['success'] = true;
                $response['class'] = 'wc_follow_success';
                $response['message'] = '<div class="wpdiscuz-fem-moderation wpdiscuz-fem-hide">';
                $response['message'] .= '<span class="wpdiscuz-fem-arrow"></span>';
                $response['message'] .= '<span class="wpdiscuz-fem-arrow-no-border"></span>';
                $response['message'] .= '<div class="wpdiscuz-fem-moderation-buttons">';
                if ('1' === $comment->comment_approved) {
                    $response['message'] .= '<span class="wpdiscuz-fem-unapprove-comment wpdiscuz-fem-button"><i class="far fa-clock"></i> ' . esc_html($this->settings->phrases['fem_phrase_unapprove']) . '</span>';
                } else {
                    $response['message'] .= '<span class="wpdiscuz-fem-approve-comment"><i class="fas fa-check"></i> ' . esc_html($this->settings->phrases['fem_phrase_approve']) . '</span>';
                }
                $response['message'] .= '<span class="wpdiscuz-fem-trash-comment wpdiscuz-fem-button"><i class="far fa-trash-alt"></i> ' . esc_html($this->settings->phrases['fem_phrase_trash']) . '</span>';
                $response['message'] .= '<span class="wpdiscuz-fem-spam-comment wpdiscuz-fem-button"><i class="fas fa-exclamation-circle"></i> ' . esc_html($this->settings->phrases['fem_phrase_spam']) . '</span>';
                $response['message'] .= '<span class="wpdiscuz-fem-email-comment wpdiscuz-fem-button"><i class="far fa-envelope"></i> ' . esc_html($this->settings->phrases['fem_phrase_email']) . '</span>';
                $response['message'] .= '<span class="wpdiscuz-fem-move-comment wpdiscuz-fem-button"><i class="fas fa-arrow-right"></i> ' . esc_html($this->settings->phrases['fem_phrase_move']) . '</span>';
                $response['message'] .= '<span class="wpdiscuz-fem-blacklist-comment wpdiscuz-fem-button"><i class="fas fa-ban"></i> ' . esc_html($this->settings->phrases['fem_phrase_blacklist']) . '</span>';
                $response['message'] .= '<span class="wpdiscuz-fem-delete-comment wpdiscuz-fem-button"><i class="fas fa-times"></i> ' . esc_html($this->settings->phrases['fem_phrase_delete']) . '</span>';
                $response['message'] .= '</div></div>';
            } else {
                $response['message'] = esc_html($this->settings->phrases['fem_phrase_cant_moderate']);
            }
        }
        wp_die(json_encode($response));
    }

    public function responseToModeration() {
        $response = array('success' => false, 'message' => esc_html($this->settings->phrases['fem_phrase_ops_message']), 'class' => 'wc_self_vote');
        if (!empty($_POST['commentID']) && ($commentID = intval($_POST['commentID'])) && !empty($_POST['status']) && ($status = trim($_POST['status'])) && in_array($status, array('approve', 'hold', 'trash', 'spam'))) {
            if (current_user_can('edit_comment', $commentID) || current_user_can('moderate_comments', $commentID)) {
                if (wp_set_comment_status($commentID, $status)) {
                    $response['success'] = true;
                    $response['class'] = 'wc_follow_success';
                    $response['message'] = '';
                    if ('approve' === $status) {
                        $response['message'] = esc_html($this->settings->phrases['fem_phrase_status_approved']);
                        $response['buttons'] = '<div class="wpdiscuz-fem-moderation wpdiscuz-fem-hide">';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-arrow"></span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-arrow-no-border"></span>';
                        $response['buttons'] .= '<div class="wpdiscuz-fem-moderation-buttons">';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-unapprove-comment wpdiscuz-fem-button"><i class="far fa-clock"></i> ' . esc_html($this->settings->phrases['fem_phrase_unapprove']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-trash-comment wpdiscuz-fem-button"><i class="far fa-trash-alt"></i> ' . esc_html($this->settings->phrases['fem_phrase_trash']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-spam-comment wpdiscuz-fem-button"><i class="fas fa-exclamation-circle"></i> ' . esc_html($this->settings->phrases['fem_phrase_spam']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-email-comment wpdiscuz-fem-button"><i class="far fa-envelope"></i> ' . esc_html($this->settings->phrases['fem_phrase_email']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-move-comment wpdiscuz-fem-button"><i class="fas fa-arrow-right"></i> ' . esc_html($this->settings->phrases['fem_phrase_move']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-blacklist-comment wpdiscuz-fem-button"><i class="fas fa-ban"></i> ' . esc_html($this->settings->phrases['fem_phrase_blacklist']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-delete-comment wpdiscuz-fem-button"><i class="fas fa-times"></i> ' . esc_html($this->settings->phrases['fem_phrase_delete']) . '</span>';
                        $response['buttons'] .= '</div></div>';
                    } else if ('hold' === $status) {
                        $response['class'] = 'wc_held_for_moderate';
                        $response['message'] = esc_html($this->settings->phrases['fem_phrase_status_unapproved']);
                        $response['buttons'] = '<div class="wpdiscuz-fem-moderation wpdiscuz-fem-hide">';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-arrow"></span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-arrow-no-border"></span>';
                        $response['buttons'] .= '<div class="wpdiscuz-fem-moderation-buttons">';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-approve-comment"><i class="fas fa-check"></i> ' . esc_html($this->settings->phrases['fem_phrase_approve']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-trash-comment wpdiscuz-fem-button"><i class="far fa-trash-alt"></i> ' . esc_html($this->settings->phrases['fem_phrase_trash']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-spam-comment wpdiscuz-fem-button"><i class="fas fa-exclamation-circle"></i> ' . esc_html($this->settings->phrases['fem_phrase_spam']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-email-comment wpdiscuz-fem-button"><i class="far fa-envelope"></i> ' . esc_html($this->settings->phrases['fem_phrase_email']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-move-comment wpdiscuz-fem-button"><i class="fas fa-arrow-right"></i> ' . esc_html($this->settings->phrases['fem_phrase_move']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-blacklist-comment wpdiscuz-fem-button"><i class="fas fa-ban"></i> ' . esc_html($this->settings->phrases['fem_phrase_blacklist']) . '</span>';
                        $response['buttons'] .= '<span class="wpdiscuz-fem-delete-comment wpdiscuz-fem-button"><i class="fas fa-times"></i> ' . esc_html($this->settings->phrases['fem_phrase_delete']) . '</span>';
                        $response['buttons'] .= '</div></div>';
                    }
                }
            } else {
                $response['message'] = esc_html($this->settings->phrases['fem_phrase_cant_moderate']);
            }
        }
        wp_die(json_encode($response));
    }

    public function sendMail() {
        $response = '<div class="wpdiscuz-fem-response"><div class="wpdiscuz-fem-information">' . esc_html($this->settings->phrases['fem_phrase_fill_correct_data']) . '</div><div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-ok">' . esc_html($this->settings->phrases['fem_phrase_ok']) . '</button></div></div>';
        if (!empty($_POST['commentID']) && ($commentID = intval($_POST['commentID'])) && isset($_POST['message']) && ($message = trim($_POST['message']))) {
            if (current_user_can('edit_comment', $commentID) || current_user_can('moderate_comments', $commentID)) {
                if ($comment = get_comment($commentID, ARRAY_A)) {
                    $commentLink = get_comment_link($commentID);
                    $subject = !empty($_POST['subject']) ? trim($_POST['subject']) : '';
                    $message = $message . "<br />" . $commentLink;
                    $mail = $comment['comment_author_email'];
                    $headers = array();
                    $content_type = apply_filters('wp_mail_content_type', 'text/html');
                    $from_name = apply_filters('wp_mail_from_name', get_option('blogname'));
                    $from_email = apply_filters('wp_mail_from', get_option('admin_email'));
                    $headers[] = "Content-Type:  $content_type; charset=UTF-8";
                    $headers[] = "From: " . $from_name . " <" . $from_email . "> \r\n";
                    if (wp_mail($mail, $subject, $message, $headers)) {
                        $response = '<div class="wpdiscuz-fem-response"><div class="wpdiscuz-fem-information">' . esc_html($this->settings->phrases['fem_phrase_email_success']) . '</div><div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-ok">' . esc_html($this->settings->phrases['fem_phrase_ok']) . '</button></div></div>';
                    } else {
                        $response = '<div class="wpdiscuz-fem-response"><div class="wpdiscuz-fem-information">' . esc_html($this->settings->phrases['fem_phrase_email_dont_sended']) . '</div><div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-ok">' . esc_html($this->settings->phrases['fem_phrase_ok']) . '</button></div></div>';
                    }
                }
            } else {
                $response = '<div class="wpdiscuz-fem-response"><div class="wpdiscuz-fem-information">' . esc_html($this->settings->phrases['fem_phrase_email_cant_mail']) . '</div><div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-ok">' . esc_html($this->settings->phrases['fem_phrase_ok']) . '</button></div></div>';
            }
        }
        wp_die($response);
    }

    public function postTitles() {
        $response = '';
        if (isset($_POST['postsLike']) && ($postsLike = trim($_POST['postsLike'])) && !empty($_POST['commentPostID']) && ($commentPostID = intval($_POST['commentPostID'])) && strlen($postsLike) > 2) {
            global $wpdb;
            $postsLike = esc_sql($postsLike);
            $wpPostTypes = get_post_types();
            $postTypes = '';
            foreach ($wpPostTypes as $type) {
                if (post_type_supports($type, 'comments')) {
                    $postTypes .= $wpdb->prepare('%s', $type) . ',';
                }
            }
            $postTypes = rtrim($postTypes, ',');
            $sql = "SELECT `ID`, `post_title`, `post_type` FROM `" . $wpdb->posts . "`  
                        WHERE `ID` != $commentPostID AND `post_type` IN ($postTypes) AND `post_status` IN ('publish', 'private') AND `comment_status` = 'open' AND 
                        `post_title` LIKE '%" . $postsLike . "%' ORDER BY `ID` ASC LIMIT 20";
            $posts = $wpdb->get_results($sql, ARRAY_A);
            $length = apply_filters('wfem_title_length', 30);
            $titleMaxLength = intval($length) ? $length : 30;
            foreach ($posts as $post) {
                $postTypeObject = get_post_type_object($post['post_type']);
                $postTypeLabel = $postTypeObject && is_object($postTypeObject) ? $postTypeObject->labels->singular_name : '';
                if (function_exists('mb_strlen')) {
                    if (function_exists('mb_substr')) {
                        $postTitle = mb_strlen($post['post_title']) < $titleMaxLength ? $post['post_title'] : mb_substr($post['post_title'], 0, $titleMaxLength) . '&hellip;';
                    } else {
                        $postTitle = mb_strlen($post['post_title']) < $titleMaxLength ? $post['post_title'] : substr($post['post_title'], 0, $titleMaxLength) . '&hellip;';
                    }
                } else {
                    if (function_exists('mb_substr')) {
                        $postTitle = strlen($post['post_title']) < $titleMaxLength ? $post['post_title'] : mb_substr($post['post_title'], 0, $titleMaxLength) . '&hellip;';
                    } else {
                        $postTitle = strlen($post['post_title']) < $titleMaxLength ? $post['post_title'] : substr($post['post_title'], 0, $titleMaxLength) . '&hellip;';
                    }
                }
                $postTitle .= ' &nbsp;|&nbsp; ID: ' . $post['ID'] . '&nbsp;|&nbsp;';
                $response .= '<div id="fem-post-' . esc_html($post['ID']) . '" class="wpdiscuz-fem-result">';
                $response .= esc_html($postTitle);
                $response .= $postTypeLabel ? __('Type: ', 'wpdiscuz-frontend-moderation') . $postTypeLabel : '';
                $response .= '</div>';
            }
        }
        wp_die($response);
    }

    public function moveComment() {
        $response = '<div class="wpdiscuz-fem-response"><div class="wpdiscuz-fem-information">' . esc_html($this->settings->phrases['fem_phrase_ops_message']) . '</div><div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-ok">' . esc_html($this->settings->phrases['fem_phrase_ok']) . '</button></div></div>';
        if (!empty($_POST['commentID']) && ($commentID = intval($_POST['commentID'])) && !empty($_POST['moveToPost']) && ($moveToPost = intval($_POST['moveToPost']))) {
            if ($comment = get_comment($commentID)) {
                if ($post = get_post($moveToPost)) {
                    if (intval($comment->comment_post_ID) !== intval($post->ID)) {
                        if (current_user_can('edit_comment', $comment->comment_ID) || current_user_can('moderate_comments', $comment->comment_ID)) {
                            if (($post->post_status === 'publish' || $post->post_status === 'private') && comments_open($post->ID) && post_type_supports($post->post_type, 'comments')) {
                                $parentCommentArray = array(
                                    'comment_ID' => $comment->comment_ID,
                                    'comment_post_ID' => $post->ID,
                                    'comment_parent' => 0,
                                );
                                $commentArray = array(
                                    'comment_post_ID' => $post->ID,
                                );
                                if (wp_update_comment($parentCommentArray)) {
                                    $children = array();
                                    $this->wpdiscuz->helperOptimization->getTreeByParentId($comment->comment_ID, $children);
                                    if ($children) {
                                        foreach ($children as $child) {
                                            $commentArray['comment_ID'] = intval($child);
                                            wp_update_comment($commentArray);
                                        }
                                    }
                                    wp_update_comment_count($post->ID);
                                    wp_update_comment_count($comment->comment_post_ID);
                                    $response = '<div class="wpdiscuz-fem-response"><div class="wpdiscuz-fem-information">' . esc_html($this->settings->phrases['fem_phrase_move_response_success']) . '</div><div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-ok">' . esc_html($this->settings->phrases['fem_phrase_ok']) . '</button></div></div>';
                                }
                            }
                        } else {
                            $response = '<div class="wpdiscuz-fem-response"><div class="wpdiscuz-fem-information">' . esc_html($this->settings->phrases['fem_phrase_cant_moderate']) . '</div><div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-ok">' . esc_html($this->settings->phrases['fem_phrase_ok']) . '</button></div></div>';
                        }
                    } else {
                        $response = '<div class="wpdiscuz-fem-response"><div class="wpdiscuz-fem-information">' . esc_html($this->settings->phrases['fem_phrase_fill_correct_data']) . '</div><div class="wpdiscuz-fem-button-align"><button type="button" class="wpdiscuz-fem-ok">' . esc_html($this->settings->phrases['fem_phrase_ok']) . '</button></div></div>';
                    }
                }
            }
        }
        wp_die($response);
    }

    public function setToBlacklist() {
        $response = array('success' => false, 'message' => esc_html($this->settings->phrases['fem_phrase_ops_message']), 'class' => 'wc_self_vote');
        if (!empty($_POST['commentID']) && ($commentID = intval($_POST['commentID']))) {
            if (current_user_can('edit_comment', $commentID) || current_user_can('moderate_comments', $commentID)) {
                if ($comment = get_comment($commentID, ARRAY_A)) {
                    $blacklist = get_option('blacklist_keys');
                    $new_blacklist = $blacklist;
                    if ('' !== $comment['comment_author_email']) {
                        $new_blacklist .= "\n" . $comment['comment_author_email'];
                    }
                    if ('' !== $comment['comment_author_url']) {
                        $new_blacklist .= "\n" . $comment['comment_author_url'];
                    }
                    if ('' !== $comment['comment_author_IP']) {
                        $new_blacklist .= "\n" . $comment['comment_author_IP'];
                    }
                    if ($new_blacklist !== $blacklist) {
                        if (update_option('blacklist_keys', $new_blacklist)) {
                            if (wp_set_comment_status($commentID, 'trash')) {
                                $response['success'] = true;
                                $response['message'] = esc_html($this->settings->phrases['fem_phrase_blacklist_success']);
                                $response['class'] = 'wc_follow_success';
                            }
                        }
                    }
                }
            } else {
                $response['message'] = esc_html($this->settings->phrases['fem_phrase_blacklist_cant_set']);
            }
        }
        wp_die(json_encode($response));
    }

    public function deleteComment() {
        $response = array('success' => false, 'message' => esc_html($this->settings->phrases['fem_phrase_ops_message']), 'class' => 'wc_self_vote');
        if (!empty($_POST['commentID']) && ($commentID = intval($_POST['commentID'])) && !empty($_POST['status']) && 'delete' === $_POST['status']) {
            if ($comment = get_comment($commentID)) {
                $current_user = wp_get_current_user();
                if ((current_user_can('edit_comment', $commentID) || current_user_can('moderate_comments', $commentID)) || ($this->settings->options['fem_user_can_delete'] === 'yes' && !empty($current_user->ID) && $comment->user_id == $current_user->ID && $this->wpdiscuz->helper->isCommentEditable($comment))) {
                    if (wp_delete_comment($commentID, true)) {
                        $response['success'] = true;
                        $response['message'] = '';
                        $response['class'] = 'wc_follow_success';
                    }
                } else {
                    $response['message'] = esc_html($this->settings->phrases['fem_phrase_delete_cant_delete']);
                }
            }
        }
        wp_die(json_encode($response));
    }

}

$wpDiscuzFrontEndModeration = new wpDiscuzFrontEndModeration();
