<?php

class wpDiscuzFrontEndModerationSettings {

    public $settingsOptionName = 'wpdiscuz_fem_settings';
    public $phrasesOptionName = 'wpdiscuz_fem_phrases';
    public $options;
    public $phrases;

    public function __construct() {
        add_action('wpdiscuz_addon_tab_title', array(&$this, 'moderationTabTitle'), 13);
        add_action('wpdiscuz_addon_tab_content', array(&$this, 'moderationTabContent'), 13);
        add_action('wpdiscuz_save_options', array(&$this, 'saveOptionsAndPhrases'), 13);
        add_action('wpdiscuz_dynamic_css', array(&$this, 'dynamicStyles'), 13);
        $this->addOptionsAndPhrases();
        $this->initOptions(get_option($this->settingsOptionName));
        $this->initPhrases(get_option($this->phrasesOptionName));
    }

    public function moderationTabTitle() {
        echo '<li>Front-end Moderation</li>';
    }

    public function moderationTabContent() {
        if (isset($_GET['fem_reset_options']) && current_user_can('manage_options') && 1 == $_GET['fem_reset_options'] && isset($_GET['_fem_nonce']) && wp_verify_nonce($_GET['_fem_nonce'], 'fem_nonce')) {
            $this->initOptions($this->defaultOptions());
            $this->initPhrases($this->defaultPhrases());
        }
        include 'html-options.php';
    }

    public function addOptionsAndPhrases() {
        add_option($this->settingsOptionName, $this->defaultOptions());
        add_option($this->phrasesOptionName, $this->defaultPhrases());
    }

    public function defaultOptions() {
        $options = array(
            'fem_form_bg_color' => '#FFFFFF',
            'fem_form_border_color' => '#333333',
            'fem_text_bottom_line_color' => '#DDDDDD',
            'fem_user_can_delete' => ''
        );
        return $options;
    }

    public function defaultPhrases() {
        $phrases = array(
            'fem_phrase_moderate' => __('Moderate', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_approve' => __('Approve', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_unapprove' => __('Unapprove', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_trash' => __('Trash', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_spam' => __('Spam', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_email' => __('Email', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_move' => __('Move', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_blacklist' => __('Blacklist', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_delete' => __('Delete', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_restore' => __('Restore', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_not_spam' => __('Not Spam', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_email_subject' => __('Email Subject', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_email_message' => __('Enter your message here', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_going_to_mail' => __('You are going to send email to', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_send' => __('Send', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_move_comment' => __('Move Comment', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_post_title' => __('Enter post title...', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_confirm' => __('Are you sure you want to set this comment as', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_unapproved' => __('unapproved', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_approved' => __('approved', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_trashed' => __('trashed', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_spam_confirm' => __('spam', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_confirm_blacklist' => __('Are you sure you want to move this user into blacklist', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_confirm_delete' => __('Are you sure you want to delete this comment', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_status_trashed' => __('Comment successfully trashed', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_status_spam' => __('Comment set as spam successfully', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_status_approved' => __('Comment successfully approved', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_status_unapproved' => __('Comment successfully unapproved', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_ops_message' => __('Ops!!! Something is wrong', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_cant_moderate' => __("You can't moderate this comment", 'wpdiscuz-frontend-moderation'),
            'fem_phrase_blacklist_success' => __('User is added to the blacklist successfully', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_blacklist_ops_message' => __('Ops!!! Something is wrong. Maybe this user already exist in the blacklist', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_blacklist_cant_set' => __('The user cannot be added in blacklist', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_ok' => __('OK', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_move_response_success' => __('Comment successfully moved', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_fill_correct_data' => __('Please fill correct data', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_delete_cant_delete' => __("You can't delete this comment", 'wpdiscuz-frontend-moderation'),
            'fem_phrase_email_cant_mail' => __("You can't send mail to this user", 'wpdiscuz-frontend-moderation'),
            'fem_phrase_email_dont_sended' => __('Message has not been sent', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_email_success' => __('Your message has been sent successfully', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_please_fill' => __('Please fill out the field', 'wpdiscuz-frontend-moderation'),
            'fem_phrase_choose_post' => __('Please choose post', 'wpdiscuz-frontend-moderation'),
        );
        return $phrases;
    }

    public function initOptions($serialize_options) {
        $options = maybe_unserialize($serialize_options);
        $default = $this->defaultOptions();
        foreach ($default as $key => $value) {
            $this->options[$key] = isset($options[$key]) ? esc_sql($options[$key]) : $value;
        }
        update_option($this->settingsOptionName, $this->options);
    }

    public function initPhrases($serialize_phrases) {
        $phrases = maybe_unserialize($serialize_phrases);
        $default = $this->defaultPhrases();
        foreach ($default as $key => $value) {
            $this->phrases[$key] = !empty($phrases[$key]) ? wp_unslash($phrases[$key]) : wp_unslash($value);
        }
        update_option($this->phrasesOptionName, $this->phrases);
    }

    public function saveOptionsAndPhrases($post) {
        $this->initOptions($post);
        $this->initPhrases($post);
    }

    public function dynamicStyles($optionsSerialized) {
        $style = '.wpdiscuz-fem-moderation{background-color:' . esc_html($this->options['fem_form_bg_color']) . ';border:1px solid ' . esc_html($this->options['fem_form_border_color']) . '}';
        $style .= '#wpcomm .wpdiscuz-fem-moderate-comments .wpdiscuz-fem-arrow{border-bottom-color:' . esc_html($this->options['fem_form_border_color']) . '}';
        $style .= '#wpcomm .wpdiscuz-fem-moderate-comments  .wpdiscuz-fem-arrow-no-border{border-bottom-color:' . esc_html($this->options['fem_form_bg_color']) . '}';
        $style .= '.wpdiscuz-fem-moderation-buttons span{border-bottom:1px solid ' . esc_html($this->options['fem_text_bottom_line_color']) . ';font-size:' . esc_html($optionsSerialized->commentTextSize) . ' !important}';
        echo $style;
    }

}
