<?php

/*
 * Plugin Name: wpDiscuz - Media Uploader
 * Description: Better comment system. Wordpress post comments and discussion plugin. Allows your visitors discuss, vote for comments and share.
 * Version: 1.3.8
 * Author: gVectors Team (A. Chakhoyan, G. Zakaryan, H. Martirosyan)
 * Author URI: https://www.gvectors.com/
 * Plugin URI: https://www.gvectors.com/wpdiscuz/
 * Text Domain: wpdiscuz-media-uploader
 * Domain Path: /languages/
 */
if (!defined('ABSPATH')) {
    exit();
}
if (!defined('WPDISCUZ_BOTTOM_TOOLBAR'))
    define('WPDISCUZ_BOTTOM_TOOLBAR', true);

define('WMUDS', DIRECTORY_SEPARATOR);
define('WMU_DIR_PATH', dirname(__FILE__));
define('WMU_DIR_NAME', basename(WMU_DIR_PATH));

require_once 'includes/gvt-api-manager.php';
include_once 'includes/interface.WMUConstants.php';
include_once 'includes/class.WMUDBManager.php';
include_once 'options/class.WMUOptionsSerialized.php';
include_once 'options/class.WMUOptions.php';
include_once 'includes/class.WMUHelper.php';
include_once 'includes/class.WMUCss.php';

class WpdiscuzMediaUploader implements WMUConstants {

    private $optionsSerialized;
    private $options;
    private $css;
    private $helper;
    private $dbManager;
    private $version;
    private $providers;

    public function __construct() {
        add_action('after_setup_theme', array(&$this, 'uploaderRequirments'));
    }

    public function uploaderRequirments() {
        if (function_exists('wpDiscuz')) {
            global $pagenow;
            new GVT_API_Manager(__FILE__, 'wpdiscuz_options_page', 'wpdiscuz_option_page');
            $this->version = get_option(self::OPTION_SLUG_VERSION);
            if (!$this->version) {
                $this->version = '1.0.0';
            }
            $this->optionsSerialized = new WMUOptionsSerialized();
            $this->options = new WMUOptions($this->optionsSerialized);
            $this->css = new WMUCss($this->optionsSerialized);
            $this->helper = new WMUHelper($this->optionsSerialized);
            $this->dbManager = new WMUDBManager();
            load_plugin_textdomain('wpdiscuz-media-uploader', false, WMU_DIR_NAME . '/languages/');
            add_action('wpdiscuz_check_version', array(&$this, 'version'));
            add_action('wpdiscuz_button', array(&$this, 'uploadButtons'), 1, 2);
            add_action('wpdiscuz_button_actions', array(&$this, 'uploadPreview'), 1, 2);
            add_action('wpdiscuz_addon_tab_title', array(&$this->options, 'tabTitle'), 1);
            add_action('wpdiscuz_addon_tab_content', array(&$this->options, 'tabContent'), 1);
            add_action('wpdiscuz_save_options', array(&$this->options, 'saveOptions'));
            add_action('wpdiscuz_dynamic_css', array(&$this->css, 'dynamicCss'));
            add_filter('wpdiscuz_js_options', array(&$this, 'getJSOptions'));
            add_filter('wp_kses_allowed_html', array(&$this, 'filterKses'));
            if ($this->optionsSerialized->wmuIsEmbed) {
                add_filter('embed_defaults', array(&$this, 'embedAttributes'), 10, 2);
                add_filter('wpdiscuz_before_comment_text', array(&$this, 'autoEmbed'), 10, 2);
                // currently unused because of not stable result..
                add_filter('oembed_ttl', array(&$this, 'oembedTtl'), 10, 4);
                if (!$this->optionsSerialized->wmuIsEmbedContent) {
                    add_filter('oembed_providers', array(&$this, 'oembedProviders'), 10, 1);
                    add_filter('oembed_fetch_url', array(&$this, 'oembedFetchUrl'), 10, 3);
                }
            }
            add_filter('wpdiscuz_after_comment_text', array(&$this, 'addAttachedFiles'), 10, 2);

            if ($pagenow == self::PAGE_EDIT_COMMENTS && $this->optionsSerialized->wmuIsShowFilesDashboard) {
                add_filter('comment_text', array(&$this, 'addAttachedFiles'), 10, 2);
            }
            add_filter('wpdiscuz_comment_post', array(&$this, 'postComment'), 10);
            add_filter('wpdiscuz_image_callbacks', array(&$this, 'addLightBox'), 10);

            add_action('admin_post_wmuResetOptions', array(&$this, 'wmuResetOptions'));
            add_action('admin_enqueue_scripts', array(&$this, 'adminOptions'));
            add_action('wpdiscuz_front_scripts', array(&$this, 'addFrontendFiles'));
            add_action('wpdiscuz_add_comment', array(&$this, 'checkUploadFiles'));
            add_action('comment_post', array(&$this, 'uploadFiles'));

            add_action('wp_ajax_removeAttachment', array(&$this, 'removeAttachment'));
            add_action('wp_ajax_nopriv_removeAttachment', array(&$this, 'removeAttachment'));

            add_action('delete_comment', array(&$this->helper, 'deleteLinkedAttachments'), 20);
            add_action('delete_attachment', array(&$this->helper, 'deleteCommentMeta'), 20);
            add_filter('wpdiscuz_privacy_personal_data_export', array(&$this, 'exportPersonalData'), 10, 2);
            add_filter('wpdiscuz_do_export_personal_data', function() {
                return true;
            });
        } else {
            add_action('admin_notices', array(&$this, 'wmuRequirements'), 1);
        }
    }

    public function wmuRequirements() {
        if (current_user_can('manage_options')) {
            echo "<div class='error'><p>" . __('wpDiscuz Media Uploader requires wpDiscuz to be installed!', 'wpdiscuz-media-uploader') . "</p></div>";
        }
    }

    public function version() {
        $pluginData = get_plugin_data(__FILE__);
        if (version_compare($pluginData['Version'], $this->version, '>')) {
            $options = get_option(self::OPTIONS_SLUG);
            $this->addNewOptions($options);
            if ($this->version === '1.0.0') {
                add_option(self::OPTION_SLUG_VERSION, $pluginData['Version']);
            } else {
                update_option(self::OPTION_SLUG_VERSION, $pluginData['Version']);
            }
        }
    }

    /**
     * merge old and new options
     */
    private function addNewOptions($options) {
        $this->optionsSerialized->initOptions($options);
        $newOptions = $this->optionsSerialized->toArray();
        update_option(self::OPTIONS_SLUG, $newOptions);
    }

    public function getJSOptions($args = array()) {
        $iconVideoAudio = plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'img' . WMUDS . 'file-icons' . WMUDS . 'video-audio-type.png');
        $iconFile = plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'img' . WMUDS . 'file-icons' . WMUDS . 'file-type.png');
        $args['url'] = admin_url('admin-ajax.php');
        $args['uploader'] = 1;
        $args['wmuVersion'] = $this->version;
        $args['iconVideoAudio'] = $iconVideoAudio;
        $args['iconFile'] = $iconFile;
        $args['wmuIsLightbox'] = $this->optionsSerialized->wmuIsLightbox ? 1 : 0;
        $args['confirmDelete'] = $this->optionsSerialized->wmuPhrases['wmuPhraseConfirmDelete'];
        $args['msgNotAllowedFiles'] = $this->optionsSerialized->wmuPhrases['msgNotAllowedFiles'];
        $args['msgMaxFileCount'] = $this->optionsSerialized->wmuPhrases['msgMaxFileCount'] . ' ' . $this->optionsSerialized->wmuMaxFileCount;
        $args['msgMaxFileSize'] = $this->optionsSerialized->wmuPhrases['msgMaxFileSize'] . ' ' . $this->optionsSerialized->wmuMaxFileSize . 'MB';
        $args['msgPostMaxSize'] = $this->optionsSerialized->wmuPhrases['msgPostMaxSize'] . ' ' . $this->optionsSerialized->wmuPostMaxSize;
        $args['wmuLazyLoadImages'] = $this->optionsSerialized->wmuLazyLoadImages;
        return $args;
    }

    public function filterKses($context) {
        global $allowedtags;
        $allowedtags['address'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['a'] = array('href' => array(), 'rel' => array(), 'rev' => array(), 'name' => array(), 'target' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(), 'download' => array(), 'hreflang' => array(), 'media' => array(),);
        $allowedtags['abbr'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['acronym'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['area'] = array('alt' => array(), 'coords' => array(), 'href' => array(), 'nohref' => array(), 'shape' => array(), 'target' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['article'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['aside'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['audio'] = array('autoplay' => array(), 'controls' => array(), 'loop' => array(), 'muted' => array(), 'preload' => array(), 'src' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['b'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['bdo'] = array('dir' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['big'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['blockquote'] = array('cite' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(), 'data-width' => array(),);
        $allowedtags['br'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['button'] = array('disabled' => array(), 'name' => array(), 'type' => array(), 'value' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['caption'] = array('align' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['cite'] = array('dir' => array(), 'lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['code'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['col'] = array('align' => array(), 'char' => array(), 'charoff' => array(), 'span' => array(), 'dir' => array(), 'valign' => array(), 'width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['colgroup'] = array('align' => array(), 'char' => array(), 'charoff' => array(), 'span' => array(), 'valign' => array(), 'width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['del'] = array('datetime' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['dd'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['dfn'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['details'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'open' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['div'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['dl'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['dt'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['em'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['fieldset'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['figure'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['figcaption'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['font'] = array('color' => array(), 'face' => array(), 'size' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['footer'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['form'] = array('action' => array(), 'accept' => array(), 'accept-charset' => array(), 'enctype' => array(), 'method' => array(), 'name' => array(), 'target' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['h1'] = array('align' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['h2'] = array('align' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['h3'] = array('align' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['h4'] = array('align' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['h5'] = array('align' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['h6'] = array('align' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['header'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['hgroup'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['hr'] = array('align' => array(), 'noshade' => array(), 'size' => array(), 'width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['i'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['img'] = array('alt' => array(), 'align' => array(), 'border' => array(), 'height' => array(), 'hspace' => array(), 'longdesc' => array(), 'vspace' => array(), 'src' => array(), 'usemap' => array(), 'width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['ins'] = array('datetime' => array(), 'cite' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['kbd'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['label'] = array('for' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['legend'] = array('align' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['li'] = array('align' => array(), 'value' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['map'] = array('name' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['mark'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['menu'] = array('type' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['nav'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['p'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['pre'] = array('width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['q'] = array('cite' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['s'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['samp'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['span'] = array('dir' => array(), 'align' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['section'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['small'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['strike'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['strong'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['sub'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['summary'] = array('align' => array(), 'dir' => array(), 'lang' => array(), 'xml:lang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['sup'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['table'] = array('align' => array(), 'bgcolor' => array(), 'border' => array(), 'cellpadding' => array(), 'cellspacing' => array(), 'dir' => array(), 'rules' => array(), 'summary' => array(), 'width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['tbody'] = array('align' => array(), 'char' => array(), 'charoff' => array(), 'valign' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['td'] = array('abbr' => array(), 'align' => array(), 'axis' => array(), 'bgcolor' => array(), 'char' => array(), 'charoff' => array(), 'colspan' => array(), 'dir' => array(), 'headers' => array(), 'height' => array(), 'nowrap' => array(), 'rowspan' => array(), 'scope' => array(), 'valign' => array(), 'width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['textarea'] = array('cols' => array(), 'rows' => array(), 'disabled' => array(), 'name' => array(), 'readonly' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['tfoot'] = array('align' => array(), 'char' => array(), 'charoff' => array(), 'valign' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['th'] = array('abbr' => array(), 'align' => array(), 'axis' => array(), 'bgcolor' => array(), 'char' => array(), 'charoff' => array(), 'colspan' => array(), 'headers' => array(), 'height' => array(), 'nowrap' => array(), 'rowspan' => array(), 'scope' => array(), 'valign' => array(), 'width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['thead'] = array('align' => array(), 'char' => array(), 'charoff' => array(), 'valign' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['title'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['tr'] = array('align' => array(), 'bgcolor' => array(), 'char' => array(), 'charoff' => array(), 'valign' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['track'] = array('default' => array(), 'kind' => array(), 'label' => array(), 'src' => array(), 'srclang' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['tt'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['u'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['ul'] = array('type' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['ol'] = array('start' => array(), 'type' => array(), 'reversed' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['var'] = array('class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['video'] = array('autoplay' => array(), 'controls' => array(), 'height' => array(), 'loop' => array(), 'muted' => array(), 'poster' => array(), 'preload' => array(), 'src' => array(), 'width' => array(), 'class' => array(), 'id' => array(), 'style' => array(), 'title' => array(), 'role' => array(),);
        $allowedtags['iframe'] = array('src' => array(), 'width' => array(), 'height' => array(), 'allowfullscreen' => array(), 'frameborder' => array(), 'title' => array(), 'id' => array(), 'class' => array(), 'style' => array());
        $allowedtags['script'] = array('async' => array(), 'src' => array());
        $allowedtags = apply_filters('wpdiscuz_media_uploader_kses', $allowedtags);
        return $allowedtags;
    }

    public function oembedFetchUrl($provider, $url, $args) {
        $found = false;
        if ($this->providers && is_array($this->providers)) {
            foreach ($this->providers as $matchmask => $data) {
                list( $providerurl, $regex ) = $data;

                // Turn the asterisk-type provider URLs into regex
                if (!$regex) {
                    $matchmask = '#' . str_replace('___wildcard___', '(.+)', preg_quote(str_replace('*', '___wildcard___', $matchmask), '#')) . '#i';
                    $matchmask = preg_replace('|^#http\\\://|', '#https?\://', $matchmask);
                }

                if (preg_match($matchmask, $url)) {
                    $found = true;
                    break;
                }
            }
        }
        return $found ? $provider : "";
    }

    public function oembedProviders($providers) {
        $this->providers = $providers;
        return $providers;
    }

    public function oembedTtl($time, $url, $attr, $post_ID) {
        global $post, $wpdiscuz;
        if ($wpdiscuz->helper->isLoadWpdiscuz($post)) {
            $key_suffix = md5($url . serialize($attr));
            $cachekey = '_oembed_' . $key_suffix;
            $cachekey_time = '_oembed_time_' . $key_suffix;

            $cache = get_post_meta($post_ID, $cachekey, true);
            // $cache_time = get_post_meta($post_ID, $cachekey_time, true);

            if ('{{unknown}}' === $cache || !$this->optionsSerialized->wmuIsEmbedContent) {
                delete_post_meta($post_ID, $cachekey);
                delete_post_meta($post_ID, $cachekey_time);
                $time = -1;
            }
        }
        return $time;
    }

    public function embedAttributes($attr, $url) {
        $attr['width'] = $this->optionsSerialized->wmuEmbedVideoWidth;
        $attr['height'] = $this->optionsSerialized->wmuEmbedVideoHeight;
        return $attr;
    }

    public function autoEmbed($text, $comment) {
        if ($this->optionsSerialized->wmuIsEmbed) {
            global $post, $wp_embed;
            $post = $comment && $comment->comment_post_ID ? get_post($comment->comment_post_ID) : null;
            if ($post != null) {
                $wp_embed->post_ID = $post->ID;
                $text = $wp_embed->run_shortcode($text);
                $text = $wp_embed->autoembed($text);
                $text = do_shortcode($text);
            }
        }
        return $text;
    }

    public function addAttachedFiles($output, $comment) {
        global $user_ID;
        if ($comment) {
            $attachments = $this->dbManager->getAttachments($comment->comment_ID);
            if ($attachments && is_array($attachments)) {
                $currentUser = get_userdata($user_ID);
                $output .= '<div class="wc-comment-attachments">';
                foreach ($attachments as $attach) {
                    if (empty($attach['meta_value'])) {
                        continue;
                    }
                    $attachIds = explode(',', trim($attach['meta_value'], ','));
                    $count = (count($attachIds) > 1) ? 'multi' : 'single';
                    if (in_array(self::METAKEY_IMAGE, $attach)) {
                        $imgHtml = $this->helper->getAttachedImages($attachIds, $currentUser);
                        $output .= '<div class="wmu-attached-images wmu-count-' . $count . '">' . $imgHtml . '</div>';
                    }
                    if (in_array(self::METAKEY_VIDEO, $attach)) {
                        $videoHtml = $this->helper->getAttachedVideos($attachIds, $currentUser);
                        $output .= '<div class="wmu-attached-videos">' . $videoHtml . '</div>';
                    }
                    if (in_array(self::METAKEY_FILE, $attach)) {
                        $filesHtml = $this->helper->getAttachedFiles($attachIds, $currentUser);
                        $output .= '<div class="wmu-attached-files">' . $filesHtml . '</div>';
                    }
                }
                $output .= '</div>';
            }
        }
        return $output;
    }

    public function postComment($messageArray) {
        $messageArray['callbackFunctions'][] = 'wmuHideAll';
        $messageArray['callbackFunctions'][] = 'wmuImagesInit';
        return $messageArray;
    }

    public function addLightBox($messageArray) {
        $messageArray['callbackFunctions'][] = 'wmuAddLightBox';
        $messageArray['callbackFunctions'][] = 'wmuImagesInit';
        return $messageArray;
    }

    public function wmuResetOptions() {
        if (isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'wmuResetOptions') && isset($_GET['reset']) && intval($_GET['reset']) == 1 && current_user_can('manage_options')) {
            delete_option(self::OPTIONS_SLUG);
            $this->optionsSerialized->addOptions();
            $this->optionsSerialized->initOptions(get_option(self::OPTIONS_SLUG));
        }
        wp_redirect(admin_url(self::PAGE_EDIT_COMMENTS . '?page=' . WpdiscuzCore::PAGE_SETTINGS));
    }

    public function uploadButtons($uniqueId, $currentUser) {
        global $post;
        if ($this->isAllowedPostType($post)) {
            $html = '';
            $isUserLoggedIn = is_user_logged_in();
            if (($this->optionsSerialized->wmuIsGuestAllowed && !$isUserLoggedIn) || ($isUserLoggedIn && $this->helper->canUserUpload($currentUser))) {
                if ($this->helper->isUploadAllowed()) {
                    $html = $this->helper->getUploadButtons($uniqueId);
                }
            }
            echo $html;
        }
    }

    public function uploadPreview($uniqueId, $currentUser) {
        global $post;
        if ($this->isAllowedPostType($post)) {
            $html = '';
            $isUserLoggedIn = is_user_logged_in();
            if (($this->optionsSerialized->wmuIsGuestAllowed && !$isUserLoggedIn) || ($isUserLoggedIn && $this->helper->canUserUpload($currentUser))) {
                if ($this->helper->isUploadAllowed()) {
                    $html = $this->helper->getUploadPreview($uniqueId);
                }
            }
            echo $html;
        }
    }

    public function addFrontendFiles($optionsSerialized) {
        if ($this->optionsSerialized->wmuIsLightbox) {
            wp_register_style('wmu-colorbox-css', plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'third-party' . WMUDS . 'colorbox' . WMUDS . 'colorbox.min.css'));
            wp_enqueue_style('wmu-colorbox-css');
            wp_register_script('wmu-colorbox-js', plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'third-party' . WMUDS . 'colorbox' . WMUDS . 'jquery.colorbox.min.js'), array('jquery'), $this->version, $optionsSerialized->isLoadScriptsInFooter);
            wp_enqueue_script('wmu-colorbox-js');
        }
        wp_register_style('wmu-frontend-css', plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'css' . WMUDS . 'wmu-frontend.css'), null, $this->version);
        wp_enqueue_style('wmu-frontend-css');

        if (is_rtl()) {
            wp_register_style('wmu-frontend-rtl-css', plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'css' . WMUDS . 'wmu-frontend-rtl.css'), null, $this->version);
            wp_enqueue_style('wmu-frontend-rtl-css');
        }

        wp_register_script("wmu-exif-js", plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'third-party' . WMUDS . 'exif' . WMUDS . "exif.min.js"), null, $this->version, $optionsSerialized->isLoadScriptsInFooter);
        wp_enqueue_script("wmu-exif-js");

        $uploadType = $this->optionsSerialized->wmuIsMultiple ? 'multiple' : 'single';
        wp_register_script("wmu-$uploadType-js", plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'js' . WMUDS . "wmu-$uploadType.min.js"), array('jquery'), $this->version, $optionsSerialized->isLoadScriptsInFooter);
        wp_enqueue_script("wmu-$uploadType-js");
    }

    public function adminOptions() {
        $args = $this->getJSOptions();
        if ($this->optionsSerialized->wmuIsLightbox) {
            wp_register_style('wmu-colorbox-css', plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'third-party' . WMUDS . 'colorbox' . WMUDS . 'colorbox.min.css'));
            wp_enqueue_style('wmu-colorbox-css');
            wp_register_script('wmu-colorbox-js', plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'third-party' . WMUDS . 'colorbox' . WMUDS . 'jquery.colorbox.min.js'), array('jquery'));
            wp_enqueue_script('wmu-colorbox-js');
        }

        wp_register_style('wmu-options-css', plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'css' . WMUDS . 'wmu-options.css'), null, $this->version);
        wp_enqueue_style('wmu-options-css');
        wp_register_script('wmu-options-js', plugins_url(WMU_DIR_NAME . WMUDS . 'assets' . WMUDS . 'js' . WMUDS . 'wmu-options.js'), array('jquery'));
        wp_localize_script('wmu-options-js', 'wmuJsObj', $args);
        wp_enqueue_script('wmu-options-js');
    }

    public function checkUploadFiles() {
        $postSize = (int) $_SERVER['CONTENT_LENGTH'];
        $allowedPostSize = intval($this->optionsSerialized->wmuPostMaxSize) * 1024 * 1024;
        $wmuMaxFileCount = $this->optionsSerialized->wmuIsMultiple ? $this->optionsSerialized->wmuMaxFileCount : 1;
        if ($postSize > $allowedPostSize) {
            $msgArray = array('code' => 'msgPostMaxSize');
            wp_die(json_encode($msgArray));
        }

        if ($this->helper->isImageAllowed()) {
            $this->helper->checkFiles($wmuMaxFileCount, 'wmu_images', $this->helper->imageAcceptExt);
        }

        if ($this->helper->isVideoAllowed()) {
            $this->helper->checkFiles($wmuMaxFileCount, 'wmu_videos', $this->helper->videoAcceptExt);
        }

        if ($this->helper->isFileAllowed()) {
            $this->helper->checkFiles($wmuMaxFileCount, 'wmu_files', $this->helper->fileAcceptExt);
        }
    }

    public function uploadFiles($cId) {
        if ($this->helper->isImageAllowed()) {
            foreach ($_FILES['wmu_images']['name'] as $index => $tmpImg) {
                $this->uploadSingleFile('wmu_images', $cId, self::METAKEY_IMAGE, $index, $tmpImg);
            }
        }

        if ($this->helper->isVideoAllowed()) {
            foreach ($_FILES['wmu_videos']['name'] as $index => $tmpVA) {
                $this->uploadSingleFile('wmu_videos', $cId, self::METAKEY_VIDEO, $index, $tmpVA);
            }
        }

        if ($this->helper->isFileAllowed()) {
            foreach ($_FILES['wmu_files']['name'] as $index => $tmpFile) {
                $this->uploadSingleFile('wmu_files', $cId, self::METAKEY_FILE, $index, $tmpFile);
            }
        }
    }

    private function uploadSingleFile($key, $cId, $metaKey, $index, $fName) {
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        $wp_upload_dir = wp_upload_dir();
        $path = $wp_upload_dir['path'] . '/';
        $pathInfo = pathinfo($fName);
        $realFileName = $pathInfo['filename'];
        $ext = $pathInfo['extension'];
        $cleanFileName = sanitize_file_name($realFileName) . '_' . $cId . '_' . ($index + 1) . '.' . strtolower($ext);
        $fileName = $path . $cleanFileName;

        if ($key == 'wmu_images' && in_array($ext, array('jpeg', 'jpg'))) {
            $this->helper->imageFixOrientation($_FILES[$key]['tmp_name'][$index]);
        }

        if (move_uploaded_file($_FILES[$key]['tmp_name'][$index], $fileName)) {
            if ($this->optionsSerialized->wmuAttachToPost && ($comment = get_comment($cId))) {
                $postParent = $comment->comment_post_ID;
            } else {
                $postParent = 0;
            }
            $filetype = wp_check_filetype(basename($fileName), null);
            $attachment = array(
                'guid' => $wp_upload_dir['url'] . '/' . $cleanFileName,
                'post_mime_type' => $filetype['type'],
                'post_title' => preg_replace('/\.[^.]+$/', '', wp_slash($fName)),
                'post_excerpt' => wp_slash($fName),
                'post_content' => '',
                'post_status' => 'inherit',
                'post_parent' => $postParent
            );

            if ($attachId = wp_insert_attachment($attachment, $fileName)) {
                add_filter('intermediate_image_sizes', array(&$this, 'getImagesSizes'));
                $attachData = wp_generate_attachment_metadata($attachId, $fileName);
                wp_update_attachment_metadata($attachId, $attachData);
                update_post_meta($attachId, '_wp_attachment_image_alt', $fName);
                $metaIds = implode(',', get_comment_meta($cId, $metaKey));
                $metaIds .= $attachId;
                if (!update_comment_meta($cId, $metaKey, $metaIds . ',')) {
                    exit('Error on updating comment meta');
                }
            }
        }
    }

    public function removeAttachment() {
        global $user_ID;
        $currentUser = get_userdata($user_ID);
        $msgArray = array();
        $attachmentId = isset($_POST['attachmentId']) ? intval($_POST['attachmentId']) : 0;
        $commentId = isset($_POST['commentId']) ? intval($_POST['commentId']) : 0;
        $metaKey = isset($_POST['metaKey']) ? trim($_POST['metaKey']) : 0;
        $attachment = get_post($attachmentId);
        if ($attachment && $this->helper->canEditAttachments($currentUser, $attachment)) {
            if ($metaKey == 'image') {
                $metaKey = self::METAKEY_IMAGE;
            } else if ($metaKey == 'video') {
                $metaKey = self::METAKEY_VIDEO;
            } else {
                $metaKey = self::METAKEY_FILE;
            }
            if ($attachmentId && $commentId && wp_delete_attachment($attachmentId, true)) {
                $pattern = "#(,|^)$attachmentId,#is";
                $commentMeta = get_comment_meta($commentId, $metaKey, true);
                $commentMeta = preg_replace($pattern, '${1}', $commentMeta);
                if (update_comment_meta($commentId, $metaKey, $commentMeta)) {
                    $msgArray['id'] = $attachmentId;
                }
            }
        }
        wp_die(json_encode($msgArray));
    }

    private function isAllowedPostType($post) {
        return ($post && is_object($post) && isset($post->post_type) && in_array($post->post_type, $this->optionsSerialized->wmuAllowedPostTypes));
    }

    public function getImagesSizes() {
        $sizes = $this->optionsSerialized->wmuImageSizes;
        if ($sizes && is_array($sizes) && !in_array('full', $sizes)) {
            $sizes[] = 'full';
        }

        if (!$sizes) {
            $sizes = array('full');
        }
        return $sizes;
    }

    public function exportPersonalData($data, $commentId) {
        $attachments = $this->dbManager->getAttachments($commentId);
        if ($attachments && is_array($attachments)) {
            foreach ($attachments as $attach) {
                if (empty($attach['meta_value'])) {
                    continue;
                }
                $attachIds = explode(',', trim($attach['meta_value'], ','));
                foreach ($attachIds as $attachId) {
                    if (intval($attachId)) {
                        $url = wp_get_attachment_url($attachId);
                        if (in_array(self::METAKEY_IMAGE, $attach)) {
                            $name = __('Attached Images', 'wpdiscuz-media-uploader');
                        } else if (in_array(self::METAKEY_VIDEO, $attach)) {
                            $name = __('Attached Videos', 'wpdiscuz-media-uploader');
                        } else if (in_array(self::METAKEY_FILE, $attach)) {
                            $name = __('Attached Files', 'wpdiscuz-media-uploader');
                        }
                        $data[] = array('name' => $name, 'value' => $url);
                    }
                }
            }
        }
        return $data;
    }

}

$wpdiscuzMU = new WpdiscuzMediaUploader();
$GLOBALS['wpdiscuz_media_uploader'] = $wpdiscuzMU;
