<?php

if (!defined('ABSPATH')) {
    exit();
}

class WMUOptionsSerialized implements WMUConstants {

    // Server allowed upload max file size
    public $wmuUploadMaxFileSize;
    // Server allowed post max size
    public $wmuPostMaxSize;

    /**
     * allowed image types
     */
    public $wmuImageTypes;

    /**
     * allowed video/audio types
     */
    public $wmuVideoTypes;

    /**
     * allowed file types
     */
    public $wmuFileTypes;

    /**
     * Type - checkbox array
     * Available Values - checked / unchecked
     * Description - enable/disable uploading for post types
     * Default Value - all checked
     */
    public $wmuAllowedPostTypes;

    /**
     * Type - checkbox array
     * Available Values - checked / unchecked
     * Description - enable/disable uploading by users roles
     * Default Value - checked
     */
    public $wmuAllowedRoles;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable image uploading
     * Default Value - checked
     */
    public $wmuIsImagesAllowed;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable video/audio uploading
     * Default Value - checked
     */
    public $wmuIsVideosAllowed;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable files uploading
     * Default Value - checked
     */
    public $wmuIsFilesAllowed;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable embed in comments
     * Default Value - checked
     */
    public $wmuIsEmbed;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable embedded content in comments
     * Default Value - checked
     */
    public $wmuIsEmbedContent;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable guest file uploading
     * Default Value - unchecked
     */
    public $wmuIsGuestAllowed;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable html5 video player
     * Default Value - checked
     */
    public $wmuIsHtml5Video;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable html5 audio player
     * Default Value - checked
     */
    public $wmuIsHtml5Audio;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable lightbox for images
     * Default Value - checked
     */
    public $wmuIsLightbox;

    /**
     * Type - radio
     * Available Values - single / multiple
     * Description - set file uploading type single or multiple
     * Default Value - single
     */
    public $wmuIsMultiple;

    /**
     * Type - input
     * Available Values - integer
     * Description - define how many files can user upload at same time
     * Default Value - 3
     */
    public $wmuMaxFileCount;

    /**
     * Type - input
     * Available Values - integer
     * Description - define maximum size of uploading file in MB's 1 = 1024kb
     * Default Value - 30
     */
    public $wmuMaxFileSize;

    /**
     * Type - input
     * Available Values - integer
     * Description - define the width of video player
     * Default Value - 320
     */
    public $wmuVideoWidth;

    /**
     * Type - input
     * Available Values - integer
     * Description - define the height of video player
     * Default Value - 200
     */
    public $wmuVideoHeight;

    /**
     * Type - input
     * Available Values - integer
     * Description - define the width of video player
     * Default Value - 400
     */
    public $wmuEmbedVideoWidth;

    /**
     * Type - input
     * Available Values - integer
     * Description - define the height of video player
     * Default Value - 300
     */
    public $wmuEmbedVideoHeight;

    /**
     * Type - input
     * Available Values - integer
     * Description - define the width of single image in comment
     * Default Value - 320
     */
    public $wmuSingleImageWidth;

    /**
     * Type - input
     * Available Values - integer
     * Description - define the height of single image in comment
     * Default Value - 200
     */
    public $wmuSingleImageHeight;

    /**
     * Type - input
     * Available Values - integer
     * Description - define the width of image in comment
     * Default Value - 320
     */
    public $wmuImageWidth;

    /**
     * Type - input
     * Available Values - integer
     * Description - define the height of image in comment
     * Default Value - 200
     */
    public $wmuImageHeight;

    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable displaying uploaded files in dashboard comments
     * Default Value - unchecked
     */
    public $wmuIsShowFilesDashboard;

    /**
     * Type - array of phrases
     * WMU Phrases
     */
    public $wmuPhrases;

    /**
     * Type - array of image sizes
     * Description - which image sizes should be generated on uploading
     */
    public $wmuImageSizes;
    
    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable uploaded images lazy loading
     * Default Value - checked
     */
    public $wmuLazyLoadImages;
    
    /**
     * Type - checkbox
     * Available Values - checked / unchecked
     * Description - enable/disable files auto attach to current post
     * Default Value - checked
     */
    public $wmuAttachToPost;

    public function __construct() {
        $this->wmuUploadMaxFileSize = ini_get('upload_max_filesize');
        $this->wmuPostMaxSize = ini_get('post_max_size');
        $this->addOptions();
        $this->initOptions(get_option(self::OPTIONS_SLUG));
    }

    public function addOptions() {
        $options = array(
            'wmuAllowedPostTypes' => $this->getDefaultPostTypes(),
            'wmuAllowedRoles' => $this->getDefaultRoles(),
            'wmuIsImagesAllowed' => 1,
            'wmuIsVideosAllowed' => 1,
            'wmuIsFilesAllowed' => 1,
            'wmuIsEmbed' => 1,
            'wmuIsEmbedContent' => 0,
            'wmuIsGuestAllowed' => 1,
            'wmuIsHtml5Video' => 1,
            'wmuIsHtml5Audio' => 1,
            'wmuIsLightbox' => 1,
            'wmuIsMultiple' => 1,
            'wmuMaxFileCount' => 3,
            'wmuMaxFileSize' => 30,
            'wmuVideoWidth' => 320,
            'wmuVideoHeight' => 200,
            'wmuEmbedVideoWidth' => 400,
            'wmuEmbedVideoHeight' => 300,
            'wmuSingleImageWidth' => 320,
            'wmuSingleImageHeight' => 200,
            'wmuImageWidth' => 90,
            'wmuImageHeight' => 90,
            'wmuIsShowFilesDashboard' => 1,
            'wmuPhrases' => $this->getDefaultPhrases(),
            'wmuImageSizes' => $this->getAllImageSizes(),
            'wmuLazyLoadImages' => 1,
            'wmuAttachToPost' => 1,
        );
        add_option(self::OPTIONS_SLUG, $options, '', 'no');
    }

    public function initOptions($options) {
        $options = maybe_unserialize($options);
        $this->wmuAllowedPostTypes = isset($options['wmuAllowedPostTypes']) ? $options['wmuAllowedPostTypes'] : $this->getDefaultPostTypes();
        $this->wmuAllowedRoles = isset($options['wmuAllowedRoles']) ? $options['wmuAllowedRoles'] : $this->getDefaultRoles();
        $this->wmuIsImagesAllowed = isset($options['wmuIsImagesAllowed']) ? $options['wmuIsImagesAllowed'] : 0;
        $this->wmuIsVideosAllowed = isset($options['wmuIsVideosAllowed']) ? $options['wmuIsVideosAllowed'] : 0;
        $this->wmuIsFilesAllowed = isset($options['wmuIsFilesAllowed']) ? $options['wmuIsFilesAllowed'] : 0;
        $this->wmuIsEmbed = isset($options['wmuIsEmbed']) ? $options['wmuIsEmbed'] : 0;
        $this->wmuIsEmbedContent = isset($options['wmuIsEmbedContent']) ? $options['wmuIsEmbedContent'] : 0;
        $this->wmuIsGuestAllowed = isset($options['wmuIsGuestAllowed']) ? $options['wmuIsGuestAllowed'] : 0;
        $this->wmuIsHtml5Video = isset($options['wmuIsHtml5Video']) ? $options['wmuIsHtml5Video'] : 0;
        $this->wmuIsHtml5Audio = isset($options['wmuIsHtml5Audio']) ? $options['wmuIsHtml5Audio'] : 0;
        $this->wmuIsLightbox = isset($options['wmuIsLightbox']) ? $options['wmuIsLightbox'] : 0;
        $this->wmuIsMultiple = isset($options['wmuIsMultiple']) ? $options['wmuIsMultiple'] : 0;
        $this->wmuMaxFileCount = isset($options['wmuMaxFileCount']) ? $options['wmuMaxFileCount'] : 3;
        $this->wmuMaxFileSize = isset($options['wmuMaxFileSize']) ? $options['wmuMaxFileSize'] : 30;
        $this->wmuVideoWidth = isset($options['wmuVideoWidth']) ? $options['wmuVideoWidth'] : 320;
        $this->wmuVideoHeight = isset($options['wmuVideoHeight']) ? $options['wmuVideoHeight'] : 200;
        $this->wmuEmbedVideoWidth = isset($options['wmuEmbedVideoWidth']) ? $options['wmuEmbedVideoWidth'] : 400;
        $this->wmuEmbedVideoHeight = isset($options['wmuEmbedVideoHeight']) ? $options['wmuEmbedVideoHeight'] : 300;
        $this->wmuSingleImageWidth = isset($options['wmuSingleImageWidth']) ? $options['wmuSingleImageWidth'] : 320;
        $this->wmuSingleImageHeight = isset($options['wmuSingleImageHeight']) ? $options['wmuSingleImageHeight'] : 200;
        $this->wmuImageWidth = isset($options['wmuImageWidth']) ? $options['wmuImageWidth'] : 90;
        $this->wmuImageHeight = isset($options['wmuImageHeight']) ? $options['wmuImageHeight'] : 90;
        $this->wmuIsShowFilesDashboard = isset($options['wmuIsShowFilesDashboard']) ? $options['wmuIsShowFilesDashboard'] : 0;
        $this->wmuPhrases = isset($options['wmuPhrases']) ? $options['wmuPhrases'] : $this->getDefaultPhrases();
        $this->wmuImageSizes = isset($options['wmuImageSizes']) && is_array($options['wmuImageSizes']) && ($sizes = array_filter($options['wmuImageSizes'])) ? $sizes : array();
        $this->wmuLazyLoadImages = isset($options['wmuLazyLoadImages']) && ($lazyLoad = intval($options['wmuLazyLoadImages'])) ? $lazyLoad : 0;
        $this->wmuAttachToPost = isset($options['wmuAttachToPost']) && ($attachToPost = intval($options['wmuAttachToPost'])) ? $attachToPost : 0;
    }

    public function toArray() {
        $options = array(
            'wmuAllowedPostTypes' => $this->wmuAllowedPostTypes,
            'wmuAllowedRoles' => $this->wmuAllowedRoles,
            'wmuIsImagesAllowed' => $this->wmuIsImagesAllowed,
            'wmuIsVideosAllowed' => $this->wmuIsVideosAllowed,
            'wmuIsFilesAllowed' => $this->wmuIsFilesAllowed,
            'wmuIsEmbed' => $this->wmuIsEmbed,
            'wmuIsEmbedContent' => $this->wmuIsEmbedContent,
            'wmuIsGuestAllowed' => $this->wmuIsGuestAllowed,
            'wmuIsHtml5Video' => $this->wmuIsHtml5Video,
            'wmuIsHtml5Audio' => $this->wmuIsHtml5Audio,
            'wmuIsLightbox' => $this->wmuIsLightbox,
            'wmuIsMultiple' => $this->wmuIsMultiple,
            'wmuMaxFileCount' => $this->wmuMaxFileCount,
            'wmuMaxFileSize' => $this->wmuMaxFileSize,
            'wmuVideoWidth' => $this->wmuVideoWidth,
            'wmuVideoHeight' => $this->wmuVideoHeight,
            'wmuEmbedVideoWidth' => $this->wmuEmbedVideoWidth,
            'wmuEmbedVideoHeight' => $this->wmuEmbedVideoHeight,
            'wmuSingleImageWidth' => $this->wmuSingleImageWidth,
            'wmuSingleImageHeight' => $this->wmuSingleImageHeight,
            'wmuImageWidth' => $this->wmuImageWidth,
            'wmuImageHeight' => $this->wmuImageHeight,
            'wmuIsShowFilesDashboard' => $this->wmuIsShowFilesDashboard,
            'wmuPhrases' => $this->wmuPhrases,
            'wmuImageSizes' => $this->wmuImageSizes,
            'wmuLazyLoadImages' => $this->wmuLazyLoadImages,
            'wmuAttachToPost' => $this->wmuAttachToPost,
        );
        return $options;
    }

    public function updateOptions() {
        update_option(self::OPTIONS_SLUG, $this->toArray());
    }

    private function getDefaultRoles() {
        return array('administrator', 'editor', 'author', 'contributor', 'subscriber');
    }

    private function getDefaultPhrases() {
        $phrases = array(
            'wmuPhraseImageFiles' => __('Photo and Image Files', 'wpdiscuz-media-uploader'),
            'wmuPhraseVideoFiles' => __('Audio and Video Files', 'wpdiscuz-media-uploader'),
            'wmuPhraseOtherFiles' => __('Other File Types', 'wpdiscuz-media-uploader'),
            'wmuPhraseClickToAdd' => __('Click to add a file', 'wpdiscuz-media-uploader'),
            'wmuPhraseConfirmDelete' => __('Are you sure you want to delete this attachment?', 'wpdiscuz-media-uploader'),
            'msgNotAllowedFiles' => __('Not allowed file type: ', 'wpdiscuz-media-uploader'),
            'msgMaxFileCount' => __('Maximum number of uploaded files is ', 'wpdiscuz-media-uploader'),
            'msgMaxFileSize' => __('Maximum upload file size is ', 'wpdiscuz-media-uploader'),
            'msgPostMaxSize' => __('Maximum post size is ', 'wpdiscuz-media-uploader'),
        );
        return $phrases;
    }

    private function getDefaultPostTypes() {
        return array('post', 'page', 'attachment');
    }

    public function getAllImageSizes() {
        $sizes = array();
        foreach (get_intermediate_image_sizes() as $_size) {
            if (in_array($_size, array('thumbnail', 'medium', 'medium_large', 'large'))) {
                $sizes[] = $_size;
            }
        }
        return $sizes;
    }

}
