<?php

if (!defined('ABSPATH')) {
    exit();
}

class WOUHelper implements WOUConstants {

    private $dbManager;
    private $options;
    public $onlineUsers;

    public function __construct($dbManager, $options) {
        $this->dbManager = $dbManager;
        $this->options = $options;
        add_action('wp_loaded', array(&$this, 'addCurrentOnlineUser'));
        add_filter('wpdiscuz_after_comment_author', array(&$this, 'statusAfterAuthorName'), 10, 3);
        add_action('transition_comment_status', array(&$this, 'commentStatusOnlineUsers'), 50, 3);
        add_action('comment_post', array(&$this, 'commentPostOnlineUsers'), 50, 3);
    }

    public function addCurrentOnlineUser() {
        $userData = $this->getUserData();
        if ($this->isValidUserdata($userData)) {
            $this->dbManager->addOnlineUser($userData);
        }
        $this->onlineUsers = $this->dbManager->getOnlineUsers($this->options->currentBlogId, $this->options->keepOnlineTime);        
    }

    private function getDefaultData() {
        return array(
            'blog_id' => $this->options->currentBlogId,
            'user_ip' => self::getRealIP(),
            'user_id' => 0,
            'user_email' => '',
            'display_name' => '',
        );
    }

    public function getUserData($email = '', $displayName = '') {
        $wpdiscuz = wpDiscuz();
        $user = WpdiscuzOnlineUsers::$CURRENT_USER;
        $defaults = $this->getDefaultData();
        $args = array(
            'user_id' => $user && $user->ID ? $user->ID : 0,
            'user_email' => $email,
            'display_name' => $displayName,
        );

        if ($wpdiscuz && $wpdiscuz->optionsSerialized->isUserByEmail && $args['user_email']) {
            $user = get_user_by('email', $args['user_email']);
            if ($user && $user->ID) {
                $args['user_id'] = $user->ID;
                $args['user_email'] = $user->user_email;
                $args['display_name'] = $user->display_name ? $user->display_name : $user->user_login;
            }
        }


        if (!$args['user_email'] || !$args['display_name']) {
            if ($user && $user->ID) {
                $args['user_email'] = $args['user_email'] ? $args['user_email'] : $user->user_email;
                if (!$args['display_name']) {
                    $args['display_name'] = $user->display_name ? $user->display_name : $user->user_login;
                }
            } else {
                $cookieData = $this->getDataFromCookie();
                if (!$args['user_email']) {
                    $args['user_email'] = $cookieData['user_email'];
                }
                if (!$args['display_name']) {
                    $args['display_name'] = $cookieData['display_name'];
                }
            }
        }
        $userData = wp_parse_args($args, $defaults);
        $userData['user_email'] = urldecode($userData['user_email']);
        $userData['display_name'] = urldecode($userData['display_name']);
        return $userData;
    }

    public static function getRealIP() {
        if (!empty($_SERVER['HTTP_CLIENT_IP'])) {   //check ip from share internet
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {   //to check ip is pass from proxy
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }
        return $ip == '::1' ? '127.0.0.1' : $ip;
    }

    public function isUserOnline($email) {
        $isOnline = false;
        if ($this->onlineUsers && is_array($this->onlineUsers)) {
            $isOnline = array_key_exists($email, $this->onlineUsers);
        }
        return $isOnline;
    }

    public function statusAfterAuthorName($afterCommentAuthorName, $comment, $user) {
        $email = $user ? $user->user_email : $comment->comment_author_email;
        $userUniqueId = $this->getUniqueId($email);
        if ($this->isUserOnline($userUniqueId)) {
            $statusPhrase = $this->options->phraseOnline;
            $statusClass = 'wou-status-online';
        } else {
            $statusPhrase = $this->options->phraseOffline;
            $statusClass = 'wou-status-offline';
        }

        $phraseContainer = "<span class='wou-status-phrase'>$statusPhrase</span>";
        if (!$statusPhrase) {
            $phraseContainer = '';
            $statusClass .= ' wou-status-only-icon'; 
        }


        $status = "<div class='wou-status $statusClass'>";
        $status .= "<i class='fas fa-circle' aria-hidden='true'></i>";
        $status .= $phraseContainer;
        $status .= "<input type='hidden' class='wou-uuid' value='$userUniqueId'/>";
        $status .= "</div>";
        $afterCommentAuthorName .= $status;
        return $afterCommentAuthorName;
    }

    public function getOnlineUsersUIDS($users = array()) {
        $uuIds = array();
        $onlineUsers = $users ? $users : $this->onlineUsers;
        if ($onlineUsers && is_array($onlineUsers)) {
            foreach ($onlineUsers as $emailKey => $nameValue) {
                $uuIds[] = $emailKey;
            }
        }
        return $uuIds;
    }

    public function getUniqueId($email) {
        return str_rot13($email);
    }

    public function isValidUserdata($userData) {
        return $userData && is_array($userData) && $userData['user_email'] && $userData['display_name'];
    }

    public function commentPostOnlineUsers($commentId, $commentApproved, $commentdata) {
        if ($commentApproved == 1 && $commentdata['comment_author_email'] && $commentdata['comment_author']) {
            $userData = $this->getUserData($commentdata['comment_author_email'], $commentdata['comment_author']);
            if ($this->isValidUserdata($userData)) {
                $email = $this->getUniqueId($commentdata['comment_author_email']);
                $name = $commentdata['comment_author'];
                $this->dbManager->addOnlineUser($userData);
                if (!array_key_exists($email, $this->onlineUsers)) {
                    $this->onlineUsers[$email] = $name;
                }
            }
        }
    }

    public function commentStatusOnlineUsers($newStatus, $oldStatus, $comment) {
        if ($newStatus != $oldStatus) {
            $userData = $this->getUserData($comment->comment_author_email, $comment->comment_author);
            if ($newStatus == 'approved' && $this->isValidUserdata($userData)) {
                $email = $this->getUniqueId($comment->comment_author_email);
                $name = $comment->comment_author;
                $this->dbManager->addOnlineUser($userData);
                if (!array_key_exists($email, $this->onlineUsers)) {
                    $this->onlineUsers[$email] = $name;
                }
            }
        }
    }

    public function getDataFromCookie() {
        $data = array(
            'user_email' => isset($_COOKIE['comment_author_email_' . COOKIEHASH]) && $_COOKIE['comment_author_email_' . COOKIEHASH] ? $_COOKIE['comment_author_email_' . COOKIEHASH] : '',
            'display_name' => isset($_COOKIE['comment_author_' . COOKIEHASH]) && $_COOKIE['comment_author_' . COOKIEHASH] ? $_COOKIE['comment_author_' . COOKIEHASH] : '',
        );
        return $data;
    }

}
