<?php

if (!defined('ABSPATH')) {
    exit();
}

class WOUOptions implements WOUConstants {

    public $checkFrequency; 
    public $checkFrequencyType;
    public $keepOnlineTime;
    public $isShowNotificationPopup;
    public $notificationPopupTimeout;
    public $notificationPopupPosition;
    public $notificationItemBG;
    public $notificationItemTextColor;
    public $onlineStatusColor;
    public $offlineStatusColor;
    public $phraseUserIsOnline;
    public $phraseUserLastComment;
    public $phraseReadMore;
    public $phraseOnline;
    public $phraseOffline;
    public $currentBlogId;

    public function __construct() {
        $this->addOptions();
        $options = maybe_unserialize(get_option(self::OPTION_MAIN));
        $this->initOptions($options);

        add_action('wpdiscuz_save_options', array(&$this, 'saveOptions'));
        add_action('admin_post_wouResetOptions', array(&$this, 'resetOptions'));
        add_action('wpdiscuz_addon_tab_title', array(&$this, 'tabTitle'), 1);
        add_action('wpdiscuz_addon_tab_content', array(&$this, 'tabContent'), 1);
        add_action('admin_head', array(&$this, 'adminDynamicCss'), 1);
        add_action('wpdiscuz_dynamic_css', array(&$this, 'dynamicCss'), 1);
    }

    public function tabTitle() {
        echo '<li>' . __('Online Users', 'wpdiscuz-ou') . '</li>';
    }

    public function tabContent() {
        include_once 'wou-options-html.php';
    }

    public function addOptions() {
        $options = array(
            'checkFrequency' => 60, // seconds
            'checkFrequencyType' => 's', // seconds
            'keepOnlineTime' => 120, // seconds
            'isShowNotificationPopup' => 1,
            'notificationPopupTimeout' => 10, // seconds
            'notificationPopupPosition' => 'bottom-right',
            'notificationItemBG' => '#eee',
            'notificationItemTextColor' => '#333',
            'onlineStatusColor' => '#00b38f',
            'offlineStatusColor' => '#ca3c3c',
            'phraseUserIsOnline' => __('is online', 'wpdiscuz-ou'),
            'phraseUserLastComment' => __('Last Comment', 'wpdiscuz-ou'),
            'phraseReadMore' => __('read this comment &raquo;', 'wpdiscuz-ou'),
            'phraseOnline' => __('Online', 'wpdiscuz-ou'),
            'phraseOffline' => __('Offline', 'wpdiscuz-ou'),
        );
        add_option(self::OPTION_MAIN, $options, '', true);
    }

    public function initOptions($options) {
        $opt = maybe_unserialize($options);
        $this->checkFrequency = isset($opt['checkFrequency']) && ($v = absint($opt['checkFrequency'])) ? $v : 60;
        $this->checkFrequencyType = isset($opt['checkFrequencyType']) && ($v = trim($opt['checkFrequencyType'])) ? $v : 's';
        $this->keepOnlineTime = ($this->checkFrequencyType == 's') ? $this->checkFrequency * 2 : $this->checkFrequency * 60 * 2;
        $this->isShowNotificationPopup = isset($opt['isShowNotificationPopup']) && ($v = absint($opt['isShowNotificationPopup'])) ? $v : 0;
        $this->notificationPopupTimeout = isset($opt['notificationPopupTimeout']) && ($v = absint($opt['notificationPopupTimeout'])) >= 1 ? $v : 10;
        $this->notificationPopupPosition = isset($opt['notificationPopupPosition']) && ($v = trim($opt['notificationPopupPosition'])) ? $v : 'bottom-right';
        $this->notificationItemBG = isset($opt['notificationItemBG']) && ($v = trim($opt['notificationItemBG'])) ? $v : '#eee';
        $this->notificationItemTextColor = isset($opt['notificationItemTextColor']) && ($v = trim($opt['notificationItemTextColor'])) ? $v : '#333';
        $this->onlineStatusColor = isset($opt['onlineStatusColor']) && ($v = trim($opt['onlineStatusColor'])) ? $v : '#00b38f';
        $this->offlineStatusColor = isset($opt['offlineStatusColor']) && ($v = trim($opt['offlineStatusColor'])) ? $v : '#ca3c3c';
        $this->phraseUserIsOnline = isset($opt['phraseUserIsOnline']) && ($v = trim($opt['phraseUserIsOnline'])) ? $v : __('is online', 'wpdiscuz-ou');
        $this->phraseUserLastComment = isset($opt['phraseUserLastComment']) && ($v = trim($opt['phraseUserLastComment'])) ? $v : __('Last Comment', 'wpdiscuz-ou');
        $this->phraseReadMore = isset($opt['phraseReadMore']) && ($v = trim($opt['phraseReadMore'])) ? $v : __('read this comment &raquo;', 'wpdiscuz-ou');
        $this->phraseOnline = isset($opt['phraseOnline']) ? trim($opt['phraseOnline']) : '';
        $this->phraseOffline = isset($opt['phraseOffline']) ? trim($opt['phraseOffline']) : '';
        $this->currentBlogId = get_current_blog_id();
    }

    public function saveOptions() {
        $this->checkFrequency = isset($_POST['checkFrequency']) && ($v = absint($_POST['checkFrequency'])) ? $v : 60;
        $this->checkFrequencyType = isset($_POST['checkFrequencyType']) && ($v = trim($_POST['checkFrequencyType'])) && absint($_POST['checkFrequency']) ? $v : 's';
        $this->isShowNotificationPopup = isset($_POST['isShowNotificationPopup']) && ($v = absint($_POST['isShowNotificationPopup'])) ? $v : 0;
        $this->notificationPopupTimeout = isset($_POST['notificationPopupTimeout']) && ($v = absint($_POST['notificationPopupTimeout'])) ? $v : 10;
        $this->notificationPopupPosition = isset($_POST['notificationPopupPosition']) && ($v = trim($_POST['notificationPopupPosition'])) ? $v : 'bottom-right';
        $this->notificationItemBG = isset($_POST['notificationItemBG']) && ($v = trim($_POST['notificationItemBG'])) ? $v : '#eee';
        $this->notificationItemTextColor = isset($_POST['notificationItemTextColor']) && ($v = trim($_POST['notificationItemTextColor'])) ? $v : '#333';
        $this->onlineStatusColor = isset($_POST['onlineStatusColor']) && ($v = trim($_POST['onlineStatusColor'])) ? $v : '#00b38f';
        $this->offlineStatusColor = isset($_POST['offlineStatusColor']) && ($v = trim($_POST['offlineStatusColor'])) ? $v : '#ca3c3c';
        $this->phraseUserIsOnline = isset($_POST['phraseUserIsOnline']) && ($v = trim($_POST['phraseUserIsOnline'])) ? $v : __('is online', 'wpdiscuz-ou');
        $this->phraseUserLastComment = isset($_POST['phraseUserLastComment']) && ($v = trim($_POST['phraseUserLastComment'])) ? $v : __('Last Comment', 'wpdiscuz-ou');
        $this->phraseReadMore = isset($_POST['phraseReadMore']) && ($v = trim($_POST['phraseReadMore'])) ? $v : __('read this comment &raquo;', 'wpdiscuz-ou');
        $this->phraseOnline = isset($_POST['phraseOnline']) ? trim($_POST['phraseOnline']) : '';
        $this->phraseOffline = isset($_POST['phraseOffline']) ? trim($_POST['phraseOffline']) : '';
        update_option(self::OPTION_MAIN, $this->toArray());
    }

    public function toArray() {
        $options = array(
            'checkFrequency' => $this->checkFrequency,
            'checkFrequencyType' => $this->checkFrequencyType,
            'isShowNotificationPopup' => $this->isShowNotificationPopup,
            'notificationPopupTimeout' => $this->notificationPopupTimeout,
            'notificationPopupPosition' => $this->notificationPopupPosition,
            'notificationItemBG' => $this->notificationItemBG,
            'notificationItemTextColor' => $this->notificationItemTextColor,
            'onlineStatusColor' => $this->onlineStatusColor,
            'offlineStatusColor' => $this->offlineStatusColor,
            'phraseUserIsOnline' => $this->phraseUserIsOnline,
            'phraseUserLastComment' => $this->phraseUserLastComment,
            'phraseReadMore' => $this->phraseReadMore,
            'phraseOnline' => $this->phraseOnline,
            'phraseOffline' => $this->phraseOffline,
        );
        return $options;
    }

    public function dynamicCss($optionsSerialized) {
        $styles = "#wpcomm .wc-comment .wou-status-online{color:{$this->onlineStatusColor};border: 1px solid {$this->onlineStatusColor};}";
        $styles .= "#wpcomm .wou-status-online i{color:{$this->onlineStatusColor};}";
        $styles .= "#wpcomm .wou-status-offline{color:{$this->offlineStatusColor};border: 1px solid {$this->offlineStatusColor};}";
        if ($this->notificationPopupPosition == 'top-left') {
            $styles .= "#wou-notification-popup{top:50px;left:50px;}";
        } else if ($this->notificationPopupPosition == 'top-right') {
            $styles .= "#wou-notification-popup{top:50px;right:50px;}";
        } else if ($this->notificationPopupPosition == 'bottom-left') {
            $styles .= "#wou-notification-popup{bottom:50px;left:50px;}";
        } else {
            $styles .= "#wou-notification-popup{bottom:50px;right:50px;}";
        }
        $styles .= "#wou-notification-popup .wou-notification-item{background:{$this->notificationItemBG};color:{$this->notificationItemTextColor};}";
        echo $styles;
    }

    public function adminDynamicCss() {
        $styles = "<style type='text/css'>";
        $styles .= ".wou-status-online{color:{$this->onlineStatusColor};border: 1px solid {$this->onlineStatusColor};}";
        $styles .= ".wou-status-offline{color:{$this->offlineStatusColor};border: 1px solid {$this->offlineStatusColor};}";
        if ($this->notificationPopupPosition == 'top-left') {
            $styles .= "#wou-notification-popup{top:50px;left:50px;}";
        } else if ($this->notificationPopupPosition == 'top-right') {
            $styles .= "#wou-notification-popup{top:50px;right:50px;}";
        } else if ($this->notificationPopupPosition == 'bottom-left') {
            $styles .= "#wou-notification-popup{bottom:50px;left:50px;}";
        } else {
            $styles .= "#wou-notification-popup{bottom:50px;right:50px;}";
        }
        $styles .= "#wou-notification-popup .wou-notification-item{background:{$this->notificationItemBG};color:{$this->notificationItemTextColor};}";
        $styles .= "</style>";
        echo $styles;
    }

    public function resetOptions() {
        if (isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'wouResetOptions') && current_user_can('manage_options')) {
            delete_option(self::OPTION_MAIN);
            $this->addOptions();
            $this->initOptions(get_option(self::OPTION_MAIN));
        }
        wp_redirect(admin_url('edit-comments.php?page=' . WpdiscuzCore::PAGE_SETTINGS));
    }

}
