<?php

/*
 * Plugin Name: wpDiscuz - Online Users
 * Description: Show online users
 * Version: 1.0.8
 * Author: gVectors Team (H. Martirosyan, G. Zakaryan, A. Chakhoyan)
 * Author URI: https://www.gvectors.com/
 * Plugin URI: https://www.gvectors.com/wpdiscuz/
 * Text Domain: wpdiscuz-ou
 * Domain Path: /languages/
 */
if (!defined('ABSPATH')) {
    exit();
}

define('WOU_DIR_PATH', dirname(__FILE__));
define('WOU_DIR_NAME', basename(WOU_DIR_PATH));

include_once 'includes/gvt-api-manager.php';
include_once 'includes/wou-constans.php';
include_once 'includes/wou-db-managaer.php';
include_once 'includes/wou-helper.php';
include_once 'includes/wou-helper-ajax.php';
include_once 'options/wou-options.php';

class WpdiscuzOnlineUsers implements WOUConstants {

    private $dbManager;
    private $options;
    private $helper;
    private $helperAjax;
    private $version;
    private $checkFrequency;
    public static $IS_USER_LOGGED_IN = false;
    public static $CURRENT_USER = null;

    public function __construct() {
        $this->version = get_option(self::OPTION_VERSION, '1.0.0');
        $this->dbManager = new WOUDBManager();
        register_activation_hook(__FILE__, array($this->dbManager, 'createTables'));
        add_action('admin_notices', array($this, 'requirements'), 1);
        add_action('plugins_loaded', array($this, 'dependencies'), 1);
    }

    public function requirements() {
        if (current_user_can('manage_options')) {
            if (!is_plugin_active('wpdiscuz/class.WpdiscuzCore.php')) {
                echo "<div class='error'><p>" . __('wpDiscuz Online Users requires wpDiscuz to be installed!', 'wpdiscuz-ou') . "</p></div>";
            }
        }
    }

    public function dependencies() {
        if (function_exists('wpDiscuz')) {
            if (is_user_logged_in()) {
                self::$IS_USER_LOGGED_IN = true;
                self::$CURRENT_USER = wp_get_current_user();
            }
            new GVT_API_Manager(__FILE__, 'wpdiscuz_options_page', 'wpdiscuz_option_page');
            $this->options = new WOUOptions();
            $this->helper = new WOUHelper($this->dbManager, $this->options);
            $this->helperAjax = new WOUHelperAjax($this->dbManager, $this->options, $this->helper);
            $this->checkFrequency = ($this->options->checkFrequencyType == 's') ? $this->options->checkFrequency * 1000 : $this->options->checkFrequency * 60 * 1000;
            load_plugin_textdomain('wpdiscuz-ou', false, WOU_DIR_NAME . '/languages/');
            add_action('wpdiscuz_check_version', array(&$this, 'newVersion'));
            add_action('admin_enqueue_scripts', array(&$this, 'backendFiles'));
            add_action('wpdiscuz_front_scripts', array(&$this, 'frontendFiles'));
        }
    }

    public function newVersion() {
        $pluginData = get_plugin_data(__FILE__);
        $newVersion = $pluginData['Version'];
        if (version_compare($newVersion, $this->version, '>')) {
            $options = get_option(self::OPTION_MAIN);
            $this->addNewOptions($options);
            update_option(self::OPTION_VERSION, $newVersion);
            $this->version = $newVersion;

            if ($newVersion == '1.0.5') {
                $this->dbManager->updateUserEmails();
            } else if ($newVersion == '1.0.6') {
                $this->dbManager->removeAll();
            }
        }
    }

    /**
     * merge old and new options
     */
    private function addNewOptions($options) {
        $this->options->initOptions($options);
        $newOptions = $this->options->toArray();
        update_option(self::OPTION_MAIN, $newOptions);
    }

    public function backendFiles() {
        $cookieData = $this->helper->getDataFromCookie();
        $currentUserEmail = $cookieData['user_email'];
        $currentUserName = $cookieData['display_name'];
        if (self::$CURRENT_USER) {
            $currentUserEmail = self::$CURRENT_USER->user_email;
            $currentUserName = self::$CURRENT_USER->display_name ? self::$CURRENT_USER->display_name : self::$CURRENT_USER->user_login;
        }

        $vars = array(
            'wouText_Yes' => __('Yes', 'wpdiscuz-ou'),
            'wouText_No' => __('No', 'wpdiscuz-ou'),
            'msgConfirmResetOptions' => __('Do you really want to reset all options?', 'wpdiscuz-ou'),
            'wouAjaxUrl' => admin_url('admin-ajax.php'),
            'wouCurrentOnlineUsers' => $this->helper->getOnlineUsersUIDS(),
            'wouCurrentUserEmail' => urldecode($currentUserEmail),
            'wouCurrentUserName' => urldecode($currentUserName),
            'wouCookieHash' => COOKIEHASH,
            'wouCheckFrequency' => $this->checkFrequency,
            'wouPhraseStatusOnline' => $this->options->phraseOnline,
            'wouPhraseStatusOffline' => $this->options->phraseOffline,
            'wouIsShowNotificationPopup' => $this->options->isShowNotificationPopup,
            'wouNotificationPopupTimeout' => $this->options->notificationPopupTimeout,
        );

        if (is_rtl()) {
            wp_register_style('wou-rtl-css', plugins_url(WOU_DIR_NAME . '/assets/css/wou-rtl.css'), null, $this->version);
            wp_enqueue_style('wou-rtl-css');
        } else {
            wp_register_style('wou-css', plugins_url(WOU_DIR_NAME . '/assets/css/wou.css'), null, $this->version);
            wp_enqueue_style('wou-css');
        }
        wp_register_script('wou-js', plugins_url(WOU_DIR_NAME . '/assets/js/wou.js'), array('jquery'), $this->version, false);
        wp_enqueue_script('wou-js');
        wp_localize_script('wou-js', 'wouVars', $vars);

        wp_enqueue_style('wou-options-css', plugins_url(WOU_DIR_NAME . '/assets/css/wou-options.css'), null, $this->version);
        wp_register_script('wou-options-js', plugins_url(WOU_DIR_NAME . '/assets/js/wou-options.js'), array('jquery'), $this->version, false);
        wp_enqueue_script('wou-options-js');
        wp_localize_script('wou-options-js', 'wouBackendVars', $vars);
    }

    public function frontendFiles($optionsSerialized) {
        $cookieData = $this->helper->getDataFromCookie();
        $currentUserEmail = $cookieData['user_email'];
        $currentUserName = $cookieData['display_name'];
        if (self::$CURRENT_USER) {
            $currentUserEmail = self::$CURRENT_USER->user_email;
            $currentUserName = self::$CURRENT_USER->display_name ? self::$CURRENT_USER->display_name : self::$CURRENT_USER->user_login;
        }

        $vars = array(
            'wouAjaxUrl' => admin_url('admin-ajax.php'),
            'wouCurrentOnlineUsers' => $this->helper->getOnlineUsersUIDS(),
            'wouCurrentUserEmail' => urldecode($currentUserEmail),
            'wouCurrentUserName' => urldecode($currentUserName),
            'wouCookieHash' => COOKIEHASH,
            'wouCheckFrequency' => $this->checkFrequency,
            'wouPhraseStatusOnline' => $this->options->phraseOnline,
            'wouPhraseStatusOffline' => $this->options->phraseOffline,
            'wouIsShowNotificationPopup' => $this->options->isShowNotificationPopup,
            'wouNotificationPopupTimeout' => $this->options->notificationPopupTimeout,
        );

        if (is_rtl()) {
            wp_register_style('wou-rtl-css', plugins_url(WOU_DIR_NAME . '/assets/css/wou-rtl.css'), null, $this->version);
            wp_enqueue_style('wou-rtl-css');
        } else {
            wp_register_style('wou-css', plugins_url(WOU_DIR_NAME . '/assets/css/wou.css'), null, $this->version);
            wp_enqueue_style('wou-css');
        }
        wp_register_script('wou-js', plugins_url(WOU_DIR_NAME . '/assets/js/wou.js'), array('jquery'), $this->version, $optionsSerialized->isLoadScriptsInFooter);
        wp_enqueue_script('wou-js');
        wp_localize_script('wou-js', 'wouVars', $vars);
    }

}

$wpdiscuzOU = new WpdiscuzOnlineUsers();
