<?php

class wpdPrivatCommentOptions {

    private $pluginOptions;
    private $optionKey = 'wpdiscuz_private_comment';

    public function __construct() {
        $this->add();
        $this->init();
        add_action('wpdiscuz_addon_tab_title', array(&$this, 'tabName'), 15);
        add_action('wpdiscuz_addon_tab_content', array(&$this, 'tabContent'), 15);
        add_action('wpdiscuz_save_options', array(&$this, 'saveOptions'));
    }

    public function tabName() {
        echo '<li>' . __('Private Comments', 'wpdiscuz_private_comment') . '</li>';
    }

    public function tabContent() {
        include_once 'options-tab-content.php';
    }

    public function getDefault() {
        $formIDs = array();
        $forms = get_posts(array('numberposts' => -1,
            'post_type' => 'wpdiscuz_form',
            'post_status' => 'publish'));
        foreach ($forms as $form) {
            $formIDs[] = $form->ID;
        }
        return array('moderator_user_groups' => array('administrator', 'editor'),
            'can_add_private_comment' => array('administrator', 'editor', 'author', 'contributor', 'subscriber'),
            'private_comment_forms' => $formIDs,
            'private_comment_notification_enabled' => 0,
            'private_comment_notification_subject' => __('New Private Comment', 'wpdiscuz_private_comment'),
            'private_comment_notification_message' => __('Hi!<br />New private comment thread is create under "[post-title]" post. <br />Please make sure you\'re logged-in before navigating to the private comment thread. <br /><br />Post URL: [post-url] <br />Comment Thread URL: [comment-url] ', 'wpdiscuz_private_comment'),
            'private_comment_set_private' => 0,
        );
    }

    public function get() {
        return $this->pluginOptions;
    }

    private function add() {
        $defaults = $this->getDefault();
        add_option($this->optionKey, $defaults);
    }

    private function init() {
        $this->pluginOptions = get_option($this->optionKey);
    }

    public function saveOptions() {
        $this->pluginOptions['moderator_user_groups'] = isset($_POST['moderator_user_groups']) ? $_POST['moderator_user_groups'] : array();
        $this->pluginOptions['can_add_private_comment'] = isset($_POST['can_add_private_comment']) ? $_POST['can_add_private_comment'] : array();
        $this->pluginOptions['private_comment_forms'] = isset($_POST['private_comment_forms']) ? $_POST['private_comment_forms'] : array();
        $this->pluginOptions['private_comment_notification_enabled'] = isset($_POST['private_comment_notification_enabled']) ? $_POST['private_comment_notification_enabled'] : 0;
        $this->pluginOptions['private_comment_notification_subject'] = isset($_POST['private_comment_notification_subject']) ? wp_unslash($_POST['private_comment_notification_subject']) : __('New Private Comment', 'wpdiscuz_private_comment');
        $this->pluginOptions['private_comment_notification_message'] = isset($_POST['private_comment_notification_message']) ? wp_unslash($_POST['private_comment_notification_message']) : __('Hi!<br />New private comment thread is create under "[post-title]" post. <br />Please make sure you\'re logged-in before navigating to the private comment thread. <br /><br />Post URL: [post-url] <br />Comment Thread URL: [comment-url] ', 'wpdiscuz_private_comment');
        $this->pluginOptions['can_add_private_comment'][] = 'administrator';
        $this->pluginOptions['moderator_user_groups'][] = 'administrator';
        $this->pluginOptions['private_comment_set_private'] = isset($_POST['private_comment_set_private']) ? $_POST['private_comment_set_private'] : 0;
        update_option($this->optionKey, $this->pluginOptions);
    }

}
