<?php

class ReCaptchaOptions {

    private $optionSlug = 'wpdiscuz_recaptcha';
    private $siteKey;
    private $secretKey;
    public $theme;
    public $lang;
    public $requestMethod;
    private $showCaptchaForMembers;

    public function __construct() {
        $this->initOptions();
    }

    public function optionTabName() {
        echo '<li>' . __('Google reCAPTCHA', 'wpdiscuz_recaptcha') . '</li>';
    }

    public function optionTabContent() {
        $this->initOptions();
        ?>
        <div>
            <h3 style="padding:5px 10px 10px 10px; margin:0px; text-align:right; border-bottom:1px solid #ddd; max-width:240px; margin:0px 0px 20px auto; font-weight:normal;"><?php _e('Google reCAPTCHA Settings', 'wpdiscuz_recaptcha'); ?></h3>
            <p>Please <a href="https://www.google.com/recaptcha/admin" target="_blank" style="font-weight:bold;">register you domain</a> with Google to obtain the Site Key and Secret Key. Then insert those keys in according fields below:</p>
            <table class="wp-list-table widefat plugins" style="margin-top:10px; border:none;">
                <tbody>
                    <tr valign="top">
                        <th scope="row" style="width:55%;">
                            <?php _e('Site Key', 'wpdiscuz_recaptcha'); ?> *
                        </th>
                        <td>
                            <input class="regular-text" type="text" name="wpdiscuz_recaptcha_sitekey" value="<?php echo $this->siteKey ? $this->siteKey : ''; ?>" style="margin:1px;padding:3px 5px; width:90%;"/>  
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <?php _e('Secret Key', 'wpdiscuz_recaptcha'); ?> *
                        </th>
                        <td>   
                            <input class="regular-text" type="text" name="wpdiscuz_recaptcha_secretkey" value="<?php echo $this->secretKey ? $this->secretKey : ''; ?>" style="margin: 1px;padding:3px 5px; width:90%;"/>  
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <?php _e('Theme', 'wpdiscuz_recaptcha'); ?>
                        </th>
                        <td>   
                            <select id="wpdiscuz_recaptcha_theme" name="wpdiscuz_recaptcha_theme">
                                <?php $theme = isset($this->theme) ? $this->theme : 'light'; ?>
                                <option value="light" <?php selected($theme, 'light'); ?>>Light</option>
                                <option value="dark" <?php selected($theme, 'dark'); ?>>Dark</option>
                            </select>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <?php _e('Recaptcha Language', 'wpdiscuz_recaptcha'); ?>
                        </th>
                        <td>
                            <input type="text" name="wpdiscuz_recaptcha_lang" value="<?php echo $this->lang ? $this->lang : ''; ?>" placeholder="<?php _e('Example en', 'wpdiscuz_recaptcha'); ?>" style="margin: 1px;padding:3px 5px; width:70px;"/><br />
                            <a target="blanck" style="font-size: 10px;" href="https://developers.google.com/recaptcha/docs/language"><?php _e('Language codes', 'wpdiscuz_recaptcha'); ?></a>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <?php _e('Request method', 'wpdiscuz_recaptcha'); ?>
                        </th>
                        <td>
                            <select id="wpdiscuz_recaptcha_request_method" name="wpdiscuz_recaptcha_request_method">
                                <?php $request_method = isset($this->requestMethod) ? $this->requestMethod : 'auto'; ?>
                                <option value="auto" <?php selected($request_method, 'auto'); ?>><?php _e('Auto', 'wpdiscuz_recaptcha'); ?></option>
                                <option value="socket" <?php selected($request_method, 'socket'); ?>>SocketPost</option>
                                <option value="curl" <?php selected($request_method, 'curl'); ?>>CurlPost</option>
                                <option value="post" <?php selected($request_method, 'post'); ?>>Post</option>
                            </select>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <?php _e('Show on subscribe form', 'wpdiscuz_recaptcha'); ?>
                        </th>
                        <td>
                            <input type="checkbox" id="isShowOnSubscribeForm" name="isShowOnSubscribeForm" <?php echo $this->isShowOnSubscribeForm ? 'checked="checked"' : ''; ?> />
                        </td>
                    </tr>
                </tbody>
            </table>
        </div>
        <?php
    }

    public function initOptions() {
        $options = get_option($this->optionSlug);
        $this->siteKey = $options['site_key'];
        $this->secretKey = $options['secret_key'];
        $this->theme = $options['theme'];
        $this->lang = $options['lang'];
        $this->requestMethod = isset($options['request_method']) ? $options['request_method'] : 'auto';
        $this->isShowOnSubscribeForm = isset($options['isShowOnSubscribeForm']) ? $options['isShowOnSubscribeForm'] : '';
    }

    public function initParentOptions($dataObject) {
        if ($dataObject instanceof wpdFormAttr\Form) {
            $dataObject->initFormFields();
            $formFields = $dataObject->getFormFields();
            if ($formFields && is_array($formFields)) {
                $captchaOptions = $formFields['wc_captcha'];
                $this->showCaptchaForMembers = $captchaOptions['show_for_users'];
            } 
        }
    }

    public function saveOptions($args) {
        $options = array();
        $options['site_key'] = trim($args['wpdiscuz_recaptcha_sitekey']);
        $options['secret_key'] = trim($args['wpdiscuz_recaptcha_secretkey']);
        $options['theme'] = trim($args['wpdiscuz_recaptcha_theme']);
        $options['lang'] = trim($args['wpdiscuz_recaptcha_lang']);
        $options['request_method'] = isset($args['wpdiscuz_recaptcha_request_method']) ? trim($args['wpdiscuz_recaptcha_request_method']) : 'auto';
        $options['isShowOnSubscribeForm'] = isset($args['isShowOnSubscribeForm']) ? trim($args['isShowOnSubscribeForm']) : '';
        update_option($this->optionSlug, $options);
    }

    public function isShowRecaptcha() {
        $userId = get_current_user_id();
        return ($userId && $this->showCaptchaForMembers) || !$userId;
    }

    public function getSecretKey() {
        return $this->secretKey;
    }

    public function addJSOption($args) {
        $args['wpDiscuzReCaptcha'] = true;
        $args['wpDiscuzReCaptchaSK'] = $this->siteKey;
        $args['wpDiscuzReCaptchaTheme'] = $this->theme;
        $args['wpDiscuzIsShowOnSubscribeForm'] = $this->isShowOnSubscribeForm;
        return $args;
    }

}
