<?php

if (!defined('ABSPATH')) {
    exit();
}

class wpDiscuzFlagHelper {

    private $dbManager;
    private $options;
    private $flaggedComments;

    public function __construct($dbManager, $options) {
        $this->dbManager = $dbManager;
        $this->options = $options;
        add_action('wpdiscuz_before_getcomments', array(&$this, 'initUserFlaggedComments'), 10, 2);
        add_filter('wpdiscuz_after_comment_link', array($this, 'addReportFlag'), 10, 4);
        add_action('wp_footer', array($this, 'showEmailDialog'), 10);
        add_action('admin_post_resetOptions', array(&$this, 'resetOptions'));
        add_action('wpdiscuz_comment_flagged', array(&$this, 'flaggedActions'));
        add_action('wpdiscuz_comment_flagged_email', array(&$this, 'flaggedActionsEmail'), 10, 3);
        add_action('wpdiscuz_add_vote', array($this, 'addVoteActions'), 10, 2);
        add_action('wpdiscuz_update_vote', array($this, 'updateVoteActions'), 10, 3);

        add_filter('admin_comment_types_dropdown', array(&$this, 'addReportedComment'));
    }

    public function initUserFlaggedComments($commentsArgs, $currentUser) {
        $postId = $commentsArgs && isset($commentsArgs['post_id']) ? intval($commentsArgs['post_id']) : 0;
        $userId = $currentUser && isset($currentUser->ID) ? intval($currentUser->ID) : 0;
        $userIp = self::getRealIPAddr();
        $this->flaggedComments = $this->dbManager->getFlaggedComments($postId, $userId, $userIp);
    }

    public function addReportFlag($output, $comment, $user, $currentUser) {
        $notAllowedCommentTypes = apply_filters('wpdiscuz_rf_exclude_comment_types', array(WpdiscuzCore::WPDISCUZ_STICKY_COMMENT));
        if ($this->options->showFlag && !in_array($comment->comment_type, $notAllowedCommentTypes) && !absint($comment->comment_karma)) {
            if ($currentUser && isset($currentUser->ID) && $currentUser->ID) {
                if ($this->options->sendFlagMessage) {
                    $title = $this->options->flagTitleOn;
                    $class = 'comment_flag_modal';
                } else {
                    $title = $this->options->flagTitleOff;
                    $class = '';
                }

                $style = "";
                if ($this->flaggedComments && is_array($this->flaggedComments) && in_array($comment->comment_ID, $this->flaggedComments)) {
                    $wpdiscuz = wpDiscuz();
                    $style = "color:{$wpdiscuz->optionsSerialized->primaryColor};";
                    $class .= ' fc_flagged';
                }

                $output .= "<div style='$style' class='comment_flag $class wc_flag_tooltipster wpf-cta wpd-field-desc' id='fc_{$comment->comment_ID}'>";
                $output .= "<i class='fas fa-flag'></i>";
                $output .= "<span>$title</span>";
                $output .= "</div>";
            } else if ($this->options->guestToFlag) {
                if ($this->options->sendFlagMessageGuest) {
                    $title = $this->options->flagTitleOn;
                    $class = 'comment_flag_modal';
                } else {
                    $title = $this->options->flagTitleOff;
                    $class = '';
                }

                $style = "";
                if ($this->flaggedComments && is_array($this->flaggedComments) && in_array($comment->comment_ID, $this->flaggedComments)) {
                    $wpdiscuz = wpDiscuz();
                    $style = "color:{$wpdiscuz->optionsSerialized->primaryColor};";
                    $class .= ' fc_flagged';
                }

                $output .= "<div style='$style' class='comment_flag $class wc_flag_tooltipster wpf-cta wpd-field-desc' id='fc_{$comment->comment_ID}'>";
                $output .= "<i class='fas fa-flag'></i>";
                $output .= "<span>$title</span>";
                $output .= "</div>";
            }
        }
        return $output;
    }

    public function showEmailDialog() {
        global $post;
        $wpdiscuz = wpDiscuz();
        if ($wpdiscuz->helper->isLoadWpdiscuz($post)) {
            $isUserLoggedIn = is_user_logged_in();
            if (($this->options->sendFlagMessage && $isUserLoggedIn) || ($this->options->sendFlagMessageGuest && !$isUserLoggedIn)) {
                $currentUser = wp_get_current_user();
                $userEmail = $currentUser && isset($currentUser->ID) && $currentUser->ID ? $currentUser->user_email : 'anonymous_' . uniqid() . '@example.com';
                include_once WPDISCUZ_FLAG_DIR_PATH . '/view/form-html.php';
            }
        }
    }

    public function addReportedComment($args) {
        $args['wpdiscuz_reported'] = __('Reported', 'wpdiscuz_fc');
        return $args;
    }

    public function flaggedActions($commentId) {
        $count = $this->dbManager->getFlaggedCount($commentId);
        $type = 'flaggings';
        if ($count >= $this->options->flagCount) {
            if ($this->dbManager->moveFlaggedComments($commentId, $this->options->autoModerateCommentType)) {
                if ($this->options->notifyAdmin == 1) {
                    if ($this->dbManager->updateCommentType($commentId)) {
                        $comment = get_comment($commentId);
                        $this->getEmailData($comment, $type);
                    }
                } else {
                    $this->dbManager->updateCommentType($commentId);
                }
            }
        }
    }

    public function flaggedActionsEmail($commentId, $report, $message) {
        $comment = get_comment($commentId);
        $currentUser = wp_get_current_user();

        if ($currentUser && isset($currentUser->ID) && $currentUser->ID) {
            $userLogin = $currentUser->user_login;
            $userEmail = $currentUser->user_email;
        } else {
            $userLogin = __('Guest', 'wpdiscuz_fc');
            $userEmail = apply_filters('wpdiscuz_flagged_guest_email', 'guest@example.com');
        }

        $postTitle = get_the_title($comment->comment_post_ID);
        $postName = get_the_permalink($comment->comment_post_ID);
        $commentInfo = $comment->comment_approved === '1' ? '<a href="' . get_comment_link($comment) . '"  target="_blank">' . get_comment_link($commentId) . '</a>' : $comment->comment_content;
        $subject = $this->options->flagedMailSubject;
        $to = $this->options->flaggedMailTo;
        $body = strtr($this->options->flagedMailMessage, array(
            '[userInfo]' => $userLogin,
            '[reason]' => $report,
            '[message]' => $message,
            '[postTitle]' => '<a href="' . $postName . ' target="_blank">' . $postTitle . '</a>',
            '[commentInfo]' => $commentInfo));


        $from_name = apply_filters('wp_mail_from_name', $userLogin);
        $from_email = apply_filters('wp_mail_from', $userEmail);
        $headers[] = 'From: ' . $from_name . ' <' . $from_email . '>';
        $sent = $this->sendEmail($to, $subject, $body, $headers);
    }

    public function addVoteActions($voteType, $comment) {
        $wpdiscuz = wpDiscuz();
        $voteCount = intval(get_comment_meta($comment->comment_ID, $wpdiscuz::META_KEY_VOTES, true));
        $temp = ($t = intval(get_comment_meta($comment->comment_ID, FC_KEY, true))) ? $t : 1;
        $type = 'down votes';
        $count = intval($this->options->voteCount) * -1;
        if ($voteCount / $temp <= $count) {
            if ($this->dbManager->moveVotedComments($comment->comment_ID, $this->options->autoModerateCommentType)) {
                if ($this->options->notifyAdmin == 1) {
                    if ($this->dbManager->updateCommentType($comment->comment_ID)) {
                        $this->getEmailData($comment, $type);
                    }
                } else {
                    $this->dbManager->updateCommentType($comment->comment_ID);
                }
            }
        }
    }

    public function updateVoteActions($voteType, $isUserVoted, $comment) {
        $wpdiscuz = wpDiscuz();
        $voteCount = intval(get_comment_meta($comment->comment_ID, $wpdiscuz::META_KEY_VOTES, true));
        $temp = ($t = intval(get_comment_meta($comment->comment_ID, FC_KEY, true))) ? $t : 1;
        $type = 'down votes';
        $count = intval($this->options->voteCount) * -1;
        if ($voteCount / $temp <= $count) {
            if ($this->dbManager->moveVotedComments($comment->comment_ID, $this->options->autoModerateCommentType)) {
                if ($this->options->notifyAdmin == 1) {
                    if ($this->dbManager->updateCommentType($comment->comment_ID)) {
                        $this->getEmailData($comment, $type);
                    }
                } else {
                    $this->dbManager->updateCommentType($comment->comment_ID);
                }
            }
        }
    }

    public function getEmailData($comment, $type) {
        $subject = sprintf($this->options->moderateEmailSubject, $type);
        $status = $this->options->autoModerateCommentType;
        $postTitle = get_the_title($comment->comment_post_ID);
        $postName = get_the_permalink($comment->comment_post_ID);
        $userLogin = get_comment_author($comment->comment_ID);
        $userEmail = get_comment_author_email($comment->comment_ID);
        $commentContent = get_comment_text($comment->comment_ID);
        $from_name = apply_filters('wp_mail_from_name', get_option('blogname'));
        $from_email = apply_filters('wp_mail_from', $this->getFromEmail(get_site_url()));
        $headers[] = "From: " . $from_name . " <" . $from_email . "> ";

        $body = strtr($this->options->reportedMailMessage, array(
            '[status]' => $status,
            '[postTitle]' => '"' . $postTitle . '"',
            '[postName]' => '<a href="' . $postName . ' target="_blank">' . $postName . '</a>',
            '[userLogin]' => $userLogin,
            '[commentContent]' => $commentContent,
            '[userEmail]' => $userEmail));
        if ($this->sendEmail($this->options->flaggedMailTo, $subject, $body, $headers)) {
            return true;
        }
    }

    public function sendEmail($to, $subject, $body, $headers) {
        $content_type = apply_filters('wp_mail_content_type', 'text/html');
        $headers[] = "Content-Type:  $content_type; charset=UTF-8";
        return wp_mail($to, $subject, $body, $headers);
    }

    /**
     * return client real ip
     */
    public static function getRealIPAddr() {
        if (isset($_SERVER['HTTP_CLIENT_IP'])) {   //check ip from share internet
            $ip = $_SERVER['HTTP_CLIENT_IP'];
        } elseif (isset($_SERVER['HTTP_X_FORWARDED_FOR'])) {   //to check ip is pass from proxy
            $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
        } else {
            $ip = $_SERVER['REMOTE_ADDR'];
        }

        if ($ip == '::1') {
            $ip = '127.0.0.1';
        }
        return $ip;
    }

    private function getFromEmail($siteUrl) {
        $parsedUrl = parse_url($siteUrl);
        $domain = isset($parsedUrl['host']) ? $parsedUrl['host'] : '';
        $fromEmail = 'no-reply@' . $domain;
        return apply_filters('wp_mail_from', $fromEmail);
    }

    public function resetOptions() {
        if (current_user_can('manage_options') && isset($_GET['_wpnonce']) && wp_verify_nonce($_GET['_wpnonce'], 'resetOptions')) {
            delete_option('wpdiscuz_fc');
            $this->options->addOptions();
            $this->options->initOptions();
        }
        wp_redirect(admin_url('edit-comments.php?page=' . WpDiscuzCore::PAGE_SETTINGS));
    }

}
