<?php

if (!defined('ABSPATH')) {
    exit();
}

class wpDiscuzFlagHelperAjax {

    private $dbManager;
    private $options;

    public function __construct($dbManager, $options) {
        $this->dbManager = $dbManager;
        $this->options = $options;

        add_action('wp_ajax_nopriv_fcReport', array($this, 'fcReport'));
        add_action('wp_ajax_fcReport', array($this, 'fcReport'));
        add_action('wp_ajax_nopriv_fcReportEmail', array($this, 'fcReportEmail'));
        add_action('wp_ajax_fcReportEmail', array($this, 'fcReportEmail'));
    }

    public function fcReport() {
        $response = array('code' => 0);
        if (isset($_POST['fcAjaxData'])) {
            parse_str($_POST['fcAjaxData'], $data);
            $commentId = (isset($data['commentId']) && ($cId = intval($data['commentId']))) ? $cId : false;
            $userIp = wpDiscuzFlagHelper::getRealIPAddr();
            if ($commentId && $userIp) {
                $currentUser = wp_get_current_user();
                $userId = (isset($currentUser->ID) && $currentUser->ID) ? $currentUser->ID : 0;
                $isFlagged = $this->dbManager->isFlagged($commentId, $userId, $userIp);
                if ($isFlagged) {
                    $response['message'] = $this->options->alreadyFlagged;
                } else {
                    $result = $this->dbManager->insertFlaggedData($commentId, $userId, $userIp);
                    if ($result) {
                        $response['code'] = 1;
                        $response['message'] = __('Flagged', 'wpdiscuz_fc');
                        do_action('wpdiscuz_comment_flagged', $commentId);
                    } else {
                        $response['message'] = $this->options->dataNotInserted;
                    }
                }
            }
        }
        wp_die(json_encode($response));
    }

    public function fcReportEmail() {
        $response = array('code' => 0, 'commentId' => 0);
        if (isset($_POST['fcAjaxData'])) {
            parse_str($_POST['fcAjaxData'], $data);
            $commentId = (isset($data['commentId']) && ($cId = intval($data['commentId']))) ? $cId : false;
            $report = (isset($data['wpdiscuz_report']) && ($r = trim($data['wpdiscuz_report']))) ? $r : '';
            $message = (isset($data['wpdiscuz_message']) && ($m = trim($data['wpdiscuz_message']))) ? $m : '';
            $userIp = wpDiscuzFlagHelper::getRealIPAddr();
            if ($commentId && $userIp) {
                $response['commentId'] = $commentId;
                $currentUser = wp_get_current_user();
                $userId = (isset($currentUser->ID) && $currentUser->ID) ? $currentUser->ID : 0;
                $isFlagged = $this->dbManager->isFlagged($commentId, $userId, $userIp);
                if ($isFlagged) {
                    $response['message'] = $this->options->alreadyFlagged;
                } else {
                    if (!$report) {
                        $response['message'] = $this->options->checkReportType;
                        wp_die(json_encode($response));
                    } else if ($report == 'other' && !$message) {
                        $response['message'] = $this->options->fillMsgField;
                        wp_die(json_encode($response));
                    } else {
                        $result = $this->dbManager->insertFlaggedData($commentId, $userId, $userIp);
                        if ($result) {
                            $response['code'] = 1;
                            $response['message'] = __('Flagged', 'wpdiscuz_fc');
                            do_action('wpdiscuz_comment_flagged_email', $commentId, $report, $message);
                        } else {
                            $response['message'] = $this->options->dataNotInserted;
                        }
                    }
                }
            }
        }
        wp_die(json_encode($response));
    }

}
