<?php

if (!defined("ABSPATH")) {
    exit();
}

class wpDiscuzFCommentOption {

    public $optionSlug = 'wpdiscuz_fc';
    public $options;    
    public $voteCount;
    public $flagCount;
    public $guestToFlag;
    public $sendFlagMessage;
    public $sendFlagMessageGuest;
    public $guestEmail;
    public $autoModerateComment;
    public $autoModerateCommentType;
    public $notifyAdmin;
    public $showFlag;
    public $votedMailSubject;
    public $votedMailMessage;
    public $flaggedMailTo;
    public $flagedMailMessage;
    public $reportedMailMessage;
    public $flagedMailSubject;
    public $reportPopupTitle;
    public $optionType;
    public $alreadyFlagged;
    public $emailNotSend;
    public $dataNotInserted;
    public $emailSend;
    public $sendBtn;
    public $messageField;
    public $flagTitleOn;
    public $flagTitleOff;
    public $reportOther;
    public $moderateEmailSubject;
    public $checkReportType;
    public $fillMsgField;
    public $notifyWhenFlagged;

    public function __construct() {
        add_action('wpdiscuz_addon_tab_title', array($this, 'optionFlagName'), 3);
        add_action('wpdiscuz_addon_tab_content', array($this, 'optionFlagContent'), 3);
        add_action('wpdiscuz_save_options', array($this, 'saveOptions'));
        $this->addOptions();
        $this->initOptions();
    }

    public function optionFlagName() {
        echo '<li>' . __('Report &amp; Flagging', 'wpdiscuz_fc') . '</li>';
    }

    public function optionFlagContent() {
        include_once 'view/flag-view.php';
    }

    public function addOptions() {
        $fc_mail_subject = __('New comment report', 'wpdiscuz_fc');
        $fc_flagged_message = __("<h2>Report details:</h2> \n<p>Name: [userInfo]</p>\n<p>Reason: [reason]</p>\n<p>Message: [message]</p>\n<p>Post: [postTitle]</p>\n<p>Comment URL: [commentInfo]</p>", 'wpdiscuz_fc');
        $fc_voted_message = __("You have a new [status] comment on the post [postTitle].\n<p>[postName]</p>\n<p><b>Comment details:</b></p>\n<p>Author: [userLogin]</p>\n<p>Email: [userEmail]</p>\n<p>Comment:<p>[commentContent]</p></p>", 'wpdiscuz_fc');
        $adminEmail = get_option('admin_email');
        $options = array(
            'voteCount' => 10,
            'flagCount' => 5,
            'guestToFlag' => 1,
            'sendFlagMessage' => 1,
            'sendFlagMessageGuest' => 0,
            'guestEmail' => __('Your Email', 'wpdiscuz_fc'),
            'autoModerateComment' => 1,
            'showFlag' => 1,
            'autoModerateCommentType' => 'unapprove',
            'notifyAdmin' => 1,
            'notifyWhenFlagged' => 1,
            'flaggedMailTo' => $adminEmail,
            'flagedMailSubject' => $fc_mail_subject,
            'flagedMailMessage' => $fc_flagged_message,
            'reportedMailMessage' => $fc_voted_message,
            'sendBtn' => __('Send', 'wpdiscuz_fc'),
            'reportPopupTitle' => __('Report this comment', 'wpdiscuz_fc'),
            'alreadyFlagged' => __('Already flagged', 'wpdiscuz_fc'),
            'emailNotSend' => __('Message sending problem', 'wpdiscuz_fc'),
            'dataNotInserted' => __('Data are not inserted', 'wpdiscuz_fc'),
            'emailSend' => __('Message sent', 'wpdiscuz_fc'),
            'reportOther' => __('Other', 'wpdiscuz_fc'),
            'messageField' => __('Enter message', 'wpdiscuz_fc'),
            'moderateEmailSubject' => __('New comment has reached to the maximum number of %s', 'wpdiscuz_fc'),
            'flagTitleOn' => __('Click to flag and open &laquo;Comment Reporting&raquo; form. You can choose reporting category and send message to website administrator. Admins may or may not choose to remove the comment or block the author. And please don\'t worry, your report will be anonymous.', 'wpdiscuz_fc'),
            'flagTitleOff' => __('You can flag a comment by clicking its flag icon. Website admin will know that you reported it. Admins may or may not choose to remove the comment or block the author. And please don\'t worry, your report will be anonymous.', 'wpdiscuz_fc'),
            'checkReportType' => __('Select bad comment category', 'wpdiscuz_fc'),
            'fillMsgField' => __('Please Insert message', 'wpdiscuz_fc'),            
            'optionType' => array(
                __('This comment is spam', 'wpdiscuz_fc'),
                __('This comment should be marked mature', 'wpdiscuz_fc'),
                __('This comment is abusive', 'wpdiscuz_fc'),
                __('This comment promotes self-harm', 'wpdiscuz_fc')
            )
        );
        add_option($this->optionSlug, $options);
        $this->options = $options;
    }

    public function margeOption() {
        $old_options = get_option($this->optionSlug);
        $mix_opt = array_merge($this->options, $old_options);
        update_option($this->optionSlug, $mix_opt);
    }

    public function initOptions() {
        $options = get_option($this->optionSlug);
        $this->voteCount = $options['voteCount'];
        $this->flagCount = $options['flagCount'];
        $this->guestToFlag = $options['guestToFlag'];
        $this->sendFlagMessage = $options['sendFlagMessage'];
        $this->sendFlagMessageGuest = $options['sendFlagMessageGuest'];
        $this->guestEmail = $options['guestEmail'];
        $this->autoModerateComment = $options['autoModerateComment'];
        $this->autoModerateCommentType = $options['autoModerateCommentType'];
        $this->notifyAdmin = $options['notifyAdmin'];
        $this->notifyWhenFlagged = isset($options['notifyWhenFlagged']) ? $options['notifyWhenFlagged'] : 1;
        $this->showFlag = $options['showFlag'];
        $this->flaggedMailTo = $options['flaggedMailTo'];
        $this->flagedMailMessage = wp_unslash($options['flagedMailMessage']);
        $this->flagedMailSubject = wp_unslash($options['flagedMailSubject']);
        $this->reportedMailMessage = wp_unslash($options['reportedMailMessage']);
        $this->reportPopupTitle = wp_unslash($options['reportPopupTitle']);
        $this->sendBtn = wp_unslash($options['sendBtn']);
        $this->optionType = wp_unslash($options['optionType']);
        $this->alreadyFlagged = wp_unslash($options['alreadyFlagged']);
        $this->emailNotSend = wp_unslash($options['emailNotSend']);
        $this->dataNotInserted = wp_unslash($options['dataNotInserted']);
        $this->emailSend = wp_unslash($options['emailSend']);
        $this->reportOther = wp_unslash($options['reportOther']);
        $this->messageField = wp_unslash($options['messageField']);
        $this->flagTitleOn = wp_unslash($options['flagTitleOn']);
        $this->flagTitleOff = wp_unslash($options['flagTitleOff']);
        $this->moderateEmailSubject = wp_unslash($options['moderateEmailSubject']);
        $this->checkReportType = wp_unslash($options['checkReportType']);
        $this->fillMsgField = wp_unslash($options['fillMsgField']);
    }

    public function saveOptions($args) {
        $this->guestToFlag = isset($args['wpdiscuz_allow_guest_flag']) ? intval($args['wpdiscuz_allow_guest_flag']) : 0;
        $this->sendFlagMessage = isset($args['wpdiscuz_allow_user_messag_flag']) ? intval($args['wpdiscuz_allow_user_messag_flag']) : 0;
        $this->sendFlagMessageGuest = isset($args['wpdiscuz_allow_messag_flag_guest']) ? intval($args['wpdiscuz_allow_messag_flag_guest']) : 0;        
        $this->autoModerateComment = isset($args['wpdiscuz_allow_auto_moderate']) ? intval($args['wpdiscuz_allow_auto_moderate']) : 0;
        $this->autoModerateCommentType = isset($args['wpdiscuz_allow_auto_moderation_type']) ? trim($args['wpdiscuz_allow_auto_moderation_type']) : 'unapprove';
        $this->flagCount = isset($args['wpdiscuz_flag_count']) ? intval(absint($args['wpdiscuz_flag_count'])) : 10;
        $this->voteCount = isset($args['wpdiscuz_vote_count']) ? intval(absint($args['wpdiscuz_vote_count'])) : 10;
        $this->notifyAdmin = isset($args['wpdiscuz_notify_admin']) ? intval($args['wpdiscuz_notify_admin']) : 0;
        $this->notifyWhenFlagged = isset($args['notifyWhenFlagged']) ? intval($args['notifyWhenFlagged']) : 0;
        $this->showFlag = isset($args['wpdiscuz_show_flag']) ? intval($args['wpdiscuz_show_flag']) : 0;
        $this->flagedMailSubject = isset($args['wpdiscuz_flagged_email_subject']) ? wp_unslash($args['wpdiscuz_flagged_email_subject']) : __('New comment report', 'wpdiscuz_fc');
        $this->flaggedMailTo = isset($args['wpdiscuz_admin_email']) ? ($args['wpdiscuz_admin_email']) : get_option('admin_email');
        $this->reportedMailMessage = isset($args['wpdiscuz_reported_email_message']) ? wp_unslash($args['wpdiscuz_reported_email_message']) : __('You have a new [status] comment on the post [postTitle].<br>\n[postName]<br>\n<b>Comment details:</b><br>\n Author: [userLogin]<br>\n Email: [userEmail]<br>\n URL:<br>\n Comment:\n [commentContent]', 'wpdiscuz_fc');
        $this->flagedMailMessage = isset($args['wpdiscuz_flagged_email_message']) ? wp_unslash($args['wpdiscuz_flagged_email_message']) : __('<h2>Report details:</h2>\n<p>Name: [userInfo]</p>\n<p>Reason: [reason],</p>\n<p>Message: [message],</p>\n<p>Post: [postTitle],</p>\n<p>Comment URL | Text: [commentInfo]</p>', 'wpdiscuz_fc');
        $this->optionType = isset($args['wpdiscuz_report_types']) ? wp_unslash($args['wpdiscuz_report_types']) : array(__('This comment is spam', 'wpdiscuz_fc'), __('This comment should be marked mature', 'wpdiscuz_fc'), __('This comment is abusive', 'wpdiscuz_fc'), __('This comment promotes self-harm', 'wpdiscuz_fc'));
        $this->reportPopupTitle = isset($args['wpdiscuz_report_title']) ? wp_unslash($args['wpdiscuz_report_title']) : __('Report this comment', 'wpdiscuz_fc');
        $this->sendBtn = isset($args['wpdiscuz_report_button']) ? wp_unslash($args['wpdiscuz_report_button']) : __('Send', 'wpdiscuz_fc');
        $this->alreadyFlagged = isset($args['wpdiscuz_already_flagged']) ? wp_unslash($args['wpdiscuz_already_flagged']) : __('Already flagged', 'wpdiscuz_fc');
        $this->emailNotSend = isset($args['wpdiscuz_email_not_send']) ? wp_unslash($args['wpdiscuz_email_not_send']) : __('Message sending problem', 'wpdiscuz_fc');
        $this->dataNotInserted = isset($args['wpdiscuz_data_not_inserted']) ? wp_unslash($args['wpdiscuz_data_not_inserted']) : __('Data are not inserted', 'wpdiscuz_fc');
        $this->emailSend = isset($args['wpdiscuz_email_send']) ? wp_unslash($args['wpdiscuz_email_send']) : __('Message sent', 'wpdiscuz_fc');
        $this->checkReportType = isset($args['wpdiscuz_check_report_type']) ? wp_unslash($args['wpdiscuz_check_report_type']) : __('Select bad comment category', 'wpdiscuz_fc');
        $this->fillMsgField = isset($args['wpdiscuz_fill_msg_field']) ? wp_unslash($args['wpdiscuz_fill_msg_field']) : __('Please Insert message', 'wpdiscuz_fc');
        $this->reportOther = isset($args['wpdiscuz_report_other']) ? wp_unslash($args['wpdiscuz_report_other']) : __('Other', 'wpdiscuz_fc');
        $this->messageField = isset($args['wpdiscuz_enter_message']) ? wp_unslash($args['wpdiscuz_enter_message']) : __('Enter message', 'wpdiscuz_fc');
        $this->flagTitleOn = isset($args['wpdiscuz_flag_title_on']) ? wp_unslash($args['wpdiscuz_flag_title_on']) : __('Click to flag and open &laquo;Comment Reporting&raquo; form. You can choose reporting category and send message to website administrator. Admins may or may not choose to remove the comment or block the author. And please don\'t worry, your report will be anonymous.', 'wpdiscuz_fc');
        $this->flagTitleOff = isset($args['wpdiscuz_flag_title_off']) ? wp_unslash($args['wpdiscuz_flag_title_off']) : __('You can flag a comment by clicking its flag icon. Website admin will know that you reported it. Admins may or may not choose to remove the comment or block the author. And please don\'t worry, your report will be anonymous.', 'wpdiscuz_fc');
        $this->moderateEmailSubject = isset($args['wpdiscuz_moderated_email_subject']) ? wp_unslash($args['wpdiscuz_moderated_email_subject']) : __('New comment has reached to the maximum number of %s', 'wpdiscuz_fc');        


        update_option($this->optionSlug, array('guestToFlag' => $this->guestToFlag,
            'sendFlagMessage' => $this->sendFlagMessage,
            'sendFlagMessageGuest' => $this->sendFlagMessageGuest,
            'guestEmail' => $this->guestEmail,
            'autoModerateComment' => $this->autoModerateComment,
            'autoModerateCommentType' => $this->autoModerateCommentType,
            'flagCount' => $this->flagCount,
            'voteCount' => $this->voteCount,
            'showFlag' => $this->showFlag,
            'notifyAdmin' => $this->notifyAdmin,
            'notifyWhenFlagged' => $this->notifyWhenFlagged,
            'flagedMailMessage' => $this->flagedMailMessage,
            'reportedMailMessage' => $this->reportedMailMessage,
            'flagedMailSubject' => $this->flagedMailSubject,
            'flaggedMailTo' => $this->flaggedMailTo,
            'optionType' => $this->optionType,
            'sendBtn' => $this->sendBtn,
            'reportOther' => $this->reportOther,
            'messageField' => $this->messageField,
            'flagTitleOn' => $this->flagTitleOn,
            'flagTitleOff' => $this->flagTitleOff,
            'moderateEmailSubject' => $this->moderateEmailSubject,
            'reportPopupTitle' => $this->reportPopupTitle,
            'emailNotSend' => $this->emailNotSend,
            'alreadyFlagged' => $this->alreadyFlagged,
            'dataNotInserted' => $this->dataNotInserted,
            'emailSend' => $this->emailSend,
            'checkReportType' => $this->checkReportType,
            'fillMsgField' => $this->fillMsgField,
        ));
    }

}
