<?php
/*
 * Plugin Name: wpDiscuz - Widgets
 * Description: Plugin for displaying the most popular and the newest comments
 * Version: 1.3.1
 * Author: gVectors Team (Mkrtich Aleksanyan and Contributors)
 * Contributors: A. Chakhoyan, G. Zakaryan, H. Martirosyan
 * Author URI: http://gvectors.com/
 * Plugin URI: http://gvectors.com/wpdiscuz-widgets/
 * Text Domain: wpdiscuz-widgets
 * Domain Path: /languages/
 */

include_once 'includes/gvt-api-manager.php';
include_once 'wpdiscuz-widgets-class.php';

class wpDiscuzWidgets {

    public $widgetOptionsName = "widget_options_array";
    public $options;
    private $version;

    /**
     * 
     */
    public function __construct() {
        add_action('plugins_loaded', array(&$this, 'pluginsLoaded'), 1894.5);
    }

    public function pluginsLoaded() {
        if (function_exists('wpDiscuz')) {
            new GVT_API_Manager(__FILE__, 'wpdiscuz_options_page', 'wpdiscuz_option_page');
            load_plugin_textdomain("wpdiscuz-widgets", false, dirname(plugin_basename(__FILE__)) . '/languages/');
            $this->version = get_option("widgetPluginVersion");
            if (!$this->version) {
                $this->version = '1.0.0';
            }
            $this->addOptions();
            $this->initOptions();
            
            add_action('wpdiscuz_check_version', array(&$this, 'widgetPluginVersion'));

            add_action("widgets_init", array(&$this, "registerWpdiscuzWidget"), 13);
            add_filter("admin_enqueue_scripts", array(&$this, "wpdiscuz_widgets_comment_vot_widg_script"), 150);

            add_action('wpdiscuz_addon_tab_title', array($this, 'widgetOptionTabName'), 13);
            add_action('wpdiscuz_addon_tab_content', array($this, 'widgetOptionTabContent'), 13);
            add_action('wpdiscuz_save_options', array($this, 'widgetSaveOptions'), 13);

            add_action('wp_head', array(&$this, 'initDynamicCss'));
        }else{
            add_action('admin_notices', array($this, 'wpDiscuzWidgets_requirements'));
        }
    }

    public function wpDiscuzWidgets_requirements() {
        if( current_user_can('manage_options') ){
            echo "<div class='error'><p>" . __('wpDiscuz Widgets requires wpDiscuz to be installed!', 'wpdiscuz-widgets') . "</p></div>";
        }
    }

    /*
     * checking a version
     */

    public function widgetPluginVersion() {
        $pluginData = get_plugin_data(__FILE__);
        if (version_compare($pluginData['Version'], $this->version, '>')) {
            update_option("widgetPluginVersion", $pluginData['Version']);
            $old_options = maybe_unserialize(get_option($this->widgetOptionsName));
            if (count($this->widgetDefaultOptions()) > count(maybe_unserialize(get_option($this->widgetOptionsName)))) {
                $result_options = array_merge($this->widgetDefaultOptions(), maybe_unserialize(get_option($this->widgetOptionsName)));
                update_option($this->widgetOptionsName, $result_options);
            }
        } else {
            update_option("widgetPluginVersion", $this->version);
        }
    }

    /**
     * Dynamic css which will be written in the head of document
     * using the values of fields from addons settings
     */
    public function initDynamicCss() {
        ?>
        <style type='text/css'>.like-count-first.wpdiscuz-widget-comment .wpdiscuz-widget-icon-show-box{background:<?php echo $this->options['wc_icons_background_color'] ?>;}#widget-comments-container .wpdiscuz-widgets-tab-title-list li .fas{color:<?php echo $this->options['wc_icons_color'] ?>;}div#widget-comments-container.widget-comments-container{max-width:<?php echo $this->options['wpdiscuz_widget_max_width']; ?>px;margin-left:<?php echo $this->options['wpdiscuz_widget_lmargin']; ?>px;margin-right:<?php echo $this->options['wpdiscuz_widget_rmargin']; ?>px;}div#widget-comments-container div.wpdiscuz-widgets-content{padding-right:<?php echo $this->options['wpdiscuz_widget_rpadding']; ?>px;padding-left:<?php echo $this->options['wpdiscuz_widget_lpadding']; ?>px;}div.widget-comments-container>div,div.widget-comments-container>ul>.tab-title-list-active-item{background-color:<?php echo $this->options['wc_widget_background_color']; ?>;}#widget-comments-container ul.wpdiscuz-widgets-tab-title-list li.tab-title-list-active-item{border-top-color:<?php echo $this->options['wc_icons_color']; ?>}#widget-comments-container .wpd_widgets_items_wrapper .slick-arrow{background:<?php echo $this->options['wc_icons_background_color'] ?>;} <?php echo $this->options['wpdiscuz_widget_custom_css'] ?></style>
        <?php
    }

    /**
     * Register widgets with widgets classnames
     */
    public function registerWpdiscuzWidget() {
        register_widget("wpDiscuzWidgetsClass");
    }

    /**
     * add widgts options tab name in wpDiscuz options addons 
     */
    public function widgetOptionTabName() {
        echo "<li>" . __('Widgets', 'wpdiscuz-widgets') . "</li>";
    }

    /**
     * Connecting css and js files to admin page
     */
    public function wpdiscuz_widgets_comment_vot_widg_script() {
        global $pagenow, $wp_styles;
        if ($pagenow == 'widgets.php' || $pagenow == 'edit-comments.php') {
            $wpdiscuz = wpDiscuz();
            wp_enqueue_style('jquery-ui-datepicker-style', plugins_url("/assets/third-party/jquery-ui.min.css", __FILE__), null, $this->version);
            wp_enqueue_style('jquery-ui-style');
            wp_enqueue_script('jquery-ui-datepicker');

            wp_register_script("wpdiscuz_widgets_comm_vot_admin_srcipt", plugins_url("/assets/js/admin-js.js", __FILE__), array('jquery'), $this->version, $wpdiscuz->optionsSerialized->isLoadScriptsInFooter);
            wp_enqueue_script("wpdiscuz_widgets_comm_vot_admin_srcipt");

            wp_enqueue_style('wpdiscuz-font-awesome');
            wp_register_style("wpdiscuz_widgets_comm_vot_admin_style", plugins_url("/assets/css/admin-style.css", __FILE__), null, $this->version);
            wp_enqueue_style("wpdiscuz_widgets_comm_vot_admin_style");
        }
    }

    public function widgetOptionTabContent() {
        $check = "wpdiscuz-widget-votes-style-label";
        $wpdiscuz_widget_displaying_guests = $this->options['wpdiscuz_widget_displaying_guests'];
        $wpdiscuz_widget_author_link = $this->options['wpdiscuz_widget_author_link'];
        $wpdiscuz_widget_slider_enable = $this->options['wpdiscuz_widget_slider_enable'];
        $wpdiscuz_widget_theme_title_struct = $this->options['wpdiscuz_widget_theme_title_struct'];
        $wpdiscuz_widget_post_title_cutting = isset($this->options['wpdiscuz_widget_post_title_cutting']) ? $this->options['wpdiscuz_widget_post_title_cutting'] : 1;
        $wpdiscuz_widgets_post_title_word_count = isset($this->options['wpdiscuz_widgets_post_title_word_count']) ? $this->options['wpdiscuz_widgets_post_title_word_count'] : 5;
        $wpdiscuz_widget_content_cutting = isset($this->options['wpdiscuz_widget_content_cutting']) ? $this->options['wpdiscuz_widget_content_cutting'] : 1;
        $wpdiscuz_widgets_post_content_word_count = isset($this->options['wpdiscuz_widgets_post_content_word_count']) ? $this->options['wpdiscuz_widgets_post_content_word_count'] : 10;
        $excluded_user_roles = isset($this->options['excluded_user_roles']) ? $this->options['excluded_user_roles'] : array();
        /**
         * Reset widget options
         */
        if (isset($_GET['wpd_widget_reset_options']) && $_GET['wpd_widget_reset_options'] == 1 && current_user_can('manage_options')) {
            $options = $this->widgetDefaultOptions();
            update_option($this->widgetOptionsName, $options);
            $this->options = $options;
        }
        include "options/addons-settings-html.php";
    }

    private function initOptions() {
        $this->options = get_option($this->widgetOptionsName);
    }

    private function addOptions() {
        $options = $this->widgetDefaultOptions();
        add_option($this->widgetOptionsName, $options);
    }

    /**
     * update widget settings from WpDiscuz settings
     * @param array $args
     */
    public function widgetSaveOptions($args) {
        $this->options['wpdiscuz_widget_post_title_cutting'] = isset($args['wpdiscuz_widget_post_title_cutting']) ? intval($args['wpdiscuz_widget_post_title_cutting']) : 'on';
        $this->options['wpdiscuz_widgets_post_title_word_count'] = isset($args['wpdiscuz_widgets_post_title_word_count']) ? intval($args['wpdiscuz_widgets_post_title_word_count']) : 5;
        $this->options['wpdiscuz_widget_content_cutting'] = isset($args['wpdiscuz_widget_content_cutting']) ? intval($args['wpdiscuz_widget_content_cutting']) : 'on';
        $this->options['wpdiscuz_widgets_post_content_word_count'] = isset($args['wpdiscuz_widgets_post_content_word_count']) ? intval($args['wpdiscuz_widgets_post_content_word_count']) : 10;
        $this->options['mvc_displaying_style'] = isset($args['mvc_displaying_style']) ? $args['mvc_displaying_style'] : "votes_style";
        $this->options['wc_widget_background_color'] = isset($args['wc_widget_background_color']) ? trim($args['wc_widget_background_color']) : "rgba(255, 255, 255, 0)";
        $this->options['wc_icons_background_color'] = isset($args['wc_icons_background_color']) ? trim($args['wc_icons_background_color']) : "#20252B";
        $this->options['wpdiscuz_widget_icon_circle'] = isset($args['wpdiscuz_widget_icon_circle']) ? intval($args['wpdiscuz_widget_icon_circle']) : 'on';
        $this->options['wc_icons_color'] = isset($args['wc_icons_color']) ? trim($args['wc_icons_color']) : "#20252B";
        $this->options['wpdiscuz_widget_max_width'] = isset($args['wpdiscuz_widget_max_width']) ? intval($args['wpdiscuz_widget_max_width']) : 350;
        $this->options['wpdiscuz_widget_lmargin'] = isset($args['wpdiscuz_widget_lmargin']) ? intval($args['wpdiscuz_widget_lmargin']) : 10;
        $this->options['wpdiscuz_widget_rmargin'] = isset($args['wpdiscuz_widget_rmargin']) ? intval($args['wpdiscuz_widget_rmargin']) : 10;
        $this->options['wpdiscuz_widget_lpadding'] = isset($args['wpdiscuz_widget_lpadding']) ? intval($args['wpdiscuz_widget_lpadding']) : 10;
        $this->options['wpdiscuz_widget_rpadding'] = isset($args['wpdiscuz_widget_rpadding']) ? intval($args['wpdiscuz_widget_rpadding']) : 10;
        $this->options['wpdiscuz_widget_displaying_guests'] = isset($args['wpdiscuz_widget_displaying_guests']) ? intval($args['wpdiscuz_widget_displaying_guests']) : 'on';
        $this->options['excluded_user_roles'] = isset($args['excluded_user_roles']) ? $args['excluded_user_roles'] : array();
        $this->options['wpdiscuz_widget_author_link'] = isset($args['wpdiscuz_widget_author_link']) ? intval($args['wpdiscuz_widget_author_link']) : 'on';
        $this->options['wpdiscuz_widget_slider_enable'] = isset($args['wpdiscuz_widget_slider_enable']) ? intval($args['wpdiscuz_widget_slider_enable']) : 'on';
        $this->options['wpdiscuz_widget_theme_title_struct'] = isset($args['wpdiscuz_widget_theme_title_struct']) ? intval($args['wpdiscuz_widget_theme_title_struct']) : 'on';
        $this->options['wpdiscuz_widget_custom_css'] = isset($args['wpdiscuz_widget_custom_css']) ? $args['wpdiscuz_widget_custom_css'] : "#widget-comments-container{font-size:100%}";

        update_option($this->widgetOptionsName, $this->options);
    }

    /**
     * 
     * @return array
     */
    public function widgetDefaultOptions() {
        $def_opt = array(
            'wpdiscuz_widget_post_title_cutting' => 'on',
            'wpdiscuz_widgets_post_title_word_count' => 5,
            'wpdiscuz_widget_content_cutting' => 'on',
            'wpdiscuz_widgets_post_content_word_count' => 10,
            'mvc_displaying_style' => 'votes_style',
            'wc_widget_background_color' => 'rgba(255, 255, 255, 0)',
            'wc_icons_background_color' => '#20252B',
            'wpdiscuz_widget_icon_circle' => 'on',
            'wc_icons_color' => '#20252B',
            'wpdiscuz_widget_max_width' => 550,
            'wpdiscuz_widget_lmargin' => 10,
            'wpdiscuz_widget_rmargin' => 10,
            'wpdiscuz_widget_lpadding' => 10,
            'wpdiscuz_widget_rpadding' => 10,
            'wpdiscuz_widget_displaying_guests' => 'on',
            'excluded_user_roles' => array(),
            'wpdiscuz_widget_author_link' => 'on',
            'wpdiscuz_widget_slider_enable' => 'on',
            'wpdiscuz_widget_theme_title_struct' => 1,
            'wpdiscuz_widget_custom_css' => "#widget-comments-container{font-size:100%}",
        );
        return $def_opt;
    }

}

$wpdiscuz_widgets = new wpDiscuzWidgets();
